unit RakDataObject;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit RakPersistent ersetzt die Unit RakDatenObj
// erstellt am  : 15.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    17.3.2002 Neue Methode Assign
    17.3.2002 Die Klassen fr den Undo/Redo-Mechanismus umbenannt
    22.3.2002 Bugfix: In den meisten CreateNow-Methoden hat der inherited-Aufruf gefehlt
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  graphics, sysutils, classes,
  WYSIWYGFont, RakBinaryStreamData,
  misctype, miscfunc, RakObject, RakCollection, RakUndoRedo,
  RakGlobalObjectId;

type
  TRakStateDataChanging = class(TRakBaseStatus);

  TRakDataObjectClass = class of TRakDataObject;
  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakDataObject
  // Beschreibung : TRakDataObject ist die Basisklasse aller Klassen, die
                    persistente Daten halten und diese in Form eines
                    Binarytags exportieren und importieren knnen.
                    Weiterhin untersttzt die Klasse den Undo/Redo-Mechanismus
                    der Unit RakUndoRedo. Auerdem trgt sich die Klasse beim
                    globalen Objektverwalter aus der Unit RakGlobalObjectID ein.
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
    24.1.2002 Neue Methoden CanCreateDestroyCommand, SaveStateInHistory und SendDestroyCommand
    17.3.2002 Neue Methode Assign
  ////////////////////////////////////////////////////////////////////////////////}
  TRakDataObject = class(TRakObject)
  private
    FHistory: TRakHistory;
    function GetGlobalId: string;
  protected
    FOwner: TObject;

    // TRakObject
    procedure CreateNow; override;

    procedure BeforeDestroyEvent; override;
    procedure AfterDestroyEvent; override;

    procedure FireDestroyingEvent; virtual;

    // Die folgenden Methoden untersttzen die globale ObjektId
    // RegisterAtGlobalObjectmanager wird beim erzeugen des Objektes aufgerufen
    procedure RegisterAtGlobalObjectmanager; virtual;
    // UnregisterAtGlobalObjectmanager wird beim zerstren des Objektes aufgerufen
    procedure UnregisterAtGlobalObjectmanager; virtual;
    // GlobalIdConflict wird beim Laden aufgerufen, wenn die gewnschte ID bereits vergeben ist
    procedure GlobalIdConflict(WantedId, FunctionName: string); virtual;

    // Die folgenden Methoden untersttzen den Undo/Redo-Mechanismus

    // read-Methode
    function GetHistory: TRakHistory; virtual;
    // write-Methode
    procedure SetHistory(const Value: TRakHistory); virtual;
    // Mit AddCommand kann ein weiteres Command in der History
    // abgelegt werden. AddCommand prft, ob berhaupt eine History
    // verfgbar ist und ob sich die Instanz nicht gerade erzeugt. Falls ja,
    // dann wird bei dem Command der Parameter ChangedObjectName noch
    // mit GlobalId versorgt und in der History gespeichert.
    // Falls nein, wird Cmd freigegeben
    procedure AddCommand(Cmd: TRakCommand); virtual;
    // Mit NewGroupStart kann eine neue Gruppierung in der History gestartet werden.
    // ACHTUNG: Jeder Aufruf von NewGroupStart MUSS mit einem Aufruf von NewGroupStop beendet werden
    procedure NewGroupStart(CmdGrp: TRakCommandGroup); virtual;
    // Mit NewGroupStop kann eine Gruppierung in der History beendet werden.
    procedure NewGroupStop; virtual;
    // CreateDestroyChangeData wird beim Zerstren des Objektes aufgerufen, bevor
    // Aufrumarbeiten durchgefhrt wurden. Standardmig wird eine
    // eine Instanz von TRakCommandGroup erzeugt. Abgeleitetee Klassen
    // knnen hier Instanzen von TRakCommandGroup abgeleiteten Klassen
    // zurckgeben, falls dies fr ein Korrektes Undo/Redo bentigt wird
    function CreateDestroyChangeData: TRakCommandGroup; virtual;
    // CanCreateDestroyCommand bestimmt, ob SaveStateInHistory und SendDestroyCommand aufgerufen werden
    // Die Defaultrckgabe ist false. Abgeleitete Klassen knnen die Methode
    // berschreiben und true zurchgeben, wenn das Wiederherstellen ber
    // den Undo/Redo-Mechanismus automatisch untersttzt werden soll.
    function CanCreateDestroyCommand: boolean; virtual;
    // SaveStateInHistory wird beim Zerstren des Objektes nach
    // CreateDestroyChangeData aufgerufen, bevor Aufrumarbeiten durchgefhrt
    // wurden. Hier knnen abgeleitete Klassen Daten in der History speichern,
    // die sie zum korrekten wiederherstellen bentigen. (s.a. CanCreateDestroyCommand)
    procedure SaveStateInHistory(History: TRakHistory); virtual;
    // SendDestroyCommand wird beim Zerstren des Objektes aufgerufen, nachdem
    // die Aufrumarbeiten durchgefhrt wurden. Hier wird ein Command in der
    // History gespeichert, die dafr sorgt, dass bei einem Undo das Objekt
    // wieder erzeugt wird und die gleiche GlobalId hat wie vor dem Zerstren
    // (s.a. CanCreateDestroyCommand)
    procedure SendDestroyCommand(History: TRakHistory); virtual;

    //read-Methode
    function GetOwner: TObject; virtual;
    //write-Methode
    procedure SetOwner(Value: TObject); virtual;

    // Ermglicht abgeleiteten Klassen das ndern des Owners zu beeinflussen
    procedure OwnerChangeBefore(var NewOwner: TObject); virtual;
    // Ermglicht abgeleiteten Klassen auf einen genderten Owner zu reagieren
    procedure OwnerChangeAfter(OldOwner: TObject); virtual;

    // Der Eigentmer des Datenobjektes
    property Owner: TObject read GetOwner write SetOwner;
  public
    procedure Assign(Source: TRakDataObject); virtual;

    procedure SaveToBinaryTag(Tag: TRakBinaryTag); virtual;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); virtual;
    function GetAsBinaryTag: TRakBinaryTag;

    property History: TRakHistory read GetHistory write SetHistory;
    property GlobalId: string read GetGlobalId;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Typ          : TRakPenStyle
  // Beschreibung : TRakPenStyle soll die grteabhngige Darstellung
                    von TPenStyle beseitigen
  // erstellt am  : 17.3.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakPenStyle = (
    rpsSolid, // Durchgngige Linie
    rpsClear, // Durchsichtige Linie / Es wird keine Linie gezeichnet
    rpsLine, // gestrichelte Linie
    rpsDotLine); // gestrichelte Linie, wobei abwechselnd zwei Linienlngen verwendet werden

  TRakPenLinePosition = (
    plpLeft, // Es wird links von der Linie gezeichnet
    plpCenter, // Es wird mittig zur Linie gezeichnet
    plpRight); // Es wird rechts von der Linie gezeichnet}

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakPenDataObject
  // Beschreibung : TRakPenDataObject hlt die Daten eines Stiftes zum Zeichnen,
                    wobei die Breite des Stiftes nicht in Pixel, sondern in
                    Millimeter angegeben wird
                    Auer dem Binarytag-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakUndoRedo untersttzt
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
    7.3.2002: Neue Methode ChangeAfter
    17.3.2002 Untersttzt jetzt Assign
    17.3.2002 Neue Properties RakPenStyle, MmLineLength und MmDotLength
    17.3.2002 In SaveToBinaryTag werden nur noch die Werte gespeichert,
              die ungleich den Defaultwerten sind
    16.4.2002 Bugfix: SetPen um den Parameter PixelPerMm erweitert
  ////////////////////////////////////////////////////////////////////////////////}
  TRakPenDataObject = class(TRakDataObject)
  private
    FColor: TColor;
    FMode: TPenMode;
    FStyle: TPenStyle;
    FmmWidth: extended;
    //FRakStyle: TRakPenStyle;
    //FMmLineLength: extended;
    //FMmDotLength: extended;
    //FMmLineSpace: extended;
    //FLinePosition: TRakPenLinePosition;
    procedure SetColor(const Value: TColor);
    procedure SetMode(const Value: TPenMode);
    procedure SetStyle(const Value: TPenStyle);
    procedure SetMmWidth(const Value: extended);
    //procedure SetRakStyle(const Value: TRakPenStyle);
    //procedure SetMmDotLength(const Value: extended);
    //procedure SetMmLineLength(const Value: extended);
    //procedure SetMmLineSpace(const Value: extended);
    //procedure SetLinePosition(const Value: TRakPenLinePosition);
  protected
    procedure CreateNow; override;

    procedure ChangeAfter; virtual;

    procedure ColorChangeAfter(OldValue: TColor); virtual;
    procedure ModeChangeAfter(OldValue: TPenMode); virtual;
    procedure StyleChangeAfter(OldValue: TPenStyle); virtual;
    //procedure LinePositionChangeAfter(OldValue: TRakPenLinePosition); virtual;
    //procedure RakStyleChangeAfter(OldValue: TRakPenStyle); virtual;
    procedure MmWidthChangeAfter(OldValue: extended); virtual;
    //procedure MmDotLengthChangeAfter(OldValue: extended); virtual;
    //procedure MmLineLengthChangeAfter(OldValue: extended); virtual;
    //procedure MmLineSpaceChangeAfter(OldValue: extended); virtual;
  public
    procedure Assign(Source: TRakDataObject); override;

    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;

    procedure SetPen(APen: TPen; PixelPerMm: extended); virtual;

    property Mode: TPenMode read FMode write SetMode; // Default: pmCopy
    property Style: TPenStyle read FStyle write SetStyle; // Default: psSolid
    //property RakStyle: TRakPenStyle read FRakStyle write SetRakStyle; // Default: rpsSolid
    // MmLineLength wird nur verwendet, wenn RakStyle in [rpsLine, rpsDotLine]
    //property MmLineLength: extended read FMmLineLength write SetMmLineLength; // Default: 3
    // MmLineSpace wird nur verwendet, wenn RakStyle in [rpsLine, rpsDotLine]
    //property MmLineSpace: extended read FMmLineSpace write SetMmLineSpace; // Default: 3
    // MmDotLength wird nur verwendet, wenn RakStyle = rpsDotLine
    //property MmDotLength: extended read FMmDotLength write SetMmDotLength; // Default: 0.25
    property Color: TColor read FColor write SetColor; // Default: clBlack
    property MmWidth: extended read FMmWidth write SetMmWidth; // Default: 0.25
    //property LinePosition: TRakPenLinePosition read FLinePosition write SetLinePosition; // Default: plpCenter
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakBrushDataObject
  // Beschreibung : TRakBrushDataObject hlt die Daten eines Pinseld zum Fllen.
                    Auer dem Binarytag-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakUndoRedo untersttzt
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
    7.3.2002: Neue Methode ChangeAfter
    17.3.2002 Untersttzt jetzt Assign
  ////////////////////////////////////////////////////////////////////////////////}
  TRakBrushDataObject = class(TRakDataObject)
  private
    FStyle: TBrushStyle;
    FColor: TColor;
    procedure SetColor(const Value: TColor);
    procedure SetStyle(const Value: TBrushStyle);
  protected
    procedure CreateNow; override;

    procedure ChangeAfter; virtual;

    procedure ColorChangeAfter(OldValue: TColor); virtual;
    procedure StyleChangeAfter(OldValue: TBrushStyle); virtual;
  public
    procedure Assign(Source: TRakDataObject); override;

    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;

    procedure SetBrush(ABrush: TBrush); virtual;

    property Style: TBrushStyle read FStyle write SetStyle;
    property Color: TColor read FColor write SetColor;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakFontDataObject
  // Beschreibung : TRakFontDataObject hlt die Daten eines Fonts zum Schreiben,
                    wobei die Hehe des Fonts nicht in 72tel-Zoll sondern
                    in Millimeter angegeben wird
                    Auer dem Binarytag-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakUndoRedo untersttzt
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
    7.3.2002: Neue Methode ChangeAfter
    17.3.2002: Methode Assign wurde in die Klasse TRakDataObject verlegt
    16.4.2002 Bugfix: SetFont um den Parameter PixelPerMm erweitert
  ////////////////////////////////////////////////////////////////////////////////}
  TRakFontDataObject = class(TRakDataObject)
  private
    FMmHeight: extended;
    FName: string;
    FColor: TColor;
    FStyle: TFontStyles;
    procedure SetColor(const Value: TColor);
    procedure SetMmHeight(const Value: extended);
    procedure SetName(const Value: string);
    procedure SetStyle(const Value: TFontStyles);
  protected
    procedure CreateNow; override;

    procedure ChangeAfter; virtual;

    procedure ColorChangeAfter(OldValue: TColor); virtual;
    procedure MmHeightChangeAfter(OldValue: extended); virtual;
    procedure NameChangeAfter(OldValue: string); virtual;
    procedure StyleChangeAfter(OldValue: TFontStyles); virtual;
  public
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;

    procedure Assign(Source: TRakDataObject); override;

    function TextWidth(Text: string): extended; virtual;
    // SetFont setzt die eigenen Werte in AFont ein
    procedure SetFont(AFont: TFont; PixelPerMm: extended); virtual;
    // WrapText zerlegt Zeile so in einzelne Zeilen, dass keine Zeile
    // breiter ist als MmBreite, wobei in jeder Zeile mindestens ein Buchstabe
    // sein muss. Beim Umbrechen wird versucht so umzubrechen, dass der
    // Text noch mglichst gut lesbar ist
    procedure WrapText(Zeile: string; MmWidth: extended; Zeilen: TStrings);

    property Name: string read FName write SetName;
    property Style: TFontStyles read FStyle write SetStyle;
    property Color: TColor read FColor write SetColor;
    property MmHeight: extended read FMmHeight write SetMmHeight;
  end;


  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakMmBoundsDataObject
  // Beschreibung : TRakMmBoundsDataObject hlt die Millimeter-Koordinaten eines Rechteckes
                    Auer dem Binarytag-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakUndoRedo untersttzt
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakMmBoundsDataObject = class(TRakDataObject)
  private
    function GetMmRight: extended;
    function GetMmBottom: extended;
    procedure SetMmWidth(const Value: extended);
    procedure SetMmHeight(const Value: extended);
    procedure SetMmLeft(const Value: extended);
    procedure SetMmTop(const Value: extended);
    procedure SetMmRight(const Value: extended);
    procedure SetMmBottom(const Value: extended);
    function GetMmHorzCenter: extended;
    function GetMmVertCenter: extended;
    procedure SetMmHorzCenter(const Value: extended);
    procedure SetMmVertCenter(const Value: extended);
  protected
    FMmLeft: extended;
    FMmTop: extended;
    FMmWidth: extended;
    FMmHeight: extended;

    procedure SetMmNow(Left, Top, Width, Height: extended); virtual;
    procedure MmChangeBefore(var NewLeft, NewTop, NewWidth,
      NewHeight: extended); virtual;
    procedure MmChangeAfter(const OldLeft, OldTop, OldWidth,
      OldHeight: extended); virtual;
  public
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;

    function GetMmRect: TExtRect;
    procedure SetMm(Left, Top, Width, Height: extended); overload;

    property MmLeft: extended read FMmLeft write SetMmLeft;
    property MmTop: extended read FMmTop write SetMmTop;
    property MmWidth: extended read FMmWidth write SetMmWidth;
    property MmHeight: extended read FMmHeight write SetMmHeight;
    property MmRight: extended read GetMmRight write SetMmRight;
    property MmBottom: extended read GetMmBottom write SetMmBottom;
    property MmHorzCenter: extended read GetMmHorzCenter write SetMmHorzCenter;
    property MmVertCenter: extended read GetMmVertCenter write SetMmVertCenter;
  end;


  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcMmLeft
  // Beschreibung : TRakZzHcMmLeft wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcMmLeft = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcMmTop
  // Beschreibung : TRakZzHcMmTop wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcMmTop = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcMmWidth
  // Beschreibung : TRakZzHcMmWidth wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcMmWidth = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcMmHeight
  // Beschreibung : TRakZzHcMmHeight wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcMmHeight = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;


  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTPenMode
  // Beschreibung : TRakZzHcTPenMode wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcTPenMode = class(TRakCommand)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    Old: TPenMode;
    New: TPenMode;

    constructor Init(Old, New: TPenMode);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTPenStyle
  // Beschreibung : TRakZzHcTPenStyle wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcTPenStyle = class(TRakCommand)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    Old: TPenStyle;
    New: TPenStyle;

    constructor Init(Old, New: TPenStyle);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTPenLinePosition
  // Beschreibung : TRakZzHcTPenLinePosition wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 19.3.2002
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakZzHcTPenLinePosition = class(TRakCommand)
  protected
    FOld: TRakPenLinePosition;
    FNew: TRakPenLinePosition;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    constructor Init(Old, New: TRakPenLinePosition);
  end;}

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTRakPenStyle
  // Beschreibung : TRakZzHcTRakPenStyle wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 17.3.2002
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakZzHcTRakPenStyle = class(TRakCommand)
  protected
    FOld: TRakPenStyle;
    FNew: TRakPenStyle;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    constructor Init(Old, New: TRakPenStyle);
  end;}


  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcPenColor
  // Beschreibung : TRakZzHcPenColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcPenColor = class(TRakHcTColor)
  protected
    procedure SetValue(Obj: TObject; Value: TColor; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcPenMmLineLength
  // Beschreibung : TRakZzHcPenMmLineLength wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakZzHcPenMmLineLength = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;}

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcPenMmLineSpace
  // Beschreibung : TRakZzHcPenMmLineSpace wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakZzHcPenMmLineSpace = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;}

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcPenMmDotLength
  // Beschreibung : TRakZzHcPenMmDotLength wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 17.3.2002
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  {TRakZzHcPenMmDotLength = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;}

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcPenMmWidth
  // Beschreibung : TRakZzHcPenMmWidth wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 17.3.2002
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcPenMmWidth = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcBrushColor
  // Beschreibung : TRakZzHcBrushColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcBrushColor = class(TRakHcTColor)
  protected
    procedure SetValue(Obj: TObject; Value: TColor; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTBrushStyle
  // Beschreibung : TRakZzHcTBrushStyle wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcTBrushStyle = class(TRakCommand)
  protected
    FOldValue: TBrushStyle;
    FNewValue: TBrushStyle;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    constructor Init(OldValue, NewValue: TBrushStyle);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcFontColor
  // Beschreibung : TRakZzHcFontColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcFontColor = class(TRakHcTColor)
  protected
    procedure SetValue(Obj: TObject; Value: TColor; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcFontMmHeight
  // Beschreibung : TRakZzHcFontMmHeight wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcFontMmHeight = class(TRakHcExtended)
  protected
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcFontName
  // Beschreibung : TRakZzHcFontName wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcFontName = class(TRakHcString)
  protected
    procedure SetValue(Obj: TObject; Value: string; var Done: boolean); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcTFontStyles
  // Beschreibung : TRakZzHcTFontStyles wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakUndoRedo verwendet
  // erstellt am  : 15.11.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcTFontStyles = class(TRakCommand)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  public
    Old: TFontStyles;
    New: TFontStyles;

    constructor Init(Old, New: TFontStyles);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakZzHcDestroyDataObject
  // Beschreibung : TRakZzHcDestroyDataObject sorgt innerhalb des Redo/Undo-Mechanismus
                    dafr, dass das Objekt wieder erzeugt, bzw. zerstrt wird
                    und dass das Objekt die richtige GlobelId hat
  // erstellt am  : 24.1.2002
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakZzHcDestroyDataObject = class(TRakHcDestroyGlobalObject)
  protected
    FKlasse: TRakDataObjectClass;

    function CheckChangedObject(Obj: TObject): boolean; override;
    function CreateObject: TObject; override;
  public
    constructor Init(Obj: TRakDataObject);
  end;


implementation

{ TRakDataObject }

procedure TRakDataObject.AddCommand(Cmd: TRakCommand);
var
  History: TRakHistory;
begin
  if Stati.Exists(TRakStatusCreating) then
  begin
    Cmd.Free;
  end
  else
  begin
    History := GetHistory;
    if History = nil then
      Cmd.Free
    else
    begin
      Cmd.ChangedObjectName :=  GlobalId;
      History.AddCommand(Cmd);
    end;
  end;
end;

procedure TRakDataObject.AfterDestroyEvent;
begin
  inherited;

  if FHistory <> nil then
  begin
    if CanCreateDestroyCommand then
      SendDestroyCommand(FHistory);
    FHistory.NewGroupStop;
  end;
  UnregisterAtGlobalObjectmanager;
end;

procedure TRakDataObject.BeforeDestroyEvent;
begin
  FHistory := GetHistory;
  if FHistory <> nil then
  begin
    FHistory.NewGroupStart(CreateDestroyChangeData);
    if CanCreateDestroyCommand then
      SaveStateInHistory(FHistory);
  end;

  inherited;

  FireDestroyingEvent;
end;

function TRakDataObject.CreateDestroyChangeData: TRakCommandGroup;
begin
  Result := TRakCommandGroup.Create;
end;


procedure TRakDataObject.CreateNow;
begin
  RegisterAtGlobalObjectmanager;
  inherited;
end;

procedure TRakDataObject.FireDestroyingEvent;
begin
end;

function TRakDataObject.GetAsBinaryTag: TRakBinaryTag;
begin
  Result := TRakBinaryTag.Create;
  try
    SaveToBinaryTag(Result);
  except
    Result.Free;
    raise;
  end;
end;

function TRakDataObject.GetGlobalId: string;
begin
  Result := RakGlobalObjectManager.GetIdOf(self);
end;

function TRakDataObject.GetHistory: TRakHistory;
begin
  Result := FHistory;
end;

function TRakDataObject.GetOwner: TObject;
begin
  Result := FOwner;
end;

procedure TRakDataObject.GlobalIdConflict(WantedId, FunctionName: string);
begin
end;

procedure TRakDataObject.NewGroupStart(CmdGrp: TRakCommandGroup);
var
  History: TRakHistory;
begin
  History := GetHistory;
  if History = nil then
    CmdGrp.Free
  else
    History.NewGroupStart(CmdGrp);
end;

procedure TRakDataObject.NewGroupStop;
var
  History: TRakHistory;
begin
  History := GetHistory;
  if History <> nil then
    History.NewGroupStop;
end;

procedure TRakDataObject.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  GlobalName: string;
begin
  inherited;

  GlobalName := Tag.AttributeAsString('GlobalId', '');
  if GlobalName <> '' then
  begin
    if not RakGlobalObjectManager.ChangeIdOf(self, GlobalName) then
      GlobalIdConflict(GlobalName, 'LoadFromBinaryTag');
  end;
end;

procedure TRakDataObject.OwnerChangeAfter(OldOwner: TObject);
begin

end;

procedure TRakDataObject.OwnerChangeBefore(var NewOwner: TObject);
begin

end;

procedure TRakDataObject.RegisterAtGlobalObjectmanager;
begin
  RakGlobalObjectManager.AddObject(self, '');
end;

procedure TRakDataObject.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  Tag.AddAttribute(TRakBinaryAnsiString, 'GlobalId', GlobalId);
end;

procedure TRakDataObject.SetHistory(const Value: TRakHistory);
begin
  FHistory := Value;
end;

procedure TRakDataObject.SetOwner(Value: TObject);
var
  lclOld: TObject;
begin
  if Value <> FOwner then
  begin
    OwnerChangeBefore(Value);

    lclOld := FOwner;
    FOwner := Value;

    OwnerChangeAfter(lclOld);
  end;
end;

procedure TRakDataObject.UnregisterAtGlobalObjectmanager;
begin
  RakGlobalObjectManager.DeleteObject(self);
end;

function TRakDataObject.CanCreateDestroyCommand: boolean;
begin
  Result := false;
end;

procedure TRakDataObject.SendDestroyCommand(History: TRakHistory);
begin
  History.AddCommand(TRakZzHcDestroyDataObject.Init(self));
end;

procedure TRakDataObject.SaveStateInHistory(History: TRakHistory);
begin
end;

procedure TRakDataObject.Assign(Source: TRakDataObject);
begin
end;

{ TRakPenDataObject }

procedure TRakPenDataObject.Assign(Source: TRakDataObject);
var
  Src: TRakPenDataObject;
begin
  inherited;
  if Source is TRakPenDataObject then
  begin
    Src := TRakPenDataObject(Source);
    Mode := Src.Mode;
    Style := Src.Style;
    Color := Src.Color;
    MmWidth := Src.MmWidth;
  end;
end;

procedure TRakPenDataObject.ChangeAfter;
begin
end;

procedure TRakPenDataObject.ColorChangeAfter(OldValue: TColor);
begin
  AddCommand(TRakZzHcPenColor.Init(OldValue, Color));
  ChangeAfter;
end;

procedure TRakPenDataObject.CreateNow;
begin
  inherited;

  FColor := clBlack;
  FMode := pmCopy;
  FStyle := psSolid;
  //FRakStyle := rpsSolid;
  FMmWidth := 0;
  //FMmLineLength := 3;
  //FMmLineSpace := 3;
  //FMmDotLength := 0.25;
  //FLinePosition := plpCenter;
end;

{procedure TRakPenDataObject.LinePositionChangeAfter(
  OldValue: TRakPenLinePosition);
begin
  AddCommand(TRakZzHcTPenLinePosition.Init(OldValue, LinePosition));
  ChangeAfter;
end;}

procedure TRakPenDataObject.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  Mode := TPenMode(Tag.AttributeAsInteger('Mode', Ord(pmCopy)));
  Style := TPenStyle(Tag.AttributeAsInteger('Style', Ord(psSolid)));
  Color := Tag.AttributeAsInteger('Color', clBlack);
  MmWidth := Tag.AttributeAsFloat('MmWidth', 0.25);
  //MmLineLength := Tag.AttributeAsFloat('MmLineLength', 3);
  //MmLineSpace := Tag.AttributeAsFloat('MmLineSpace', 3);
  //MmDotLength := Tag.AttributeAsFloat('MmDotLength', 0.25);
  //LinePosition := TRakPenLinePosition(Tag.AttributeAsInteger('LinePosition', Ord(plpCenter)));
end;

{procedure TRakPenDataObject.MmDotLengthChangeAfter(OldValue: extended);
begin
  AddCommand(TRakZzHcPenMmDotLength.Init(OldValue, MmWidth));
  ChangeAfter;
end;

procedure TRakPenDataObject.MmLineSpaceChangeAfter(OldValue: extended);
begin
  AddCommand(TRakZzHcPenMmLineSpace.Init(OldValue, MmWidth));
  ChangeAfter;
end;

procedure TRakPenDataObject.MmLineLengthChangeAfter(OldValue: extended);
begin
  AddCommand(TRakZzHcPenMmLineLength.Init(OldValue, MmWidth));
  ChangeAfter;
end;}

procedure TRakPenDataObject.MmWidthChangeAfter(OldValue: extended);
begin
  AddCommand(TRakZzHcPenMmWidth.Init(OldValue, MmWidth));
  ChangeAfter;
end;

procedure TRakPenDataObject.ModeChangeAfter(OldValue: TPenMode);
begin
  AddCommand(TRakZzHcTPenMode.Init(OldValue, Mode));
  ChangeAfter;
end;

{procedure TRakPenDataObject.RakStyleChangeAfter(OldValue: TRakPenStyle);
begin
  AddCommand(TRakZzHcTRakPenStyle.Init(OldValue, RakStyle));
  ChangeAfter;
end;}

procedure TRakPenDataObject.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  if Mode <> pmCopy then
    TRakBinaryByte.AddToTag(Tag, 'Mode', Ord(Mode));
  if Style <> psSolid then
    TRakBinaryByte.AddToTag(Tag, 'Style', Ord(Style));
  {if RakStyle <> rpsSolid then
  begin
    TRakBinaryByte.AddToTag(Tag, 'RakStyle', Ord(RakStyle));

    if RakStyle in [rpsDotLine, rpsLine] then
    begin
      if MmLineLength <> 3 then
        TRakBinaryExtended.AddToTag(Tag, 'MmLineLength', MmLineLength);
      if MmLineSpace <> 3 then
        TRakBinaryExtended.AddToTag(Tag, 'MmLineSpace', MmLineSpace);
      if (RakStyle = rpsDotLine) and (MmDotLength <> 0.25) then
        TRakBinaryExtended.AddToTag(Tag, 'MmDotLength', MmDotLength);
    end;
  end;}
  if Color <> clBlack then
    TRakBinaryLongint.AddToTag(Tag, 'Color', Color);
  if MmWidth <> 0.25 then
    TRakBinaryExtended.AddToTag(Tag, 'MmWidth', MmWidth);
  //if LinePosition <> plpCenter then
  //  TRakBinaryByte.AddToTag(Tag, 'LinePosition', Ord(LinePosition));
end;

procedure TRakPenDataObject.SetColor(const Value: TColor);
var
  OldValue: TColor;
begin
  if FColor <> Value then
  begin
    OldValue := FColor;

    FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

{procedure TRakPenDataObject.SetLinePosition(
  const Value: TRakPenLinePosition);
var
  OldValue: TRakPenLinePosition;
begin
  if FLinePosition <> Value then
  begin
    OldValue := FLinePosition;

    FLinePosition := Value;

    LinePositionChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.SetMmDotLength(const Value: extended);
var
  OldValue: extended;
begin
  if FMmDotLength <> Value then
  begin
    OldValue := FMmDotLength;

    FMmDotLength := Value;

    MmDotLengthChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.SetMmLineLength(const Value: extended);
var
  OldValue: extended;
begin
  if FMmLineLength <> Value then
  begin
    OldValue := FMmLineLength;

    FMmLineLength := Value;

    MmLineLengthChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.SetMmLineSpace(const Value: extended);
var
  OldValue: extended;
begin
  if FMmLineSpace <> Value then
  begin
    OldValue := FMmLineSpace;

    FMmLineSpace := Value;

    MmLineSpaceChangeAfter(OldValue);
  end;
end;}

procedure TRakPenDataObject.SetMmWidth(const Value: extended);
var
  OldValue: extended;
begin
  if FMmWidth <> Value then
  begin
    OldValue := FMmWidth;

    FMmWidth := Value;

    MmWidthChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.SetMode(const Value: TPenMode);
var
  OldValue: TPenMode;
begin
  if FMode <> Value then
  begin
    OldValue := FMode;

    FMode := Value;

    ModeChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.SetPen(APen: TPen; PixelPerMm: extended);
begin
  APen.Mode := Mode;
  APen.Style := Style;
  APen.Color := Color;
  APen.Width := round(MmWidth * PixelPerMm);
end;

{procedure TRakPenDataObject.SetRakStyle(const Value: TRakPenStyle);
var
  OldValue: TRakPenStyle;
begin
  if FRakStyle <> Value then
  begin
    OldValue := FRakStyle;

    FRakStyle := Value;

    RakStyleChangeAfter(OldValue);
  end;
end;}

procedure TRakPenDataObject.SetStyle(const Value: TPenStyle);
var
  OldValue: TPenStyle;
begin
  if FStyle <> Value then
  begin
    OldValue := FStyle;

    FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakPenDataObject.StyleChangeAfter(OldValue: TPenStyle);
begin
  AddCommand(TRakZzHcTPenStyle.Init(OldValue, Style));
  ChangeAfter;
end;

{ TRakBrushDataObject }

procedure TRakBrushDataObject.Assign(Source: TRakDataObject);
var
  Src: TRakBrushDataObject;
begin
  inherited;
  if Source is TRakBrushDataObject then
  begin
    Src := TRakBrushDataObject(Source);
    Style := Src.Style;
    Color := Src.Color;
  end;
end;

procedure TRakBrushDataObject.ChangeAfter;
begin
end;

procedure TRakBrushDataObject.ColorChangeAfter(OldValue: TColor);
begin
  AddCommand(TRakZzHcBrushColor.Init(OldValue, Color));
  ChangeAfter;
end;

procedure TRakBrushDataObject.CreateNow;
begin
  inherited;

  FStyle := bsSolid;
  FColor := clBlack;
end;

procedure TRakBrushDataObject.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  Style := TBrushStyle(Tag.AttributeAsInteger('Style', Ord(Style)));
  Color := Tag.AttributeAsInteger('Color', Color);
end;

procedure TRakBrushDataObject.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  Tag.AddAttribute(TRakBinaryByte, 'Style', IntToStr(Ord(Style)));
  Tag.AddAttribute(TRakBinaryLongint, 'Color', IntToStr(Color));
end;

procedure TRakBrushDataObject.SetBrush(ABrush: TBrush);
begin
  ABrush.Color := Color;
  ABrush.Style := Style;
end;

procedure TRakBrushDataObject.SetColor(const Value: TColor);
var
  OldValue: TColor;
begin
  if FColor <> Value then
  begin
    OldValue := FColor;

    FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

procedure TRakBrushDataObject.SetStyle(const Value: TBrushStyle);
var
  OldValue: TBrushStyle;
begin
  if FStyle <> Value then
  begin
    OldValue := FStyle;

    FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakBrushDataObject.StyleChangeAfter(OldValue: TBrushStyle);
begin
  AddCommand(TRakZzHcTBrushStyle.Init(OldValue, Style));
  ChangeAfter;
end;

{ TRakFontDataObject }

procedure TRakFontDataObject.Assign(Source: TRakDataObject);
var
  Src: TRakFontDataObject;
begin
  if Source is TRakFontDataObject then
  begin
    Src := TRakFontDataObject(Source);

    MmHeight := Src.MmHeight;
    Name := Src.Name;
    Color := Src.Color;
    Style := Src.Style;
  end;
end;

procedure TRakFontDataObject.ChangeAfter;
begin
end;

procedure TRakFontDataObject.ColorChangeAfter(OldValue: TColor);
begin
  AddCommand(TRakZzHcFontColor.Init(OldValue, Color));
  ChangeAfter;
end;

procedure TRakFontDataObject.CreateNow;
begin
  inherited;

  FMmHeight := 3.5;
  FName := 'Arial';
  FColor := clBlack;
  FStyle := [];
end;

procedure TRakFontDataObject.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  lclStyles: TFontStyles;
begin
  inherited;

  Color := Tag.AttributeAsInteger('Color', Color);
  MmHeight := Tag.AttributeAsFloat('MmHeight', MmHeight);
  Name := Tag.AttributeAsString('Name', Name);

  lclStyles := [];
  if Tag.AttributeAsBoolean('Bold', false) then
    lclStyles := lclStyles + [fsBold];

  if Tag.AttributeAsBoolean('Italic', false) then
    lclStyles := lclStyles + [fsItalic];

  if Tag.AttributeAsBoolean('Underline', false) then
    lclStyles := lclStyles + [fsUnderline];

  if Tag.AttributeAsBoolean('StrikeOut', false) then
    lclStyles := lclStyles + [fsStrikeOut];

  Style := lclStyles;
end;

procedure TRakFontDataObject.MmHeightChangeAfter(OldValue: extended);
begin
  AddCommand(TRakZzHcFontMmHeight.Init(OldValue, MmHeight));
  ChangeAfter;
end;

procedure TRakFontDataObject.NameChangeAfter(OldValue: string);
begin
  AddCommand(TRakZzHcFontName.Init(OldValue, Name));
  ChangeAfter;
end;

procedure TRakFontDataObject.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  Tag.AddAttribute(TRakBinaryLongint, 'Color', IntToStr(Color));
  Tag.AddAttribute(TRakBinaryExtended, 'MmHeight', FloatToNormStr(MmHeight));
  Tag.AddAttribute(TRakBinaryAnsiString, 'Name', Name);
  Tag.AddAttribute(TRakBinaryBoolean, 'Bold', BoolToStr(fsBold in Style));
  Tag.AddAttribute(TRakBinaryBoolean, 'Italic', BoolToStr(fsItalic in Style));
  Tag.AddAttribute(TRakBinaryBoolean, 'Underline', BoolToStr(fsUnderline in Style));
  Tag.AddAttribute(TRakBinaryBoolean, 'StrikeOut', BoolToStr(fsStrikeOut in Style));
end;

procedure TRakFontDataObject.SetColor(const Value: TColor);
var
  OldValue: TColor;
begin
  if FColor <> Value then
  begin
    OldValue := FColor;

    FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

procedure TRakFontDataObject.SetFont(AFont: TFont; PixelPerMm: extended);
begin
  AFont.Color := Color;
  AFont.Name := Name;
  AFont.Style := Style;
  AFont.Height := round(MmHeight * PixelPerMm);
end;

procedure TRakFontDataObject.SetMmHeight(const Value: extended);
var
  OldValue: extended;
begin
  if FMmHeight <> Value then
  begin
    OldValue := FMmHeight;

    FMmHeight := Value;

    MmHeightChangeAfter(OldValue);
  end;
end;

procedure TRakFontDataObject.SetName(const Value: string);
var
  OldValue: string;
begin
  if FName <> Value then
  begin
    OldValue := FName;

    FName := Value;

    NameChangeAfter(OldValue);
  end;
end;

procedure TRakFontDataObject.SetStyle(const Value: TFontStyles);
var
  OldValue: TFontStyles;
begin
  if FStyle <> Value then
  begin
    OldValue := FStyle;

    FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakFontDataObject.StyleChangeAfter(OldValue: TFontStyles);
begin
  AddCommand(TRakZzHcTFontStyles.Init(OldValue, Style));
  ChangeAfter;
end;

function TRakFontDataObject.TextWidth(Text: string): extended;
var
  Font: TRakWYSIWYGFont;
begin
  Font := TRakWYSIWYGFont.Create;
  try
    Font.Font.Height := trunc(MmHeight * 10);
    Font.Font.Name := Name;
    Font.Font.Style := Style;
    Result := Font.TextWidth(Text) / 10;
  finally
    Font.Free;
  end;
end;

procedure TRakFontDataObject.WrapText(Zeile: string; MmWidth: extended;
  Zeilen: TStrings);
const
  WHITESPACE: set of char = [#0..#32];
  ZEILENANFANG: set of char = ['<','(','[','{','#','$','&','*','+','/','@','\','`','^','|',#126,#127,#129..#191];
  ZEILENENDE: set of char = ['>',')',']','}','-','.',';',',',':','!','?','%','=',#128];
  GROSSBUCHSTABEN: set of char = ['A'..'Z','','',''];
  BINDUNG_LOSE: set of char = ['0'..'9','_','''','"',#192..#255];
var
  Rest: string;
  Idx: integer;
  TmpWidth: extended;
  MinWidth: extended;
  MinChar: integer;
  MaxChar: integer;
  TryChar: integer;
begin
  Zeilen.Clear;
  Rest := trim(Zeile);
  while Rest <> '' do //Von Rest wird jetzt solange was abgezwickt, was in den zur Verfgung stehenden Platz passt, bis Rest leer ist
  begin
    Zeile := Rest;
    //Zeile wird jetzt solange vergrert und verkleinert, bis die Zeile ideal in den zur Verfgung stehenden Platz passt
    TmpWidth := TextWidth(Zeile);
    if TmpWidth > MmWidth then
    begin // Wenn die Zeile noch zu breit ist
      MinChar := 1; //Mindestens ein Zeichen muss in die Zeile
      MaxChar := length(Zeile);//Es knnen maximal alle Zeichen in der Zeile sein
      TryChar := MaxChar;//TmpWidth enthlt die Width von allen Zeichen
      MinWidth := 0;//MinChar hat mindestens eine Breite von 0
      while MinChar < MaxChar do
      begin //Solange nicht klar ist, wieviele Zeichen am besten in den Platz passen
        if TmpWidth = MinWidth then //Wenn bei der Division durch (TmpBreite - MinBreite) ein Fehler auftreten wrde
          TryChar := MinChar + 1
        else
        begin // Wenn das neue TryChar ideal berechnet werden kann
          // Bei der folgenden Berechnung von TryChar wird davon ausgegangen, dass alle Zeichen die gleich Breite haben.
          // So wird mit Hilfe der zentrischen Streckung der wahrscheinlich beste Wert fr TryChar ermittelt
          TryChar := trunc(MinChar + (TryChar - MinChar) / (TmpWidth - MinWidth) * (MmWidth - MinWidth));
          if TryChar > MaxChar then //Wenn TryChar einen Wert angenommen hat, von dem wir schon wissen, dass er nicht in MmBreite reinpasst
            TryChar := MaxChar
          else if TryChar <= MinChar then //Wenn TryChar einen Wert angenommen hat, von dem wir schon wissen, dass er in MmBreite reinpasst
            TryChar := MinChar + 1;
        end;
        //Die Breite ermitteln
        TmpWidth := TextWidth(Copy(Rest, 1, TryChar));
        if TmpWidth > MmWidth then // Wenn es zu breit ist
          MaxChar := TryChar - 1 //Dann wissen wir, dass maximal ein Zeichen weniger reinpasst
        else if TmpWidth < MmWidth then
        begin // Wenn es reinpasst, aber nicht exakt
          MinChar := TryChar; //Dann wissen wir wieviel Zeichen mindestens reinpassen
          MinWidth := TmpWidth; //und merken uns die ermittelte Breite fr die nchste Berechnung von TryChar
        end
        else
        begin // Wenn es exakt gepasst hat
          // Dann knnen wir fr einen Abbruch der Schleife sorgen
          MinChar := TryChar;
          MaxChar := TryChar;
        end;
      end; // while MinChar < MaxChar do
      //In Zeile den String speichern, der den zur Verfgung stehenden Platz ideal ausntzt
      Zeile := Copy(Rest, 1, MinChar);
    end;
    // Zeile passt jetzt rein.

    if (length(Zeile) < length(Rest)) and not (Rest[length(Zeile)+1] in WHITESPACE) then
    begin // Wenn die Zeile nicht der ganze Rest ist und das erste Zeichen von Rest, das nicht in Zeile enthalten ist, kein Whitespace ist
      // Dann schauen wir, ob wir in Zeile eine bessere Stelle finden, an der wir einen Zeilenumbruch vornehmen knnen
      // Dazu suchen wir zuerst nach dem letzten Whitespace in Zeile
      Idx := PosRight(WHITESPACE, Zeile);
      if Idx > 0 then
      begin // Wenn in Zeile ein Whitespace vorhanden war
        Zeile := Left(Zeile, Idx);
      end
      else
      begin // Wenn in der ganzen Zeile kein einziges Whitespace vorhanden ist
        Idx := PosRight(ZEILENANFANG+ZEILENENDE, Zeile);
        if (Idx > 1) or ((Idx > 0) and (Zeile[Idx] in ZEILENENDE)) then
        begin // Wenn in Zeile ein Sonderzeichen vorhanden war
          if Zeile[Idx] in ZEILENANFANG then
            Zeile := Left(Zeile, Idx-1)
          else
            Zeile := Left(Zeile, Idx);
        end
        else
        begin // Wenn in der ganzen Zeile kein einziges Sonderzeichen vorhanden ist
          Idx := PosRight(GROSSBUCHSTABEN, Zeile);
          if Idx > 1 then
            Zeile := Left(Zeile, Idx-1)
          else
          begin
            Idx := PosRight(BINDUNG_LOSE, Zeile);
            if Idx > 0 then
              Zeile := Left(Zeile, Idx);
          end;
        end;
      end;
    end;

    // Die neue Zeile merken
    Zeilen.Add(trim(Zeile));
    // Den restlichen String ermitteln, wobei fhrende Whitespaces gleich entfernt werden
    Rest := trim(CutLeft(Rest, length(Zeile)));
  end;
end;

{ TRakMmBoundsDataObject }

function TRakMmBoundsDataObject.GetMmHorzCenter: extended;
begin
  Result := MmLeft + MmWidth / 2;
end;

function TRakMmBoundsDataObject.GetMmRight: extended;
begin
  Result := MmLeft + MmWidth;
end;

function TRakMmBoundsDataObject.GetMmRect: TExtRect;
begin
  Result := ExtBounds(MmLeft, MmTop, MmWidth, MmHeight);
end;

function TRakMmBoundsDataObject.GetMmBottom: extended;
begin
  Result := MmTop + MmHeight;
end;

function TRakMmBoundsDataObject.GetMmVertCenter: extended;
begin
  Result := MmTop + MmHeight / 2;
end;

procedure TRakMmBoundsDataObject.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  MmLeft := Tag.AttributeAsFloat('Left', MmLeft);
  MmTop := Tag.AttributeAsFloat('Top', MmTop);
  MmWidth := Tag.AttributeAsFloat('Width', MmWidth);
  MmHeight := Tag.AttributeAsFloat('Height', MmHeight);
end;

procedure TRakMmBoundsDataObject.MmChangeAfter(const OldLeft, OldTop, OldWidth,
  OldHeight: extended);
begin

end;

procedure TRakMmBoundsDataObject.MmChangeBefore(var NewLeft, NewTop, NewWidth,
  NewHeight: extended);
begin

end;

procedure TRakMmBoundsDataObject.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  Tag.AddAttribute(TRakBinaryExtended, 'Left', FloatToNormStr(MmLeft));
  Tag.AddAttribute(TRakBinaryExtended, 'Top', FloatToNormStr(MmTop));
  Tag.AddAttribute(TRakBinaryExtended, 'Width', FloatToNormStr(MmWidth));
  Tag.AddAttribute(TRakBinaryExtended, 'Height', FloatToNormStr(MmHeight));
end;

procedure TRakMmBoundsDataObject.SetMm(Left, Top, Width, Height: extended);
begin
  if Width < 0 then
    Width := 0;

  if Height < 0 then
    Height := 0;

  if (Left <> MmLeft) or (Top <> MmTop) or (Width <> MmWidth)
    or (Height <> MmHeight) then
  begin
    SetMmNow(Left, Top, Width, Height);
  end;
end;

procedure TRakMmBoundsDataObject.SetMmWidth(const Value: extended);
begin
  SetMm(MmLeft, MmTop, Value, MmHeight);
end;

procedure TRakMmBoundsDataObject.SetMmHeight(const Value: extended);
begin
  SetMm(MmLeft, MmTop, MmWidth, Value);
end;

procedure TRakMmBoundsDataObject.SetMmHorzCenter(const Value: extended);
begin
  MmLeft := Value - MmWidth / 2;
end;

procedure TRakMmBoundsDataObject.SetMmLeft(const Value: extended);
begin
  SetMm(Value, MmTop, MmWidth, MmHeight);
end;

procedure TRakMmBoundsDataObject.SetMmNow(Left, Top, Width, Height: extended);
var
  lclLeft, lclTop, lclWidth, lclHeight: extended;
begin
  MmChangeBefore(Left, Top, Width, Height);

  lclLeft := FMmLeft;
  lclTop := FMmTop;
  lclWidth := FMmWidth;
  lclHeight := FMMHeight;

  FMmLeft := Left;
  FMmTop := Top;
  FMmWidth := Width;
  FMMHeight := Height;

  MmChangeAfter(lclLeft, lclTop, lclWidth, lclHeight);
end;

procedure TRakMmBoundsDataObject.SetMmTop(const Value: extended);
begin
  SetMm(MmLeft, Value, MmWidth, MmHeight);
end;

procedure TRakMmBoundsDataObject.SetMmRight(const Value: extended);
begin
  MmLeft := Value - MmWidth;
end;

procedure TRakMmBoundsDataObject.SetMmBottom(const Value: extended);
begin
  MmTop := Value - MmHeight;
end;

procedure TRakMmBoundsDataObject.SetMmVertCenter(const Value: extended);
begin
  MmTop := Value - MmHeight / 2;
end;

{ TRakZzHcTPenMode }

constructor TRakZzHcTPenMode.Init(Old, New: TPenMode);
begin
  self.Old := Old;
  self.New := New;
end;

function TRakZzHcTPenMode.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).Mode := New;
end;

function TRakZzHcTPenMode.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).Mode := Old;
end;

{ TRakZzHcTPenStyle }

constructor TRakZzHcTPenStyle.Init(Old, New: TPenStyle);
begin
  self.Old := Old;
  self.New := New;
end;

function TRakZzHcTPenStyle.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).Style := New;
end;

function TRakZzHcTPenStyle.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).Style := Old;
end;

{ TRakZzHcTBrushStyle }

constructor TRakZzHcTBrushStyle.Init(OldValue, NewValue: TBrushStyle);
begin
  Create;
  FOldValue := OldValue;
  FNewValue := NewValue;
end;

function TRakZzHcTBrushStyle.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakBrushDataObject;
  if Result then
    TRakBrushDataObject(Obj).Style := FNewValue;
end;

function TRakZzHcTBrushStyle.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakBrushDataObject;
  if Result then
    TRakBrushDataObject(Obj).Style := FOldValue;
end;

{ TRakZzHcTFontStyles }

constructor TRakZzHcTFontStyles.Init(Old, New: TFontStyles);
begin
  self.Old := Old;
  self.New := New;
end;


function TRakZzHcTFontStyles.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakFontDataObject;
  if Result then
    TRakFontDataObject(Obj).Style := New;
end;

function TRakZzHcTFontStyles.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakFontDataObject;
  if Result then
    TRakFontDataObject(Obj).Style := Old;
end;

{ TRakZzHcPenColor }

procedure TRakZzHcPenColor.SetValue(Obj: TObject; Value: TColor;
  var Done: boolean);
begin
  TRakPenDataObject(Obj).Color := Value;
end;

{ TRakZzHcPenMmWidth }

procedure TRakZzHcPenMmWidth.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakPenDataObject(Obj).MmWidth := Value;
end;

{ TRakZzHcBrushColor }

procedure TRakZzHcBrushColor.SetValue(Obj: TObject; Value: TColor;
  var Done: boolean);
begin
  TRakBrushDataObject(Obj).Color := Value;
end;

{ TRakZzHcFontColor }

procedure TRakZzHcFontColor.SetValue(Obj: TObject; Value: TColor;
  var Done: boolean);
begin
  TRakFontDataObject(Obj).Color := Value;
end;

{ TRakZzHcFontMmHeight }

procedure TRakZzHcFontMmHeight.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakFontDataObject(Obj).MmHeight := Value;
end;

{ TRakZzHcFontName }

procedure TRakZzHcFontName.SetValue(Obj: TObject; Value: string;
  var Done: boolean);
begin
  TRakFontDataObject(Obj).Name := Value;
end;

{ TRakZzHcMmLeft }

procedure TRakZzHcMmLeft.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakMmBoundsDataObject(Obj).MmLeft := Value;
end;

{ TRakZzHcMmTop }

procedure TRakZzHcMmTop.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakMmBoundsDataObject(Obj).MmTop := Value;
end;

{ TRakZzHcMmWidth }

procedure TRakZzHcMmWidth.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakMmBoundsDataObject(Obj).MmWidth := Value;
end;

{ TRakZzHcMmHeight }

procedure TRakZzHcMmHeight.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  TRakMmBoundsDataObject(Obj).MmHeight := Value;
end;

{ TRakZzHcDestroyDataObject }

function TRakZzHcDestroyDataObject.CheckChangedObject(
  Obj: TObject): boolean;
begin
  Result := true;
end;

function TRakZzHcDestroyDataObject.CreateObject: TObject;
begin
  Result := FKlasse.Create;
end;

constructor TRakZzHcDestroyDataObject.Init(Obj: TRakDataObject);
begin
  inherited Init(Obj.GlobalId);

  FKlasse := TRakDataObjectClass(Obj.ClassType);
end;

{ TRakZzHcTRakPenStyle }

{constructor TRakZzHcTRakPenStyle.Init(Old, New: TRakPenStyle);
begin
  Create;

  FOld := Old;
  FNew := New;
end;

function TRakZzHcTRakPenStyle.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).RakStyle := FNew;
end;

function TRakZzHcTRakPenStyle.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).RakStyle := FOld;
end;}

{ TRakZzHcPenMmLineLength }

{procedure TRakZzHcPenMmLineLength.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  (Obj as TRakPenDataObject).MmLineLength := Value;
end;}

{ TRakZzHcPenMmDotLength }

{procedure TRakZzHcPenMmDotLength.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  (Obj as TRakPenDataObject).MmDotLength := Value;
end;}

{ TRakZzHcPenMmLineSpace }

{procedure TRakZzHcPenMmLineSpace.SetValue(Obj: TObject; Value: extended;
  var Done: boolean);
begin
  (Obj as TRakPenDataObject).MmLineSpace := Value;
end;}

{ TRakZzHcTPenLinePosition }

{constructor TRakZzHcTPenLinePosition.Init(Old, New: TRakPenLinePosition);
begin
  Create;
  FOld := Old;
  FNew := New;
end;

function TRakZzHcTPenLinePosition.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).LinePosition := FNew;
end;

function TRakZzHcTPenLinePosition.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj is TRakPenDataObject;
  if Result then
    TRakPenDataObject(Obj).LinePosition := FOld;
end;}

end.
