unit RakFuncPainting;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung :  Die Unit RakFuncPainting enthlt Hilfsfunktionen
                  fr das Zeichnen.
// erstellt am  : 17.3.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  // Kasparsoft
  miscfunc, misctype, RakCanvasData, RakDataObject,
  // Delphi
  Classes, Windows, Graphics;

{procedure PaintLine(Canvas: TCanvas; cd: TRakCanvasData; Pen: TRakPenDataObject;
  StartPoint: TExtPoint; Length: extended; Horizontal: boolean); overload;
procedure PaintLine(Canvas: TCanvas; cd: TRakCanvasData; Pen: TRakPenDataObject;
  StartPoint, EndPoint: TExtPoint); overload;}

implementation

{procedure PaintLine(Canvas: TCanvas; cd: TRakCanvasData; Pen: TRakPenDataObject;
  StartPoint: TExtPoint; Length: extended; Horizontal: boolean);
var
  EndPoint: TExtPoint;
  sPoint, ePoint: TPoint;
  csPoint, cePoint: TPoint;
  Breite: TPoint;
  xBreite, yBreite: extended;
  eX, eY: extended;
  Poly: array[0..3] of TPoint;
  Sign: integer;
  PenWidth: extended;
begin
  if (Pen.RakStyle <> rpsClear) and not FloatEqual(Length , 0) then
  begin // Wenn die Linie wirklich gezeichnet werden soll und sie berhaupt existiert
    Sign := SignToInt(Length);
    EndPoint := StartPoint;
    xBreite := 0;
    yBreite := 0;

    // Wir verpassen dem Stift eine Mindestbreite
    PenWidth := Pen.MmWidth;
    if FloatEqual(PenWidth, 0) then
      PenWidth := 0.001
    else if abs(PenWidth) < 0.01 then
      PenWidth := 0.001 * SignToInt(PenWidth);

    if Horizontal then
    begin
      EndPoint.X := EndPoint.X + Length;
      yBreite := PenWidth * Sign;
    end
    else
    begin
      EndPoint.Y := EndPoint.Y + Length;
      xBreite := -PenWidth * Sign
    end;

    Breite.X := cd.MmToPixelX(xBreite, false);
    Breite.Y := cd.MmToPixelY(yBreite, false);
    if (abs(Breite.X) > 1) or (abs(Breite.Y) > 1) then
    begin
      //dec(Breite.X, SignToInt(Breite.X));
      //dec(Breite.Y, SignToInt(Breite.Y));
    end
    else if xBreite <> 0 then
      Breite.X := SignToInt(xBreite)
    else
      Breite.Y := SignToInt(yBreite);
    // Jetzt wissen wir, wie breit die Linie zu zeichnen ist

    // Die Pixelwerte der Linie ermitteln
    sPoint := cd.MmPointToPixelPoint(StartPoint, true);
    ePoint := cd.MmPointToPixelPoint(EndPoint, true);

    // Jetzt initialisieren wir den Canvas
    Canvas.Pen.Mode := Pen.Mode;
    Canvas.Brush.Color := Pen.Color;
    Canvas.Brush.Style := bsSolid;

    if Pen.RakStyle = rpsSolid then
    begin
      Canvas.FillRect(Rect(sPoint.X, sPoint.Y, ePoint.X + Breite.X, ePoint.Y + Breite.Y));
    end
    else
    begin

    end;
  end;
end;


procedure PaintLine(Canvas: TCanvas; cd: TRakCanvasData; Pen: TRakPenDataObject;
  StartPoint, EndPoint: TExtPoint); overload;
var
  sPoint, ePoint: TPoint;
  csPoint, cePoint: TPoint;
  Breite: TPoint;
  xBreite, yBreite: extended;
  eX, eY: extended;
  eLen: extended;
  Poly: array[0..3] of TPoint;
begin
  if Pen.RakStyle <> rpsClear then
  begin // Wenn die Linie wirklich gezeichnet werden soll

    // Zuerst wird die Breite der Linie vorbereitet
    // Dazu ermitteln wir den NormVektor der Linie
    eX := (EndPoint.X - StartPoint.X); // Die Linie als Vektor
    eY := (EndPoint.Y - StartPoint.Y);
    eLen := sqrt(sqr(eX) + sqr(ey)); // Die Lnge des Vektors
    if not FloatEqual(eLen , 0) then
    begin // Wenn unsere Linie berhaupt existiert
      eX := eX / eLen; // Den Vektor auf eine Lnge von 1 bringen
      eY := eY / eLen;
      // Mit dem Normvektor knnen wir jetzt den Offset der Linie berechnen,
      // damit wir die korrekte Breite darstellen knnen.
      xBreite := -eY*Pen.MmWidth;
      yBreite := eX*Pen.MmWidth;
      // Wir ermitteln gleich die Pixelwerte
      Breite.X := cd.MmToPixelX(xBreite, false);
      Breite.Y := cd.MmToPixelY(yBreite, false);
      if Breite.X > 0 then
        dec(Breite.X)
      else if Breite.X < 0 then
        inc(Breite.X);
      if Breite.Y > 0 then
        dec(Breite.Y)
      else if Breite.Y < 0 then
        inc(Breite.Y);
      // Jetzt wissen wir, wie breit die Linie zu zeichnen ist

      // Die Pixelwerte der Linie ermitteln
      sPoint := cd.MmPointToPixelPoint(StartPoint, true);
      ePoint := cd.MmPointToPixelPoint(EndPoint, true);

      // Jetzt initialisieren wir den Canvas
      Canvas.Pen.Mode := Pen.Mode;
      Canvas.Pen.Color := Pen.Color;
      Canvas.Pen.Width := 0;
      Canvas.Pen.Style := psClear;
      Canvas.Brush.Color := Pen.Color;
      Canvas.Brush.Style := bsSolid;

      if Pen.RakStyle = rpsSolid then
      begin
        Poly[0].X := sPoint.X;
        Poly[0].Y := sPoint.Y;
        Poly[1].X := ePoint.X;
        Poly[1].Y := ePoint.Y;
        Poly[2].X := ePoint.X + Breite.X;
        Poly[2].Y := ePoint.Y + Breite.Y;
        Poly[3].X := sPoint.X + Breite.X;
        Poly[3].Y := sPoint.Y + Breite.Y;

        Canvas.Polygon(Poly);
      Canvas.Pen.Style := psSolid;
      Canvas.Brush.Style := bsClear;
        Canvas.Polygon(Poly);
      end
      else
      begin

      end;
    end;
  end;
end;}


end.
