unit RakLocal;
{*******************************************************************************
** Copyright  Richard Kaspar Softwareentwicklung, 2002. Alle Rechte vorbehalten
** Unit         : RakLocal
** Autor        : Richard Kaspar
** Beschreibung : RakLocal stellt Schnittstellen und Klassen zur Verfgung,
                  die nur innerhalb einer Methode exisitieren und sich
                  am Ende selbst freigeben, wenn sie nicht mehr bentigt werden.
                  Mit den Schnitstellen bleibt einem das
                    try
                    finally
                      EinObjekt.Free
                    end;
                  erspart, wenn man sicherstellen will, dass das Objekt auch bei einer
                  Exception freigegeben wird.
** nderungen    :
	8.4.2003 TRakInterfacedList.Add ist jetzt virtual;
*******************************************************************************}
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  classes, contnrs;

type
  IRakList = interface (IUnknown)
    function GetCapacity: Integer;
    function GetCount: Integer;
    function GetItems(Index: Integer): Pointer;
    function GetList: PPointerList;
    procedure SetCapacity(const Value: Integer);
    procedure SetCount(const Value: Integer);
    procedure SetItems(Index: Integer; const Value: Pointer);

    procedure Assign(Source: IRakList);
    function Add(Item: Pointer): Integer;
    procedure Clear;
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Expand: TList;
    function Extract(Item: Pointer): Pointer;
    function First: Pointer;
    function IndexOf(Item: Pointer): Integer;
    procedure Insert(Index: Integer; Item: Pointer);
    function Last: Pointer;
    procedure Move(CurIndex, NewIndex: Integer);
    function Remove(Item: Pointer): Integer;
    procedure Pack;
    procedure Sort(Compare: TListSortCompare);

    property Capacity: Integer read GetCapacity write SetCapacity;
    property Count: Integer read GetCount write SetCount;
    property Items[Index: Integer]: Pointer read GetItems write SetItems; default;
    property List: PPointerList read GetList;
  end;

  TRakInterfacedList = class (TInterfacedObject, IRakList)
  private
    FList: TList;
  protected
    function CreateList: TList; virtual;
  protected
    function GetCapacity: Integer;
    function GetCount: Integer;
    function GetItems(Index: Integer): Pointer;
    function GetList: PPointerList;
    procedure SetCapacity(const Value: Integer);
    procedure SetCount(const Value: Integer);
    procedure SetItems(Index: Integer; const Value: Pointer);

  public
    constructor Create;
    destructor Destroy; override;

    procedure Assign(Source: IRakList); virtual;
    function Add(Item: Pointer): Integer; overload; virtual; 
    procedure Clear; virtual;
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Expand: TList;
    function Extract(Item: Pointer): Pointer; overload;
    function First: Pointer;
    function IndexOf(Item: Pointer): Integer; overload;
    procedure Insert(Index: Integer; Item: Pointer); overload;
    function Last: Pointer;
    procedure Move(CurIndex, NewIndex: Integer);
    function Remove(Item: Pointer): Integer; overload;
    procedure Pack;
    procedure Sort(Compare: TListSortCompare);

    property Capacity: Integer read GetCapacity write SetCapacity;
    property Count: Integer read GetCount write SetCount;
    property Items[Index: Integer]: Pointer read GetItems write SetItems; default;
    property List: PPointerList read GetList;
  end;

  IRakClassList = interface (IRakList)
    function GetRakClassItems(Index: Integer): TClass;
    procedure SetRakClassItems(Index: Integer; const Value: TClass);

    function AddClass(aClass: TClass): Integer; overload;
    function RemoveClass(aClass: TClass): Integer;
    function IndexOfClass(aClass: TClass): Integer;
    procedure InsertClass(Index: Integer; aClass: TClass);

    property Items[Index: Integer]: TClass read GetRakClassItems write SetRakClassItems; default;
  end;

  TRakInterfacedClassList = class(TRakInterfacedList, IRakClassList)
  private
    FList: TClassList;
  protected
    function GetRakClassItems(Index: Integer): TClass;
    procedure SetRakClassItems(Index: Integer; const Value: TClass);
  protected
    function CreateList: TList; override;
  public
    function AddClass(aClass: TClass): Integer;
    function RemoveClass(aClass: TClass): Integer;
    function IndexOfClass(aClass: TClass): Integer;
    procedure InsertClass(Index: Integer; aClass: TClass);

    property Items[Index: Integer]: TClass read GetRakClassItems write SetRakClassItems; default;
  end;


implementation


{ TRakInterfacedList }

function TRakInterfacedList.Add(Item: Pointer): Integer;
begin
  Result := FList.Add(Item);
end;

procedure TRakInterfacedList.Assign(Source: IRakList);
var
  i: integer;
begin
	Clear;
  for i := 0 to Source.Count - 1 do
  	Add(Source[i]);
end;

procedure TRakInterfacedList.Clear;
begin
  FList.Clear;
end;

constructor TRakInterfacedList.Create;
begin
  inherited;
  FList := CreateList;
end;

function TRakInterfacedList.CreateList: TList;
begin
  Result := TList.Create;
end;

procedure TRakInterfacedList.Delete(Index: Integer);
begin
  FList.Delete(Index);
end;

destructor TRakInterfacedList.Destroy;
begin
  FList.Free;

  inherited;
end;

procedure TRakInterfacedList.Exchange(Index1, Index2: Integer);
begin
  FList.Exchange(Index1, Index2);
end;

function TRakInterfacedList.Expand: TList;
begin
  Result := FList.Expand
end;

function TRakInterfacedList.Extract(Item: Pointer): Pointer;
begin
  Result := FList.Extract(Item);
end;

function TRakInterfacedList.First: Pointer;
begin
  Result := FList.First;
end;

function TRakInterfacedList.GetCapacity: Integer;
begin
  Result := FList.Capacity;
end;

function TRakInterfacedList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TRakInterfacedList.GetItems(Index: Integer): Pointer;
begin
  Result := FList.Items[Index];
end;

function TRakInterfacedList.GetList: PPointerList;
begin
  Result := FList.List;
end;

function TRakInterfacedList.IndexOf(Item: Pointer): Integer;
begin
  Result := FList.IndexOf(Item);
end;

procedure TRakInterfacedList.Insert(Index: Integer; Item: Pointer);
begin
  FList.Insert(Index, Item);
end;

function TRakInterfacedList.Last: Pointer;
begin
  Result := FList.Last;
end;

procedure TRakInterfacedList.Move(CurIndex, NewIndex: Integer);
begin
  FList.Move(CurIndex, NewIndex);
end;

procedure TRakInterfacedList.Pack;
begin
  FList.Pack;
end;

function TRakInterfacedList.Remove(Item: Pointer): Integer;
begin
  Result := FList.Remove(Item);
end;

procedure TRakInterfacedList.SetCapacity(const Value: Integer);
begin
  FList.Capacity := Value;
end;

procedure TRakInterfacedList.SetCount(const Value: Integer);
begin
  FList.Count := Value;
end;

procedure TRakInterfacedList.SetItems(Index: Integer; const Value: Pointer);
begin
  FList.Items[Index] := Value;
end;

procedure TRakInterfacedList.Sort(Compare: TListSortCompare);
begin
  FList.Sort(Compare);
end;

{ TRakInterfacedClassList }

function TRakInterfacedClassList.AddClass(aClass: TClass): Integer;
begin
  Result := FList.Add(aClass);
end;

function TRakInterfacedClassList.CreateList: TList;
begin
  FList := TClassList.Create;
  Result := FList;
end;

function TRakInterfacedClassList.GetRakClassItems(Index: Integer): TClass;
begin
  Result := FList.Items[Index];
end;

function TRakInterfacedClassList.IndexOfClass(aClass: TClass): Integer;
begin
  Result := FList.IndexOf(aClass);
end;

procedure TRakInterfacedClassList.InsertClass(Index: Integer; aClass: TClass);
begin
  FList.Insert(Index, aClass);
end;

function TRakInterfacedClassList.RemoveClass(aClass: TClass): Integer;
begin
  Result := FList.Remove(aClass);
end;

procedure TRakInterfacedClassList.SetRakClassItems(Index: Integer;
  const Value: TClass);
begin
  FList.Items[Index] := Value;
end;

end.
