unit RakNetz;

{
<unit name="RakNetz" location=".../src">
  <copyright year="2001">
  </copyright>
  <created date="3.5.2001"/>

  <description>
    Es wird die Basisklasse TRakNetNode zur Verfgung gestellt, deren
    Instanzen sich miteinander verbinden knnen und somit ein Netz
    aufspannen knnen.
  </description>

  <publicClasses>
    <publicClass name="TRakNetNode">
      <shortdescription>
        Instanzen von TRakNetNode knnen sich miteinander verbinden
        und somit ein Netz aufspannen, dessen Knoten sich unter
        bestimmten Bedingungen selbst lschen
      </shortdescription>
    </publicClass>
    <publicClass name="TRakGroupNode">
      <shortdescription>
        TRakGroupNode ist von TRakNetNode abgeleitet und verbindet
        Knoten zu einer Gruppe, wobei ein Knoten dieser Gruppe eine
        Fhrungsrolle hat. Der automatische Freigabemechnismus von TRakNetNode
        wurde berschrieben.
      </shortdescription>
    </publicClass>
    <publicClass name="TRakDestroyNode">
      <shortdescription>
        TRakDestroyNode ist von TRakNetNode abgeleitet und lst
        ein Ereignis aus, wenn es sich zerstrt.
      </shortdescription>
    </publicClass>
  </publicClasses>

  <changes>
    <change date="29.9.2001" author="Richard Kaspar">
      Neue Klasse TRakDestroyNode
    </change>
    <change date="7.10.2001" author="Richard Kaspar">
      Schnittstellennderung bei den Methoden CreateDataObject und GetDataObj
      der Klasse TRakNetNode
    </change>
    <change date="5.11.2001" author="Richard Kaspar">
      Bugfix: Klasse TRakNetNode hat sich beim Zerstren nicht von den anderen Knoten getrennt
      Neu: Methode TRakNetNode.DisconnectAll
      Bugfix: TRakGroupNode.HasAnker hat immer genau den falschen Wert zurckgegeben
    </change>
    <change date="17.2.2001" author="Richard Kaspar">
      Bugfix: TRakGroupNode.HasAnker hat false zurckgegeben, wenn MemberCount = 1
    </change>
    <change date="2.6.2001" author="Richard Kaspar">
      Bugfix: TRakGroupNode konnte in einer Endlosschleife landen, wenn DestroyWhenLeaderDisconnects
      true zurck gab.
    </change>
    <change date="2.4.2003" author="Richard Kaspar">
      Neue Methode TRakNetNode.Isolate
    </change>
  </changes>
</unit> TRakNetNode
}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses contnrs,
  miscfunc, RakObject, MultiEvent, RakDestroy;

type
  TRakConnectionID = Int64;

const
  NULL_CONNECTION = low(TRakConnectionID);
type
  {****************************************************************************
  <class name="TRakNetNode" superclass="TRakObject" public="true" defaultproperty="Nodes">
    <description>
      Instanzen von TRakNetNode knnen sich miteinander verbinden
      und somit ein Netz aufspannnen. In dem Netz knnen einzelne Knoten
      zu Ankern gemacht werden. Knoten die keine Anker sind, werden automatisch
      gelscht, wenn sie zu keinem Ankerknoten mehr eine direkte oder indirekte
      Verbindung haben. s.a. Property
      <link class="TRakNetNode" prop="AutoDestroy">AutoDestroy</link><br>
      Eine Instanz von TRakNetNode drfte wenig Sinn machen.<br>
      Eine von TRakNetNode abgeleitete Klassen knnte z.B. ein Ereignis
      auslsen, wenn die Instanz zerstrt wird. Somit kann man Objekte mit
      zerstren, die zwar an dem Knoten hngen, aber nicht von TRakNetNode
      abgeleitet werden knnen, da sie z.B. von TComponent abgeleitet werden
      mssen. Siehe Klasse <link class="TRakDestroyNode">TRakDestroyNode</link>
    </description>

    <variable name="FConnections" type="TObjectList" access="protected" isClassVariable="false">
      <description>
        Wird intern genutzt, um sich zu merken, mit welchen anderen "TRakNodes"
        das Objekt verbunden ist.
      </description>
    </variable>

    <property name="AutoDestroy" type="boolean" access="public" isArray="false">
      <description>
        Gibt an, ob sich die Instanz zestrt, sobald es mit keinem "TRakNetNode"
        mehr verbunden ist, bei dem AutoDestroy false ist.
      </description>
    </property>

    <property name="Nodes" type="TRakNetNode" access="public" isArray="true">
      <description>
        Ermglicht den Zugriff auf alle Knoten, die mit diesem Knoten verbunden
        sind.<br>
        Index = 0..NodesCount-1
      </description>
    </property>

    <property name="NodesCount" type="integer" access="public" isArray="false">
      <description>
        Gibt an, mit wievielen Knoten dieser Knoten verbunden ist.
      </description>
    </property>


    <method name="CheckAutoDestroy">
      <shortdescription>
        CheckAutoDestroy prft, ob die Instanz zerstrt werden kann, und
        zerstrt diese dementsprechend.<br>
        Der Knoten wird zerstrt, wenn kein Status gesetzt ist und wenn er
        mit keinem Knoten mehr verbunden ist, bei dem AutoDestroy = false.
      </shortdescription>
    </method>

    <method name="CreateNow">
      <shortdescription>
        Konstruktor
      </shortdescription>
    </method>

    <method name="DestroyNow">
      <shortdescription>
        Destruktor
      </shortdescription>
    </method>

    <method name="GetAutoDestroy">
      <shortdescription>
        read-Methode des Properties AutoDestroy.
      </shortdescription>
    </method>

    <method name="GetConnectionIdxOfNode(Node: TRakNetNode)">
      <shortdescription>
        Ermittelt den Index damit die Verbindungsdaten aus FConnections
        zu einem bestimmten Knoten besorgt werden knnen.<br>
        Gibt -1 zurck, wenn nicht mit Node verbunden.
        Wenn es mehrere Verindungen zu dem Knoten gibt, dann wird die Verbindung
        zurckgeliefert, die am lngsten besteht.
      </shortdescription>
    </method>

    <method name="GetNodeOfConnection(Connection: TRakConnectionID)">
      <shortdescription>
        Ermittelt den Knoten, der ber eine bestimmte Verbindung angehngt ist.<br>
        Da ein Knoten mehrfach mit demselben Knoten verbunden werden kann,
        kann bei verschiedenen "Connection" der selbe Knoten zurckgegeben werden.<br>
        "Connection" ist der Wert, der von der Funktion "Connect" zurckgegeben wird.
      </shortdescription>
    </method>

    <method name="HasAnker">
      <shortdescription>
        HasAnker stellt fest, ob dieser Knoten mit mind. einem anderen Knoten
        direkt oder indirekt verbunden ist, bei dem Autodestroy = false.
        HasAnker liefert auch dann true zurck, wenn das Autodestroy des Knoten
        selbst true ist.
      </shortdescription>
    </method>

    <method name="PlugOn(Node: TRakNetNode; Connection: TRakConnectionID)">
      <shortdescription>
        PlugOn wird von einem anderen Knoten (Node) aufgerufen, wenn sich der
        andere mit diesem verbindet. Connection ist der Verbindungsschlssel
        zwischen den beiden Knoten.<br>
        PlugOn sorgt dafr, dass sich die neue Verbindung gemerkt wird.<br>
        Wenn man eine Verbindung zwischen zwei Knoten erzeugen mchte, dann
        sollte man nicht PlugOn, sondern Connect aufrufen.
      </shortdescription>
    </method>

    <method name="PlugOff(Idx: integer)">
      <shortdescription>
        PlugOff sorgt dafr, das alle Verbindungungen zu einem bestimmten Knoten
        vergessen werden. Idx gibt den Index an, mit dem die Verbindungen in
        FConnections gespeichert sind.<br>
        Wenn man die Verbindungen zu einem Knoten lsen mchte, dann sollte man
        nicht PlugOff, sondern Disconnect aufrufen.
      </shortdescription>
    </method>

    <method name="PlugOff(Node: TRakNetNode)">
      <shortdescription>
        PlugOff sorgt dafr, dass alle Verbindungungen zu einem bestimmten Knoten
        vergessen werden.<br>
        Wenn man die Verbindungen zu einem Knoten lsen mchte, dann sollte man
        nicht PlugOff, sondern Disconnect aufrufen.
      </shortdescription>
    </method>

    <method name="PlugOff(Connection: TRakConnectionID)">
      <shortdescription>
        PlugOff sorgt dafr, dass eine bestimmte Verbindungung zu einem Knoten
        vergessen wird. Connection ist der Wert, den man von der Funktion
        Connect zurck bekommt.<br>
        Wenn man eine Verbindung zu einem Knoten lsen mchte, dann sollte man
        nicht PlugOff, sondern Disconnect aufrufen.
      </shortdescription>
    </method>

    <method name="CreateDataObject(Connection: TRakConnectionID)">
      <shortdescription>
        CreateDataObject wird aufgerufen, wenn fr eine Verbindung das
        erste mal das Datanobjekt angefordert wird (Funktion GetDataObj).<br>
        Abgeleitete Klassen, die zu Verbindungen weitere Daten Speichern
        mchten, sollten hier eine Instanz des Datenobjektes erzeugen und
        zurckgeben.
      </shortdescription>
    </method>

    <method name="GetDataObj(Connection: TRakConnectionID)">
      <shortdescription>
        Mit GetDataObj bekommt man das Datenobjekt, in dem weitere
        Informationen zu der Verbindung gespeichert sind.<br>
        Klassen, die diese Funktionalitt nutzen mchten, mssen
        die Methode CreateDataObject berschreiben.
      </shortdescription>
    </method>

    <method name="Connect(Node: TRakNetNode)">
      <shortdescription>
        Connect sorgt dafr, dass sich das Objekt mit "Node" verbindet.
        Ein Objekt kann mehrfach mit dem selben Objekt verbunden werden.<br>
        Connect gibt einen Wert vom Typ TRakConnectionID zurck, welcher
        der eindeutige Schlssel der Verbindung ist und von einigen anderen
        Methoden als Eingangsparameter erwartet wird.
      </shortdescription>
    </method>

    <method name="Disconnect(Node: TRakNetNode)">
      <shortdescription>
        Disconnect lst alle Verbindungen zu einem anderen Knoten.
      </shortdescription>
    </method>

    <method name="Disconnect(Connection: TRakConnectionID)">
      <shortdescription>
        Disconnect lst eine bestimmte Verbindung zu einem anderen Knoten.<br>
        Connection ist der Wert, den man von der Funktion Connect zurckbekommt
      </shortdescription>
    </method>

    <method name="IsConnected(Node: TRakNetNode)">
      <shortdescription>
        Prft, ob dieser Knoten mit einem bestimmten anderen Knoten verbunden ist.
      </shortdescription>
    </method>

    <method name="NodeConnectionsCount(Node: TRakNetNode): integer">
      <shortdescription>
        Gibt zurck, wie oft dieser Knoten mit einem bestimmten anderen
        Knoten verbunden ist.
      </shortdescription>
    </method>

    <method name="NodeConnections(Node: TRakNetNode; Idx: integer): TRakConnectionID">
      <shortdescription>
        Gibt die Id einer bestimmten Verbindung zu einem bestimmten Knoten zurck.
        Der so erhaltene Wert kann z.B. an Disconnect bergeben werden.
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakNetNode = class(TRakObject)
  private
    function GetNodesCount: integer;
    function GetNodes(Idx: integer): TRakNetNode;
    function GetConnections(Idx: integer): TRakConnectionID;
    function GetConnectionsCount: integer;
  protected
    FConnections: TObjectList;

    procedure CheckAutoDestroy; virtual;

    procedure CreateNow; override;
    procedure DestroyNow; override;

    function GetAutoDestroy: boolean; virtual;
    function GetConnectionIdxOfNode(Node: TRakNetNode): integer;
    function GetNodeOfConnection(Connection: TRakConnectionID): TRakNetNode;

    function HasAnker: boolean; virtual;

    procedure PlugOn(Node: TRakNetNode; Connection: TRakConnectionID); virtual;
    procedure PlugOff(Idx: integer); overload; virtual;
    procedure PlugOff(Node: TRakNetNode); overload; virtual;
    procedure PlugOff(Connection: TRakConnectionID); overload; virtual;

    function CreateDataObject(Connection: TRakConnectionID): IRakOwned; virtual;
    function GetDataObj(Connection: TRakConnectionID): TObject; virtual;
    function Connect(Node: TRakNetNode; DataObj: IRakOwned): TRakConnectionID; overload; virtual;
  public
    function Connect(Node: TRakNetNode): TRakConnectionID; overload; virtual;
    function Disconnect(Node: TRakNetNode): boolean; overload; virtual;
    function Disconnect(Connection: TRakConnectionID): boolean; overload; virtual;
    procedure DisconnectAll; virtual;
    // Isolate trennt sich s wie DisconnectAll von allen Knoten,
    // aber der Knoten wird auch dann nicht gelscht, wenn AutoDestroy = true
    procedure Isolate; virtual;
    function IsConnected(Node: TRakNetNode): boolean; virtual;
    function NodeConnectionsCount(Node: TRakNetNode): integer; virtual;
    function NodeConnections(Node: TRakNetNode; Idx: integer): TRakConnectionID; virtual;

    property AutoDestroy: boolean read GetAutoDestroy;
    property ConnectionsCount: integer read GetConnectionsCount;
    property Connections[Idx: integer]: TRakConnectionID read GetConnections;
    property NodesCount: integer read GetNodesCount;
    property Nodes[Idx: integer]: TRakNetNode read GetNodes; default;
  end;

  {****************************************************************************
  <class name="TRakGroupNode" superclass="TRakNetNode" public="true">
    <description>
      TRakGroupNode gruppiert Knoten. TRakGroupNode selbst gehrt der Gruppe nicht an,
      sondern ist nur mit jedem Mitglied der Gruppe verbunden. Ein Knoten
      kann einer Gruppe nicht mehrfach angehren. Die Gruppe besteht aus
      einem Fhrer und normalen Gruppenmitgliedern. Mit dem Property Members
      kann man auf die normalen Gruppenmitglieder ohne den Fhrer zugreifen.
      Mit dem geerbten Property Nodes kann man auf alle Gruppenmitglieder
      zugreifen.
      TRakGroupNode zerstrt sich selbst, wenn verschiedene Bedingungen eintreten:
      1. DestroyWhenLeaderDisconnects = true und der Knoten Leader lst seine
          Verbindung zu TRakGroupNode
          Hinweis: Wenn der Knoten Leader seine Verbindung lst und
                  DestroyWhenLeaderDisconnects = false, dann steigt das erste
                  normale Mitglied (Members[0]) zum Fhrer auf.
      2. NeedsMembers = true und MembersCount fllt auf den Wert 0
      3. Das geerbte Property NodesCount fllt auf den Wert 0
    </description>

    <method name="GetAutoDestroy">
      <shortdescription>
        GetAutoDestroy gibt true zurck und sorgt damit dafr, dass
        der automatische Lschmechanismus luft
      </shortdescription>
    </method>
    <method name="HasAnker">
      <shortdescription>
        Das berschreiben von HasAnker sorgt dafr, dass der automatische
        Lschmechanismus richtig funktioniert.
      </shortdescription>
    </method>
    <method name="LeaderLost">
      <shortdescription>
        LeaderLost wird aufgerufen, wenn DestroyWhenLeaderDisconnects = true
        und die Verbindung zum Knoten Leader aufgelst werden soll.
        LeaderLost gibt die Instanz frei.
      </shortdescription>
    </method>
    <method name="DestroyWhenLeaderDisconnects">
      <shortdescription>
        DestroyWhenLeaderDisconnects legt fest, ob sich die Instanz selbst zerstrt,
        wenn die Verknpfung zwischen dem Knoten "Leader" und der Gruppe aufgelst wird.
        DestroyWhenLeaderDisconnects gibt false zurck.
      </shortdescription>
    </method>
    <method name="PlugOff(Idx: integer)">
      <shortdescription>
        Das berschreiben von PlugOff sorgt dafr, dass der automatische
        Lschmechanismus richtig funktioniert
      </shortdescription>
    </method>
    <method name="NeedsMembers">
      <shortdescription>
        NeedsMembers legt fest, ob sich die Instanz selbst zerstrt,
        wenn der Wert von MembersCount auf 0 fllt.
        NeedsMembers gibt false zurck.
      </shortdescription>
    </method>
    <method name="Connect(Node: TRakNetNode)">
      <shortdescription>
        Mit Connect kann man einen Knoten der Gruppe hinzufgen.
        Wenn es der erste Knoten ist, dann wird dieser automatisch zum "Leader"
      </shortdescription>
    </method>
    <method name="IdxOfMember(Member: TRakNetNode)">
      <shortdescription>
        IdxOfMember gibt den Idx-Wert eines Members zurck, den man
        an die Methode MoveMember oder das Property Members bergeben kann.
        Rckgabe = -1, wenn "Member" kein Member ist.
        Hinweis: IdxOfMember(Leader) gibt auch -1 zurck, da Leader kein
        normales Mitglied ist.
      </shortdescription>
    </method>
    <method name="MoveMember(FromIdx, ToIdx: integer)">
      <shortdescription>
        ndert die Reihenfolge der normalen Mitglieder
      </shortdescription>
    </method>

    <property name="Leader" type="TRakNetNode" access="public" isArray="false">
      <description>
        Leader ist der Anfhrer der Gruppe. Wenn Leader <> nil, dann Leader = Nodes[0]
      </description>
    </property>
    <property name="MembersCount" type="integer" access="public" isArray="false" readOnly="true">
      <description>
        MembersCount gibt die Anzahl der "normalen" Mitglieder der Gruppe
        zurck. MembersCount ist meistens gleich NodesCount-1.
      </description>
    </property>
    <property name="Members" type="TRakNetNode" access="public" isArray="true" Range="0..MembersCount-1">
      <description>
        Mit Members kann auf die normalen Mitglieder der Gruppe zugegriffen
        werden. Mit Members kann nicht auf den "Leader" zugegriffen werden.
        Mit dem geerbten Property Nodes kann auf alle Mitglieder
        zugegriffen werden.
      </description>
    </property>
    <property name="Komposition" type="TRakKomposition " access="public" isArray="false">
      <description>
        Mit Komposition kann man das Objekt so fest an andere Objekte binden,
        dass jedes dieser Objekte freigegeben wird, sobald eines davon freigegben
        wird.
      </description>
    </property>
  </class>
  ****************************************************************************}
  TRakGroupNode = class(TRakNetNode) //TRakObject
  private
    FKomposition: IRakKomposition;
    procedure SetLeader(const Value: TRakNetNode);
    function GetLeader: TRakNetNode;
    function GetMembers(Idx: integer): TRakNetNode;
    function GetMembersCount: integer;
    procedure SetKomposition(const Value: IRakKomposition);
  protected
    //TRakObject
    function GetKomposition: IRakKomposition; override;
    //TRakNetNode
    procedure PlugOff(Idx: integer); override;
    function GetAutoDestroy: boolean; override;
    function HasAnker: boolean; override;

    procedure LeaderLost; virtual;
    function DestroyWhenLeaderDisconnects: boolean; virtual;
    function NeedsMembers: boolean; virtual;
  public
    function Connect(Node: TRakNetNode): TRakConnectionID; override;

    function IdxOfMember(Member: TRakNetNode): integer; virtual;
    procedure MoveMember(FromIdx, ToIdx: integer); overload; virtual;

    property Leader: TRakNetNode read GetLeader write SetLeader;

    property MembersCount: integer read GetMembersCount;
    property Members[Idx: integer]: TRakNetNode read GetMembers;

    property Komposition: IRakKomposition read GetKomposition write SetKomposition;
  end;

  {****************************************************************************
  <class name="TRakDestroyNode" superclass="TRakNetNode" public="true">
    <description>
      TRakDestroyNode erweitert TRakNetNode um das MultiEvent OnDestroy.
    </description>

    <property name="OnDestroy" type="TRakNotifyEventVerwalter" access="public" isArray="false">
      <description>
        Mit OnDestroy knnen sich beliebig viele andere Objekte ein Ereignis
        schicken lassen, wenn sich das Objekt zerstrt, um eventuell
        Referenzen auf das Objekt zu entsorgen, damit diese nicht ins Nirvana zeigen.
      </description>
    </property>
  </class>
  ****************************************************************************}
  TRakDestroyNode = class(TRakNetNode)
  private
    FOnDestroy: TRakNotifyEventVerwalter;
  protected
    procedure CreateNow; override;
    procedure DestroyNow; override;
    procedure BeforeDestroyEvent; override;
  public
    property OnDestroy: TRakNotifyEventVerwalter read FOnDestroy;
  end;

  {****************************************************************************
  <class name="TRakKompositionNode" superclass="TRakNetNode" public="true">
    <description>
      TRakKompositionNode erweitert TRakNetNode so, dass es sich in eine
      IRakKomposition einhngen kann. Mit einer IRakKomposition werden
      Objekte so fest miteinander verbunden, dass alle Objekte freigegben
      werden, sobald eines der Objekte freigegeben wird.
    </description>

    <property name="Komposition" type="IRakKomposition" access="public" isArray="false">
      <description>
        Komposition legt fest, mit welchen anderen Objekten die Instanz von
        TRakKompositionNode fest verschweit ist. Wird ein Objekt der Komposition
        zerstrt, dann werden automatisch auch alle anderen Objekte
        der Komposition zerstrt.
      </description>
    </property>
  </class>
  ****************************************************************************}
  TRakKompositionNode = class(TRakNetNode) // TRakObject
  private
    FKomposition: IRakKomposition;
  protected
    function GetKomposition: IRakKomposition; override;
    procedure SetKomposition(const Value: IRakKomposition); virtual;

    procedure PlugOffKomposition; virtual;
    procedure PlugOnKomposition; virtual;
  public
    property Komposition: IRakKomposition read GetKomposition write SetKomposition;
  end;

implementation

var
  NextConnectionID: TRakConnectionID = low(TRakConnectionID) + 1;

function GetFreeConnectionID: TRakConnectionID;
begin
  Result := NextConnectionID;
  inc(NextConnectionID);
end;

type
  TRakStatusAnkerSearching = class(TRakBaseStatus);
  TRakStatusDisconnecting = class(TRakBaseStatus);
  TRakStatusLeaderLost = class(TRakBaseStatus);

  TConnection = class(TRakOwnedSingleOwner)
  private
    function GetData: IRakOwned;
    procedure SetData(Value: IRakOwned);
  public
    Connection: TRakConnectionID;

    property Data: IRakOwned read GetData write SetData;
  end;

  TConnectionList = class(TRakOwner)
  private
    function GetItems(Idx: integer): TConnection;
  public
    function ItemByConnection(Connection: TRakConnectionID): TConnection;

    property Items[Idx: integer]: TConnection read GetItems; default;
  end;

  TConnectedNode = class
  private
    FDatas: TConnectionList;
  public
    Node: TRakNetNode;

    constructor Create;
    destructor Destroy; override;

    property Datas: TConnectionList read FDatas;
  end;

{ TRakNetNode }

procedure TRakNetNode.CheckAutoDestroy;
begin
  if Stati.Count = 0 then
  begin
    if not HasAnker then
      Free;
  end;
end;

function TRakNetNode.Connect(Node: TRakNetNode): TRakConnectionID;
var
  DataObj: IRakOwned;
begin
  DataObj := nil;
  Result := Connect(Node, DataObj);
end;

function TRakNetNode.Connect(Node: TRakNetNode;
  DataObj: IRakOwned): TRakConnectionID;
var
  Connection: TConnectedNode;
  Idx: integer;
  Con: TConnection;
begin
  Result := low(TRakConnectionID);
  if Node <> nil then
  begin
    Idx := GetConnectionIdxOfNode(Node);
    if Idx = -1 then
    begin
      Connection := TConnectedNode.Create;
      FConnections.Add(Connection);
      Connection.Node := Node;
    end
    else
      Connection := TConnectedNode(FConnections[Idx]);

    Con := TConnection.Create;
    Con.ISetOwner(Connection.Datas);
    Con.Connection := GetFreeConnectionID;
    Con.Data := DataObj;

    Result := Con.Connection;
    Node.PlugOn(self, Result);
  end;
end;

function TRakNetNode.CreateDataObject(
  Connection: TRakConnectionID): IRakOwned;
begin
  Result := nil;
end;

procedure TRakNetNode.CreateNow;
begin
  inherited;

   FConnections := TObjectList.Create;
end;

procedure TRakNetNode.DestroyNow;
begin
  DisconnectAll;
  FreeTObject(FConnections);
  inherited;
end;

function TRakNetNode.Disconnect(Connection: TRakConnectionID): boolean;
var
  Node: TRakNetNode;
begin
  Stati.Add(TRakStatusDisconnecting);
  try
    Node := GetNodeOfConnection(Connection);
    PlugOff(Connection);
     Node.PlugOff(Connection);
  finally
    Stati.Delete(TRakStatusDisconnecting);
  end;
  CheckAutoDestroy;

  Result := true;
end;

function TRakNetNode.Disconnect(Node: TRakNetNode): boolean;
begin
  Stati.Add(TRakStatusDisconnecting);
  try
    if Node <> nil then
    begin
      PlugOff(Node);
      Node.PlugOff(self);
    end;
  finally
    Stati.Delete(TRakStatusDisconnecting);
  end;
  CheckAutoDestroy;

  Result := true;
end;

function TRakNetNode.GetAutoDestroy: boolean;
begin
  Result := false;
end;

function TRakNetNode.GetConnectionIdxOfNode(Node: TRakNetNode): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to FConnections.Count - 1 do
  begin
    if TConnectedNode(FConnections[i]).Node = Node then
    begin
      Result := i;
      break;
    end;
  end;
end;

function TRakNetNode.GetConnections(Idx: integer): TRakConnectionID;
var
  i: integer;
  Node: TRakNetNode;
  Anz: integer;
begin
  Result := NULL_CONNECTION;
  Anz := 0;
  for i := 0 to NodesCount - 1 do
  begin
    Node := Nodes[i];
    Anz := Anz + NodeConnectionsCount(Node);
    if Anz > Idx then
    begin
      Result := NodeConnections(Node, Idx - Anz + NodeConnectionsCount(Node));
      break;
    end;
  end;
end;

function TRakNetNode.GetConnectionsCount: integer;
var
  i: integer;
begin
  Result := 0;
  for i := 0 to NodesCount - 1 do
    Result := Result + NodeConnectionsCount(Nodes[i]);
end;

function TRakNetNode.GetNodesCount: integer;
begin
  Result := FConnections.Count;
end;

function TRakNetNode.GetDataObj(Connection: TRakConnectionID): TObject;
var
  i: integer;
  ConNode: TConnectedNode;
  Con: TConnection;
  Owned: IRakOwned;
begin
  Result := nil;
  for i := 0 to FConnections.Count - 1 do
  begin
    ConNode := TConnectedNode(FConnections[i]);
    Con := ConNode.Datas.ItemByConnection(Connection);
    if Con <> nil then
    begin
      Owned := Con.Data;
      if Owned = nil then
        Con.Data := CreateDataObject(Connection);
      if Con.Data <> nil then
        Result := Con.Data.IGetObject;
      break;
    end;
  end;
end;

function TRakNetNode.GetNodeOfConnection(
  Connection: TRakConnectionID): TRakNetNode;
var
  i: integer;
  ConNode: TConnectedNode;
begin
  Result := nil;
  for i := 0 to FConnections.Count - 1 do
  begin
    ConNode := TConnectedNode(FConnections[i]);
    if ConNode.Datas.ItemByConnection(Connection) <> nil then
    begin
      Result := ConNode.Node;
      break;
    end;
  end;
end;

function TRakNetNode.GetNodes(Idx: integer): TRakNetNode;
begin
  Result := TConnectedNode(FConnections[Idx]).Node;
end;

function TRakNetNode.HasAnker: boolean;
var
  i: integer;
begin
  Stati.Add(TRakStatusAnkerSearching);
  try
    Result := not AutoDestroy;
    if not Result then
    begin
      i := 0;
      while (i < NodesCount) and not Result do
      begin
        if not Nodes[i].Stati.Exists(TRakStatusAnkerSearching) then
          Result := Nodes[i].HasAnker;
        inc(i);
      end;
    end;
  finally
    Stati.Delete(TRakStatusAnkerSearching);
  end;
end;

function TRakNetNode.IsConnected(Node: TRakNetNode): boolean;
begin
  Result := GetConnectionIdxOfNode(Node) >= 0;
end;

function TRakNetNode.NodeConnections(Node: TRakNetNode;
  Idx: integer): TRakConnectionID;
var
  Idx2: integer;
  Con: TConnection;
  ConNode: TConnectedNode;
begin
  Result := NULL_CONNECTION;
  Idx2 := GetConnectionIdxOfNode(Node);
  if Idx2 >= 0 then
  begin
    ConNode := TConnectedNode(FConnections[Idx2]);
    if (0 <= Idx) and (Idx < ConNode.Datas.Count) then
    begin
      Con := TConnection(ConNode.Datas[Idx]);
      Result := Con.Connection;
    end;
  end;
end;

function TRakNetNode.NodeConnectionsCount(Node: TRakNetNode): integer;
var
  Idx: integer;
begin
  Result := 0;
  Idx := GetConnectionIdxOfNode(Node);
  if Idx >= 0 then
    Result := TConnectedNode(FConnections[Idx]).Datas.Count;
end;

procedure TRakNetNode.PlugOff(Node: TRakNetNode);
begin
  PlugOff(GetConnectionIdxOfNode(Node));
end;

procedure TRakNetNode.PlugOff(Connection: TRakConnectionID);
var
  i: integer;
  ConNode: TConnectedNode;
  Con: TConnection;
begin
  for i := 0 to FConnections.Count - 1 do
  begin
    ConNode := TConnectedNode(FConnections[i]);
    Con := ConNode.Datas.ItemByConnection(Connection);
    if Con <> nil then
    begin
      Con.Free;
      if ConNode.Datas.Count = 0 then
        PlugOff(i);
      //Achtung:  Die Instanz knnte jetzt zerstrt sein. Ab hier darf
      //           also nicht mehr auf Instanzvariablen zugegriffen werden!
      break;
    end;
  end;
end;

procedure TRakNetNode.PlugOff(Idx: integer);
begin
  if (Idx >= 0) and (Idx < FConnections.Count) then
  begin
     FConnections.Delete(Idx);
    CheckAutoDestroy;
  end;
end;

procedure TRakNetNode.PlugOn(Node: TRakNetNode; Connection: TRakConnectionID);
var
  Idx: integer;
  ConNode: TConnectedNode;
  Con: TConnection;
begin
  Idx := GetConnectionIdxOfNode(Node);
  if Idx = -1 then
  begin
    ConNode := TConnectedNode.Create;
    FConnections.Add(ConNode);
    ConNode.Node := Node;
  end
  else
    ConNode := TConnectedNode(FConnections[Idx]);

  Con := TConnection.Create;
  Con.ISetOwner(ConNode.Datas);
  Con.Connection := Connection;
end;


procedure TRakNetNode.DisconnectAll;
begin
  Stati.Add(TRakStatusDisconnecting);
  try
    while NodesCount > 0 do
      Disconnect(Nodes[0]);
  finally
    Stati.Delete(TRakStatusDisconnecting);
  end;
  CheckAutoDestroy;
end;

procedure TRakNetNode.Isolate;
begin
  Stati.Add(TRakStatusDisconnecting);
  try
    while NodesCount > 0 do
      Disconnect(Nodes[0]);
  finally
    Stati.Delete(TRakStatusDisconnecting);
  end;
end;

{ TConnectedNode }

constructor TConnectedNode.Create;
begin
  FDatas := TConnectionList.Create;
end;

destructor TConnectedNode.Destroy;
begin
  FDatas.Free;

  inherited;
end;

{ TConnection }

function TConnection.GetData: IRakOwned;
begin
  Result := IOwned;
end;


procedure TConnection.SetData(Value: IRakOwned);
begin
  IOwned := Value;
end;

{ TConnectionList }

function TConnectionList.GetItems(Idx: integer): TConnection;
begin
  Result := TConnection(IGetItem(Idx).IGetObject);
end;

function TConnectionList.ItemByConnection(
  Connection: TRakConnectionID): TConnection;
var
  i: integer;
begin
  Result := nil;

  for i := 0 to Count - 1 do
  begin
    if TConnection(Items[i]).Connection = Connection then
    begin
      Result := TConnection(Items[i]);
      break;
    end;
  end;
end;

{ TRakDestroyNode }

procedure TRakDestroyNode.BeforeDestroyEvent;
begin
  OnDestroy.FireEvent(self);
end;

procedure TRakDestroyNode.CreateNow;
begin
  inherited;

  FOnDestroy := TRakNotifyEventVerwalter.Create;
end;


procedure TRakDestroyNode.DestroyNow;
begin
  FreeTObject(FOnDestroy);

  inherited;
end;

{ TRakGroupNode }


function TRakGroupNode.Connect(Node: TRakNetNode): TRakConnectionID;
begin
  if IsConnected(Node) then
    Result := NodeConnections(Node, 0)
  else
    Result := inherited Connect(Node);
end;

function TRakGroupNode.DestroyWhenLeaderDisconnects: boolean;
begin
  Result := false;
end;

function TRakGroupNode.GetAutoDestroy: boolean;
begin
  Result := true;
end;

function TRakGroupNode.GetKomposition: IRakKomposition;
begin
  Result := FKomposition;
end;

function TRakGroupNode.GetLeader: TRakNetNode;
begin
  Result := nil;
  if NodesCount > 0 then
    Result := Nodes[0];
end;

function TRakGroupNode.GetMembers(Idx: integer): TRakNetNode;
begin
  Result := Nodes[Idx+1];
end;

function TRakGroupNode.GetMembersCount: integer;
begin
  Result := NodesCount - 1;
  if Result < 0 then
    Result := 0;
end;

function TRakGroupNode.HasAnker: boolean;
begin
  Result := (MembersCount > 0)
    or (not NeedsMembers and (NodesCount > 0));
end;

function TRakGroupNode.IdxOfMember(Member: TRakNetNode): integer;
begin
  Result := GetConnectionIdxOfNode(Member);
  if Result = 0 then
    Result := -1;
end;

procedure TRakGroupNode.LeaderLost;
begin
  if Stati.Count = 0 then
    Free
  else
    DisconnectAll;
end;

procedure TRakGroupNode.MoveMember(FromIdx, ToIdx: integer);
var
  iMax: integer;
begin
  iMax := MembersCount-1;
  if IsBetween(FromIdx, 0, iMax) and IsBetween(ToIdx, 0, iMax) then
    FConnections.Move(FromIdx + 1, ToIdx + 1);
end;

function TRakGroupNode.NeedsMembers: boolean;
begin
  Result := false;
end;

procedure TRakGroupNode.PlugOff(Idx: integer);
begin
  if (Idx = 0) and DestroyWhenLeaderDisconnects then
  begin
    Stati.Add(TRakStatusLeaderLost);
    try
      inherited;
    finally
      Stati.Delete(TRakStatusLeaderLost);
    end;
    LeaderLost;
  end
  else
    inherited;
end;

procedure TRakGroupNode.SetKomposition(const Value: IRakKomposition);
begin
  if FKomposition <> nil then
    FKomposition.DeleteMember(self);

  FKomposition := Value;

  if FKomposition <> nil then
    FKomposition.AddMember(self);
end;

procedure TRakGroupNode.SetLeader(const Value: TRakNetNode);
var
  Idx: integer;
begin
  if (Value <> nil) and (Leader <> Value) then
  begin
    Idx := GetConnectionIdxOfNode(Value);
    if Idx = -1 then
    begin
      Connect(Value);
      Idx := GetConnectionIdxOfNode(Value)
    end;
    FConnections.Move(Idx, 0);
  end;
end;


{ TRakKompositionNode }

function TRakKompositionNode.GetKomposition: IRakKomposition;
begin
  Result := FKomposition;
end;

procedure TRakKompositionNode.PlugOffKomposition;
begin
  if FKomposition <> nil then
    FKomposition.DeleteMember(self);
end;

procedure TRakKompositionNode.PlugOnKomposition;
begin
  if FKomposition <> nil then
    FKomposition.AddMember(self);
end;

procedure TRakKompositionNode.SetKomposition(const Value: IRakKomposition);
begin
  if Value <> FKomposition then
  begin
    PlugOffKomposition;

    FKomposition := Value;

    PlugOnKomposition;
  end;
end;

end.
