unit RakObject;
{
<unit name="RakObject" location=".../src">
  <copyright year="2001">
  </copyright>
  <created date="3.5.2001"/>

  <description>
    Es wird die Basisklasse TRakObject zur Verfgung gestellt, die
    Stati verwalten kann.
  </description>

  <publicClasses>
    <publicClass name="TRakObject">
      <shortdescription>
        Eine Basisklasse, die Zustaende haben kann.
        Diese Klasse kann anstelle von TObject verwendet werden, wenn man
        Zustaende eines Objektes speichern moechte.
      </shortdescription>
    </publicClass>
    <publicClass name="TRakKompositionObject">
      <shortdescription>
        TRakKompositionObject vervollstndigt das TRakKompositions-Konzept,
        das bereits in der Klasse TRakObject teilweise untersttzt wird.
      </shortdescription>
    </publicClass>
    <publicClass name="TRakStati">
      <shortdescription>
        Eine Klasse zum Verwalten von Stati
      </shortdescription>
    </publicClass>
    <publicClass name="TRakBaseStatus">
      <shortdescription>
        Basisklasse aller Statusklassen
      </shortdescription>
    </publicClass>
    <publicClass name="TRakStatusDestroying">
      <shortdescription>
        Status, der angibt, dass sich das Objekt gerade zerstrt
      </shortdescription>
    </publicClass>
    <publicClass name="TRakStatusCreating">
      <shortdescription>
        Status, der angibt, dass sich das Objekt gerade erzeugt
      </shortdescription>
    </publicClass>
  </publicClasses>

  <changes>
    <change date="29.9.2001" author="Richard Kaspar">
      Neue Methoden BeforeDestroyEvent und AfterDestroyEvent
      in der Klasse TRakObject
      Neue Funktion FreeTRakObject
    </change>
    <change date="7.10.2001" author="Richard Kaspar">
      Neue Methoden GetKomposition in der Klasse TRakObject
      Neue Klasse TRakKompositionObject
    </change>
    <change date="4.12.2001" author="Richard Kaspar">
      Bei TRakObject.Destroy das Entfernen des Status TRakStatusDestroying rausgenommen
    </change>
  </changes>
</unit>
}
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses miscfunc, RakDestroy;

type
  {****************************************************************************
  <classof name="TRakStatus" of="TRakBaseStatus" public="true">
    <description>
      TRakStatus ist der Typ fr einen Status.
      Ein Status wird zwar von einer von TRakBaseStatus abgeleiteten Klasse
      reprsentiert, es wird aber nie eine Instanz einer solchen Klasse erzeugt.
      Statt dessen wird mit der Klasse selbst gearbeitet.
      TRakStatus wird fr das Property
      <link class="TRakStati" prop="Stati">Stati</link> der Klasse
      <link class="TRakStati">TRakStati</link> bentigt.
    </description>
  </classof>
  ****************************************************************************}
  TRakStatus = class of TRakBaseStatus;

  {****************************************************************************
  <class name="TRakBaseStatus" superclass="TObject" public="true">
    <description>
      TRakBaseStatus bildet die Basisklassen aller Status-Klassen, die von
      TRakStati verwaltet werden. Wenn man einen neuen Status erzeugen
      will, dann muss man direkt oder indirekt eine neue Klasse von
      TRakBaseStatus ableiten.
      Beispiele dafr sind die Klassen
      <link class="TRakStatusDestroying">TRakStatusDestroying</link> und
      <link class="TRakStatusCreating">TRakStatusCreating</link>
    </description>
  </class>
  ****************************************************************************}
  TRakBaseStatus = class
  end;

  {****************************************************************************
  <array name="TRakStatiArr" of="TRakStatus" public="false">
    <description>
      TRakStatiArr wird intern von <link class="TRakStati">TRakStati</link> genutzt.
    </description>
  </array>
  ****************************************************************************}
  TRakStatiArr = array of TRakStatus;

  {****************************************************************************
  <class name="TRakStatusDestroying" superclass="TRakBaseStatus" public="true">
    <description>
      TRakStatusDestroying ist der Zustand, der anzeigt, dass sich das Objekt
      gerade zerstrt. TRakStatusDestroying wird von der Klasse
      <link class="TRakObject">TRakObject</link> genutzt.
    </description>
  </class>
  ****************************************************************************}
  TRakStatusDestroying = class(TRakBaseStatus);

  {****************************************************************************
  <class name="TRakStatusCreating" superclass="TRakBaseStatus" public="true">
    <description>
      TRakStatusCreating ist der Zustand, der anzeigt, dass sich das Objekt
      gerade erzeugt. TRakStatusCreating wird von der Klasse
      <link class="TRakObject">TRakObject</link> genutzt.
    </description>
  </class>
  ****************************************************************************}
  TRakStatusCreating = class(TRakBaseStatus);

  {****************************************************************************
  <class name="TRakStati" superclass="TObject" public="true" defaultproperty="Stati">
    <description>
      TRakStati ist ein Container fr Stati.
      TRakStati dient dazu, dass ein Objekt Flags setzen kann, damit
      bestimmte Prezeduren anders reagieren knnen, wenn das Flag gesetzt ist.
      Man kann den selben Status (Flag) mehrfach setzen. Erst wenn
      der Status das letztemal weggenommen wurde, wird das Flag wirklich
      entfernt.
      Da Klassen als Stati verwendet werden kann man die Vererbung zum Gruppieren
      von Stati einsetzen.
      Beispiel:
      TStatus1 = class(TRakBaseStatus);
      TStatusGruppe = class(TRakBaseStatus);
      TStatus2 = class(TStatusGruppe);
      TStatus3 = class(TStatusGruppe);
      Wenn man jetzt wissen will, ob einer der Stati TStatus2 oder TStatus3
      gesetzt ist, dann ruft man Instanz.ExistsInherited(TStatusGruppe) auf.
      Ob es dafr einen sinnvollen Einsatz gibt steht in den Sternen :-)

      Ein neuer Status kann durch das Ableiten von der Klasse
      <link class="TRakBaseStatus">TRakBaseStatus</link> erzeugt werden.
    </description>

    <variable name="FStati" type="TRakStatiArr" access="protected" isClassVariable="false">
      <description>
        Wird intern genutzt, um sich die gesetzten Stati zu merken
      </description>
    </variable>

    <property name="Count" type="integer" access="public" isArray="false">
      <description>
        Gibt die Anzahl der gesetzten Stati zurck. Sollte ein Status
        mehrfach gesetzt sein, so wird er trotzdem nur einmal gezhlt.
        Beispiel:
          Add(Status1);
          Add(Status2);
          Add(Status1);
          Anz := Count;
          // In Anz steht jetzt 2
      </description>
    </property>

    <property name="Stati" type="TRakStatus" access="public" isArray="true" range="0..Count-1">
      <description>
        Stati hat soviele Elemente, wie das Property Count angibt.
        ber Stati kann man feststellen, welche Stati gesetzt sind.
      </description>
    </property>

    <method name="GetCount">
      <shortdescription>
        read-Methode des Properties Count
      </shortdescription>
    </method>

    <method name="GetStati(Idx: integer)">
      <shortdescription>
        read-Methode des Properties Stati
      </shortdescription>
    </method>

    <method name="Delete(Idx: integer)">
      <shortdescription>
        Lscht ein Element aus FStati
      </shortdescription>
    </method>

    <method name="IndexOf(Status: TRakStatus)">
      <shortdescription>
        Gibt den Index des ersten Vorkommens eines Status in FStati zurck
      </shortdescription>
    </method>

    <method name="IndexOfInherited(Status: TRakStatus)">
      <shortdescription>
        Gibt den Index des ersten Vorkommens eines Status oder davon
        abgeleiteten Status in FStati zurck
      </shortdescription>
    </method>

    <method name="Add(Status: TRakStatus)">
      <shortdescription>
        Add setzt einen Status. Man kann den gleichen Status fters setzen
        man muss ihn aber dann auch genauso oft wieder lschen
      </shortdescription>
    </method>

    <method name="Delete(Status: TRakStatus)">
      <shortdescription>
        Delete lscht einen Status
      </shortdescription>
    </method>

    <method name="Exists(Status: TRakStatus)">
      <shortdescription>
        Exists prft, ob ein Status mindestens einmal gesetzt ist
      </shortdescription>
    </method>

    <method name="ExistsInherited(Status: TRakStatus)">
      <shortdescription>
        ExistsInherited prft, ob ein Status oder von ihm abgeleiteter Status
        mindestens einmal gesetzt ist
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakStati = class
  private
    function GetCount: integer;
    function GetStati(Idx: integer): TRakStatus;
  protected
    FStati: TRakStatiArr;

    procedure Delete(Idx: integer); overload; virtual;
    function IndexOf(Status: TRakStatus): integer; virtual;
    function IndexOfInherited(Status: TRakStatus): integer; virtual;
  public
    constructor Create;

    procedure Add(Status: TRakStatus); overload; virtual;
    procedure Delete(Status: TRakStatus); overload; virtual;

    function Exists(Status: TRakStatus): boolean; overload; virtual;
    function ExistsInherited(Status: TRakStatus): boolean; overload; virtual;

    property Count: integer read GetCount;
    property Stati[Idx: integer]: TRakStatus read GetStati; default;
  end;

  {****************************************************************************
  <class name="TRakObject" superclass="TObject" public="true">
    <description>
      TRakObject ist eine Basisklasse, die bereits Stati verwaltet (siehe Klasse
      <link class="TRakStati">TRakStati</link>). Die Klasse verwendet die Stati
      <link class="TRakStatusCreating">TRakStatusCreating</link>
      und <link class="TRakStatusDestroying">TRakStatusDestroying</link>.
      Wenn man von TRakObject ableitet, dann sollte man den constructor Create
      und den destructor Destroy nicht berschreiben, sondern statt dessen
      die Prozeduren <link class="TRakObject" meth="CreateNow">CreateNow</link>
      und <link class="TRakObject" meth="DestroyNow">DestroyNow</link>.
      Somit knnen alle aufgerufenen Prozeduren feststellen, ob sich das
      Objekt gerade erzeugt oder zerstrt und gesondert darauf reagieren.
      Falls ein Objekt whrend des Erzeugens nicht alles intialisieren konnte,
      da nicht sicher gestellt werden kann, dass bereits alle bentigten
      Werte korrekt intialisiert sind, dann kann die abschliesende Initialisierung
      durch berschreiben der Prozedur Created erfolgen.
      Das funktioniert allerdingas nur, wenn Created von aussen aufgerufen wird:-)
      ACHTUNG:   Eine Instanz kann nur dann freigegeben werden, wenn
                nicht der Status TRakStatusDestroying gesetzt ist.
                ==> Den Status TRakStatusDestroying sollte man nie selbst setzen,
                    das macht TRakObject.
                siehe auch Methode <link class="TRakObject" meth="CanDestroy">CanDestroy</link>
    </description>

    <variable name="FStati" type="TRakStati" access="private" isClassVariable="false" isReference="false">
      <description>
        Wird intern genutzt, zum Verwalten der Stati.
        ber das Property <link class="TRakObject" prop="Stati">Stati</link>
        kann von abgeleiteten Klassen darauf zugegriffen werden.
      </description>
    </variable>

    <property name="Stati" type="TRakStati" access="protected" isArray="false">
      <description>
        Wird zum Verwalten der Stati verwendet. Wenn z.B. eine Methode
        nicht ausgefhrt werden darf, da sich das Objekt zerstrt,
        dann kann dies durch folgenden Aufruf erledigt werden:
        if not Stati.Exists(TRakStatusDestroying) then
          ...

        In Stati kann man beliebig viele Stati ablegen, damit bekannt ist,
        was das Objekt gerade tut und man eventuell anders vorgehen kann.
        Man kann jederzeit neue Stati erzeugen, indem man eine Klasse
        von <link class="TRakBaseStatus">TRakBaseStatus</link> ableitet.
      </description>
    </property>

    <method name="Created">
      <shortdescription>
        Created muss von auen aufgerufen werden nachdem sich das Objekt erzeugt hat.
        In Stati ist dann der Status TRakStatusCreating nicht mehr
        enthalten. Hier knnen abgeleitete Klassen dann eine abschlieende
        Initialisierung vornehmen.
      </shortdescription>
    </method>

    <method name="CreateNow">
      <shortdescription>
        CreateNow sollte nie selbst aufgerufen werden,
        aber sie sollte an Stelle von Create berschrieben werden.
      </shortdescription>
    </method>

    <method name="DestroyNow">
      <shortdescription>
        DestroyNow sollte nie selbst aufgerufen werden,
        aber sie sollte an Stelle von Destroy berschrieben werden.
      </shortdescription>
    </method>

    <method name="BeforeDestroyEvent">
      <shortdescription>
        In BeforeDestroyEvent knnen abgeleitete Klassen anderen Objekten
        mitteilen, dass sich dieses Objekt jetzt zerstren wird.
        In BeforeDestroyEvent sollte man keine Aufrumarbeiten durchfhren,
        diese sollten komplett in der Methode DestroyNow erledigt werden.
        BeforeDestroyEvent wird vom Destruktor vor der Methode DestroyNow aufgerufen
      </shortdescription>
    </method>

    <method name="AfterDestroyEvent">
      <shortdescription>
        In AfterDestroyEvent knnen abgeleitete Klassen anderen Objekten
        mitteilen, dass sich dieses Objekt jetzt zerstrt hat.
        In AfterDestroyEvent sollte man keine Aufrumarbeiten durchfhren,
        diese sollten komplett in der Methode DestroyNow erledigt werden
        AfterDestroyEvent wird vom Destruktor nach der Methode DestroyNow aufgerufen
      </shortdescription>
    </method>

    <method name="GetKomposition">
      <shortdescription>
        GetKomposition wird im Destruktor aufgerufen. GetKomposition gibt
        nil zurck. Abgeleiteten Klassen knnen hier eine Referenz auf
        ein Interface von IRakKomposition (Unit RakDestroy) zurckgeben.
        Mit IRakKomposition kann man mehrere Objekte so fest aneinander binden,
        dass alle freigegeben werden, sobald eines dieser Objekte freigegeben wird.
      </shortdescription>
    </method>

    <method name="CanDestroy">
      <shortdescription>
        Mit CanDestroy kann festgestellt werden, ob die Instanz zerstrt werden
        darf.<br>
        Standardmig wird der Rckgabewert auf false gesetzt, wenn
        die Instanz den Zustand
        <link class="TRakStatusDestroying">TRakStatusDestroying</link> hat.
      </shortdescription>
    </method>

    <method name="DestroySelf">
      <shortdescription>
        DestroySelf macht das Gleiche wie Free mit dem Unterschied, dass
        auch noch ber die Methode CanDestroy festgestellt wird, ob die
        Instanz wirklich freigegeben werden darf.
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakObject = class
  private
    FStati: TRakStati;
  protected
    procedure CreateNow; virtual;
    procedure DestroyNow; virtual;

    function GetKomposition: IRakKomposition; virtual;

    procedure BeforeDestroyEvent; virtual;
    procedure AfterDestroyEvent; virtual;

    property Stati: TRakStati read FStati;
  public
    // Den Konstrukor und den Destruktor sollte man nicht berschreiben.
    // Statt dessen sollte man die Prozeduren CreateNow und DestroyNow berschreiben.
    // Somit ist in Stati der Status TRakStatusCreating oder TRakStatusDestroying
    // gesetzt, den vielleicht einige Funktionen auswerten mssen, damit
    // nicht auf nicht initialisierte Zeiger zugegriffen wird
    constructor Create;
    destructor Destroy; override;

    function CanDestroy: boolean; virtual;
    procedure Created; virtual;

    procedure DestroySelf;
  end;

  {****************************************************************************
  <class name="TRakKompositionObject" superclass="TRakObject" public="true">
    <description>
      TRakKompositionObject vervollstndigt das TRakKompositions-Konzept,
      das bereits in der Klasse TRakObject teilweise untersttzt wird.
      TRakKompositionObject untersttzt nur die Zugehrigkeit zu einer
      Komposition. Prinzipiell kann ein Objekt zu beliebig vielen
      Kompositionen gehren, das mssen abgeleiteten Klassen allerdings
      selbst implementieren.
    </description>

    <variable name="FKomposition" type="IRakKomposition" access="private" isClassVariable="false" isReference="true">
      <description>
        Die Referenz auf die Komposition, zu der das Objekt gehrt.
      </description>
    </variable>

    <property name="Komposition" type="IRakKomposition" access="public" isArray="false">
      <description>
        Legt die Komposition fest, zu der das Objekt gehrt.
        Sobald ein Objekt einer Komposition freigegeben wird, werden alle
        Objekte der Komposition freigegeben.
      </description>
    </property>

    <method name="GetKomposition">
      <shortdescription>
        read-Methode
      </shortdescription>
    </method>
    <method name="SetKomposition(const Value: IRakKomposition)">
      <shortdescription>
        write-Methode
      </shortdescription>
    </method>

    <method name="PlugOffKomposition">
      <shortdescription>
        Wird aufgerufen, wenn sich das Objekt von seiner Komposiition loslst
      </shortdescription>
    </method>

    <method name="PlugOnKomposition">
      <shortdescription>
        Wird aufgerufen, wenn sich das Objekt einer Komposiition anschliet
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakKompositionObject = class(TRakObject)
  private
    FKomposition: IRakKomposition;
  protected
    function GetKomposition: IRakKomposition; override;
    procedure SetKomposition(const Value: IRakKomposition); virtual;

    procedure PlugOffKomposition; virtual;
    procedure PlugOnKomposition; virtual;
  public
    property Komposition: IRakKomposition read GetKomposition write SetKomposition;
  end;

procedure FreeTRakObject(var Objekt);

implementation

procedure FreeTRakObject(var Objekt);
var
  Obj : TObject;
begin
  Obj := TObject(Objekt);
  TObject(Objekt) := nil;
  if Obj is TRakObject then
    TRakObject(Obj).DestroySelf
  else
    Obj.Free;
end;
{ TRakObject }

procedure TRakObject.AfterDestroyEvent;
begin
end;

procedure TRakObject.BeforeDestroyEvent;
begin
end;

function TRakObject.CanDestroy: boolean;
begin
  Result := not FStati.Exists(TRakStatusDestroying);
end;

constructor TRakObject.Create;
begin
  FStati := TRakStati.Create;
  FStati.Add(TRakStatusCreating);
  try
    CreateNow;
  finally
    FStati.Delete(TRakStatusCreating);
  end;
end;

procedure TRakObject.Created;
begin
end;

procedure TRakObject.CreateNow;
begin
end;

destructor TRakObject.Destroy;
var
  Komposition: IRakKomposition;
begin
  FStati.Add(TRakStatusDestroying);

  // Zuerst besorgen wir uns die Komposition, zu der dieses Objekt gehrt
  Komposition := GetKomposition;
  // Dann nehmen wir das Objekt aus der Komposition heraus, um Seiteneffekte
  // durch den anschlieenden Aufruf von BeforeDestroyEvent zu unterbinden
  if Komposition <> nil then
    Komposition.DeleteMember(self);

  // Dann geben wir abgeleiteten Klassen die Mglichkeit etwas zu machen, solange die Klasse noch komplett ist
  BeforeDestroyEvent;

  // Jetzt sorgen wir dafr, dass alle Objekte, die zur gleichen Komposition gehren, auch freigegben werden
  if Komposition <> nil then
    Komposition.MemberDestroying(self);

  // Jetzt werden die eigentlichen Aufrumarbeiten durchgefhrt
  DestroyNow;

  // Abgeleiteten Klassen die Mglichkeit geben noch was zu machen, wenn alle Aufrumarbeiten durchgefhrt wurden
  AfterDestroyEvent;

  FreeTObject(FStati);

  inherited;
end;

procedure TRakObject.DestroyNow;
begin
end;

procedure TRakObject.DestroySelf;
begin
  if (self <> nil) and CanDestroy then
    Free;
end;

function TRakObject.GetKomposition: IRakKomposition;
begin
  Result := nil;
end;

{ TRakStati }

procedure TRakStati.Add(Status: TRakStatus);
var
  Anz: integer;
begin
  Anz := length(FStati);
  SetLength(FStati, Anz + 1);
  FStati[Anz] := Status;
end;

constructor TRakStati.Create;
begin
  SetLength(FStati, 0);
end;

procedure TRakStati.Delete(Status: TRakStatus);
begin
  Delete(IndexOf(Status));
end;

procedure TRakStati.Delete(Idx: integer);
var
  i: integer;
  Anz: integer;
begin
  Anz := length(FStati);
  if (Idx >= 0) and (Idx < Anz) then
  begin
    for i := Idx to Anz - 2 do
      FStati[i] := FStati[i+1];
    SetLength(FStati, Anz - 1);
  end;
end;

function TRakStati.Exists(Status: TRakStatus): boolean;
begin
  Result := IndexOf(Status) >= 0;
end;

function TRakStati.ExistsInherited(Status: TRakStatus): boolean;
begin
  Result := IndexOfInherited(Status) >= 0;
end;

function TRakStati.GetCount: integer;
var
  i, t: integer;
  lclStati: TRakStatiArr;
  Anz: integer;
  Status: TRakStatus;
  Found: boolean;
begin
  Anz := length(FStati);
  SetLength(lclStati, Anz);
  Result := 0;
  for i := 0 to Anz - 1 do
  begin
    Status := FStati[i];
    Found := false;
    t := 0;
    while not Found and (t <  Result) do
    begin
      Found := lclStati[t] = Status;
      inc(t);
    end;

    if not Found then
    begin
      lclStati[Result] := Status;
      inc(Result);
    end;
  end;
end;

function TRakStati.GetStati(Idx: integer): TRakStatus;
var
  i, t: integer;
  lclStati: TRakStatiArr;
  Anz: integer;
  Counter: integer;
  Status: TRakStatus;
  Found: boolean;
begin
  Result := nil;
  Anz := length(FStati);
  SetLength(lclStati, Anz);
  Counter := 0;
  for i := 0 to Anz - 1 do
  begin
    Status := FStati[i];
    Found := false;
    t := 0;
    while not Found and (t <  Counter) do
    begin
      Found := lclStati[t] = Status;
      inc(t);
    end;

    if not Found then
    begin
      if Counter < Idx then
      begin
        lclStati[Counter] := Status;
        inc(Counter);
      end
      else
      begin
        Result := Status;
        break;
      end;
    end;
  end;
end;

function TRakStati.IndexOf(Status: TRakStatus): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to length(FStati) - 1 do
  begin
    if FStati[i] = Status then
    begin
      Result := i;
      break;
    end;
  end;
end;

function TRakStati.IndexOfInherited(Status: TRakStatus): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to length(FStati) - 1 do
  begin
    if FStati[i].InheritsFrom(Status)  then
    begin
      Result := i;
      break;
    end;
  end;
end;

{ TRakKompositionObject }

function TRakKompositionObject.GetKomposition: IRakKomposition;
begin
  Result := FKomposition;
end;

procedure TRakKompositionObject.PlugOffKomposition;
begin
  if FKomposition <> nil then
    FKomposition.DeleteMember(self);
end;

procedure TRakKompositionObject.PlugOnKomposition;
begin
  if FKomposition <> nil then
    FKomposition.AddMember(self);
end;

procedure TRakKompositionObject.SetKomposition(const Value: IRakKomposition);
begin
  if Value <> FKomposition then
  begin
    PlugOffKomposition;

    FKomposition := Value;

    PlugOnKomposition;
  end;
end;

end.
