unit RakUndoRedo;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit RakUndoRedo stellt Klassen zur Verfgung, die ein
                  allgemeines Konzept zur Implementierung eines
                  Undo/Redo-Mechanismuses implementieren.
                  Beispielanwendungen kann man von http://www.kasparsoft.de herunterladen.
// erstellt am  : 4.1.2001
  25.4.2002 TRakHistory erweitert
  23.5.2002 Bugfix in TRakHistory
  25.5.2002 TRakHistory erweitert
            TRakCommandGroup erweitert
            Neue Exception ERakUndoRedoSynchronisationLost
  7.6.2002  TRakHistory erweitert
  16.2.2003 Neue Klasse TRakHcMove
  19.7.2003 Neue Klasse TRakHcTShiftState
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  Sysutils, Graphics, Classes, Windows, Messages, Forms,
  miscfunc, misctype, MultiEvent, RakObject, RakCollection,
  RakBinaryStreamData;

type

  TRakStatusReUndoing = class(TRakBaseStatus);
  TRakStatusRedoing = class(TRakStatusReUndoing);
  TRakStatusUndoing = class(TRakStatusReUndoing);
  TRakStatusIgnoreAddings = class(TRakBaseStatus);

  TRakCommand = class;
  TRakCustomCommand = class;
  TRakCommandGroup = class;
  TRakCommandGroupCol = class;
  TRakCustomCommandClass = class of TRakCustomCommand;

  ERakUndoRedoSynchronisationLost = class(Exception);

  TChangeEventRedo = procedure (Command: TRakCommand; var DoIt, Done: boolean) of object;
  TChangeEventUndo = procedure (Command: TRakCommand; var DoIt, Done: boolean) of object;

{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
  {$DEFINE _RAK_UNDO_REDO_SERVER_OR_CLIENT}
{$ENDIF}
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  {$DEFINE _RAK_UNDO_REDO_SERVER_OR_CLIENT}
{$ENDIF}


{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHistory
// Beschreibung : TRakHistory bildet das Kernstck des Undo/Redo-Mechanismuses.
                  Von TRakHistory wird es in der Regel nur eine Instanz geben.
                  Diese Instanz speichert alle Daten, die ntig sind, um eine
                  nderung rckgngig zu machen bzw. die nderung erneut auszufhren.

                  Eine neue nderung muss mit der Methode AddCommand bekannt
                  gemacht werden. Wenn man mehrere nderungen gruppieren
                  mchte, so kann dies durch die Methoden NewGroupStart
                  und NewGroupStop gemacht werden. NewGroupStart und
                  NewGroupStop knnen auch geschachtelt werden.
                  Durch Aufruf der Methode Undo kann eine nderung
                  rckgngig gemacht werden. Das Gegenstck zu Undo ist Redo.
                  Zum Enablen und Beschriften von Undo/Redo-Menpunkten oder
                  ~Buttons kann man auf die Properties Count, CommandGroup und UndoPtr
                  und das Ereignis OnCommandGroupAdded zurckgreifen.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  25.4.2002 Neue Methode FreeOldest
            Neues Property MaxCount; Man kann jetzt die Anzahl der gespeicherten
              nderungen beschrnken, damit es nicht den Speicher zerreit.
  23.5.2002 Bugfix in NewGroupStop: Wenn es die letzte Gruppierung war und diese leer
              war, dann wurde das Flag FIsAddingCommands nicht zurck gesetzt
  25.5.2002 Neues Property ActiveCommandGroup
            Neue Methoden AbortAdding und AbortUntil;
  7.6.2002  Neues Ereignis OnCommandGroupDismissed
////////////////////////////////////////////////////////////////////////////////}
  TRakHistory = class(TComponent)
  private
    FOnCommandGroupAdded: TNotifyEvent;
    FOnCommandGroupDismissed: TNotifyEvent;
    FOnRedo: TChangeEventRedo;
    FOnUndo: TChangeEventUndo;
    FOnAfterRedo: TRakNotifyEventVerwalter;
    FOnAfterUndo: TRakNotifyEventVerwalter;
    FUndoPtr: integer;
    FStati: TRakStati;
    FIsAddingCommands: boolean;
    FMaxCount: integer;

{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
    // Variablen fr das Debugging
    FMonitorID: integer;
    FMonitorHandle: THandle;
{$ENDIF}

    function GetCount: integer;
    function GetIsUnRedoing: boolean;
    function GetCommandGroup(Idx: integer): TRakCommandGroup;
    function GetIgnoreAddings: boolean;
    procedure SetIgnoreAddings(const Value: boolean);
    procedure SetMaxCount(const Value: integer);
    function GetIsDestroying: boolean;
  protected
    FCommandGroupCol: TRakCommandGroupCol;
    FActiveCommandGroup: TRakCommandGroup;

    procedure CreateNow; virtual;
    procedure DestroyNow; virtual;

    procedure ChangeFinished; virtual;

    property Stati: TRakStati read FStati;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    // Mit Redo wird die zuletzt rckgngig gemachte nderung erneut durchgefhrt
    function Redo: boolean;
    // Undo macht die letzte nderung rckgngig
    function Undo: boolean;
    // Clear lscht alle Daten zu allen nderungen
    procedure Clear;


    // Die folgenden Methoden werden zur Untersttzung des Debugging-Monitors eingesetzt
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    class procedure RegisterCustomCommand(TheClass: TRakCustomCommandClass);
    class function FindBestClass(ClassNameList: string): TRakCustomCommandClass;
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
    function FindByCmdId(CmdId: longint): TRakCustomCommand;
{$ENDIF}

    // Mit AddCommand wird eine neue nderung bekannt gemacht.
    // AddCommand erwartet ein Objekt, das in der Lage ist die nderung rckgngig zu machen, bzw. diese erneut durchfhren kann
    // Command geht in den Besitzt von TRakHistory ber!
    procedure AddCommand(Command: TRakCommand); virtual;
    // Mit NewGroupStart wird eine neue Gruppe von nderungen begonnen. Alle folgenden AddCommand-Aufrufe gehren dann zu dieser Gruppe
    procedure NewGroupStart(Group: TRakCommandGroup); virtual;
    // Mit NewGroupStop muss eine Gruppe von nderungen abgeschlossen werden.
    // Sollte die Gruppe leer sein, dann wird sie wieder verworfen, andernfalls werden alle nderungen,
    // die fr den Aufruf von Redo bereitstehen, gelscht
    procedure NewGroupStop; virtual;
    // Mit AbortAdding knnen alle mit NewGroupStart geffneten Gruppierungen geschlossen werden
    // AbortAdding wir z.B. am Anfang von den Methoden Undo und Redo aufgerufen
    procedure AbortAdding; virtual;
    // AbortUntil schliet alle Gruppierungen bis zu UntilGroup.
    // OnlySubGroups legt fest, ob UntilGroup auch geschlossen wird
    procedure AbortUntil(UntilGroup: TRakCommandGroup; OnlySubGroups: boolean); virtual;

    // Mit FreeOldest kann man die ltesten nderungen lschen; FreeCounter gibt an, wie viele gelscht werden sollen.
    procedure FreeOldest(FreeCounter: integer = 1);

    // Count gibt die Anzahl aller nderungen zurck.
    property Count: integer read GetCount;
    // Mit CommandGroup kann auf alle nderungen der ersten Ebene zugegriffen werden ( 0 <= Idx < Count )
    property CommandGroup[Idx: integer]: TRakCommandGroup read GetCommandGroup; default;
    // UndoPtr zeigt an, welche nderung beim nchsten Aufruf von Undo betroffen wre. UndoPtr+1 wre vom Aufruf von Redo betroffen
    property UndoPtr: integer read FUndoPtr;
    // IsUnRedoing gibt an, ob gerade ein Undo oder Redo durchgefhrt wird
    property IsUnRedoing: boolean read GetIsUnRedoing;
    // IsAddingCommands gibt an, ob gerade weitere nderungen der History hinzugefgt werden
    // IsAddingCommands ist z.B. solange true, bis alle NewGroupStart-Aufrufe durch entsprechend viele NewGroupStop-Aufrufe abgeschlossen sind
    property IsAddingCommands: boolean read FIsAddingCommands;
    // IsDestroying zeigt an, ob die Instanz gerade freigegeben wird
    property IsDestroying: boolean read GetIsDestroying;
    // Mit IgnoreAddings kann die History angewiesen werden Aufrufe von AddCommand zu ignorieren
    property IgnoreAddings: boolean read GetIgnoreAddings write SetIgnoreAddings;
    // Whrend IsAddingCommands = true kann man mit ActiveCommandGroup feststellen,
    // welches Object neue Commands aufnimmt, die mit AddCommand bergeben werden.
    property ActiveCommandGroup: TRakCommandGroup read FActiveCommandGroup;

    // OnAfterRedo wird am Ende der Funktion Redo ausgelst
    property OnAfterRedo: TRakNotifyEventVerwalter read FOnAfterRedo;
    // OnAfterUndo wird am Ende der Funktion Undo ausgelst
    property OnAfterUndo: TRakNotifyEventVerwalter read FOnAfterUndo;
  published
    // MaxCount legt fest, wieviele nderungen maximal gespeichert werden.
    // Wenn Count = MaxCount und eine neue nderung hinzukommt, dann wird die lteste gelscht.
    // Der Wert 0 steht fr unendlich viele nderungen.
    property MaxCount: integer read FMaxCount write SetMaxCount default 0;

    // OnCommandGroupAdded wird ausgelst, wenn eine neue nderung hinzugekommen ist
    property OnCommandGroupAdded: TNotifyEvent read FOnCommandGroupAdded write FOnCommandGroupAdded;
    // OnCommandGroupDismissed wird ausgelst, wenn eine neue nderung verworfen wurde
    property OnCommandGroupDismissed: TNotifyEvent read FOnCommandGroupDismissed write FOnCommandGroupDismissed;
    // Die Ereignisse OnRedo und OnUndo drften inzwischen keinen Sinn mehr machen, da
    // TRakCustomCommand inzwischen so fexibel geworden ist, dass es wahrscheinlich
    // keinen Fall mehr gibt, bei dem von auen in den Undo/Redo-Mechanismus eingegriffen werden muss
    // OnRedo wird ausgelst, wenn die Funktion Redo aufgerufen wurde. Es werden die Daten bergeben, mit dessen Hilfe die nderung erneut ausgefhrt werden muss
    property OnRedo: TChangeEventRedo read FOnRedo write FOnRedo;
    // OnUndo wird ausgelst, wenn die Funktion Undo aufgerufen wurde. Es werden die Daten bergeben, mit dessen Hilfe die nderung rckgngig gemacht werden muss
    property OnUndo: TChangeEventUndo read FOnUndo write FOnUndo;
  end;



{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCustomCommand
// Beschreibung : TRakCustomCommand ist die Basisklasse aller Klassen, welche die
                  Daten speichern mssen, mit denen eine nderung rckgngig
                  gemacht, bzw. erneut durchgefhrt werden kann.
                  TRakCustomCommand selbst muss auch das Undo und Redo durchfhren
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  27.1.2002 Neue Methode AfterDone
  2.3.2002 Bugfix: Die Paramter von CanUndo und CanRedo waren nicht "var".
////////////////////////////////////////////////////////////////////////////////}
  TRakCustomCommand = class(TRakCollectionItem)
  private
    function GetCommandGroup: TRakCommandGroup;
    function GetHistory: TRakHistory;
    function GetTopCommandGroup: TRakCommandGroup;
  protected
    // AfterAdded wird aufgerufen, nachdem alle Informationen zu einer nderung bekannt sind
    procedure AfterAdded; virtual;

    // RedoUndoSubCommand gibt dem kompletten Pfad die Chance irgendwie auf einzelne Redo/Undo-Aktionen zu reagieren
    // Command ist die nderung mit der etwas gemacht wird
    // Redo gibt an, ob ein Redo oder Undo auf die nderung durchgefhrt wird
    // Before gibt an, ob die nderung gleich durchgefhrt wird oder bereits erfolgte
    procedure RedoUndoSubCommand(Command: TRakCustomCommand; Redo, Before: boolean); virtual;

    // Die nchsten beiden Methoden werden fr den Debugging-Monitor eingesetzt
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); virtual;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); virtual;

    // CanUndo kann von abgeleiteten Klassen berschrieben werden, um festzustellen,
    // ob ein Undo berhaupt durchgefhrt werden kann. CanUndo wird innerhalb von
    // Undo, vor UndoNow aufgerufen
    procedure CanUndo(var DoIt, Done: boolean); virtual;
    procedure CanRedo(var DoIt, Done: boolean); virtual;

    // UndoNow muss von abgeleiteten Klassen berschrieben werden. Hier mssen
    // abgeleitete Klassen eine nderung rckgngig machen. UndoNow wird
    // innerhalb von Undo aufgerufen. Ob UndoNow aufgerufen wird, wird von
    // CanUndo bestimmt.
    function UndoNow: boolean; virtual; abstract;
    function RedoNow: boolean; virtual; abstract;
    // AfterDone wird aufgerufen, nachdem alle Commands einer Undo- oder Redoaktion durchgefhrt wurden
    // Hier knnen abgeleitete Klassen noch irgendwelche Aktionen durchfhren, die sie beim
    // Aufruf von UndoNow oder RedoNow noch nicht durchfhren konnten, da zu dem Zeitpunkt
    // noch nicht alle bentigten Informationen zugreifbar waren.
    function AfterDone(Undo: boolean): boolean; virtual;
  public
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
    DataAsTag: TRakBinaryTag;
    CmdId: longint;
{$ENDIF}
    destructor Destroy; override;

    // Der Aufruf von Undo macht die nderung wieder rckgngig
    function Undo: boolean;
    // Der Aufruf von Redo fhrt die nderung erneut durch
    function Redo: boolean;

    // CommandGroup ist die Gruppierung, durch die dieses TRakCustomCommand gruppiert ist.
    // Es wird nil zurckgegeben, falls es selbst die oberste Gruppierung ist
    property CommandGroup: TRakCommandGroup read GetCommandGroup;
    // TopCommandGroup ist die oberste Gruppierung, die alle Teilnderungen einer Gesamtnderung zusammenfasst
    property TopCommandGroup: TRakCommandGroup read GetTopCommandGroup;
    // History ist die History, der diese TRakCustomCommand gehrt
    property History: TRakHistory read GetHistory;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCommand
// Beschreibung : TRakCommand ist die Basisklasse fr atomare Teilnderungen.
                  Eine oder mehrere atomare Teilnderungen bilden zusammen
                  eine nderung.
                  TRakCommand wird immer durch TRakCommandGroup gruppiert, selbst
                  dann, wenn die Gruppe nur aus einem TRakCommand besteht.
                  TRakCommand ist nicht in der Lage nderungen rckgngig zu machen
                  oder erneut durchzufhren. Diese Funktionalitt mssen die
                  von TRakCommand abgeleiteten Klassen hinzufgen.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakCommand = class(TRakCustomCommand)
  protected
    FChangedObjectName: string;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    // CanUndo lst das Ereignis OnUndo von TRakHistory aus
    procedure CanUndo(var DoIt, Done: boolean); override;
    // CanRedo lst das Ereignis OnRedo von TRakHistory aus
    procedure CanRedo(var DoIt, Done: boolean); override;

    // GetChangedObject ermittelt mit Hilfe der globalen Objektverwaltung die Referenz von ChangedObjectName
    function GetChangedObject: TObject; virtual;
    // write-Methode
    procedure SetChangedObjectName(Value: string); virtual;

    // CheckChangedObject prft nur, ob Obj <> nil. Abgeleitete Klassen
    // knnen die Methode berschreiben, um genauere Prfungen durchzufhren,
    // ob Obj wirklich auf das Objekt verweist, dass durch die nderung betroffen ist
    function CheckChangedObject(Obj: TObject): boolean; virtual;
  public
    // ChangedObject gibt die Referenz des Objektes ChangedObjectName zurck
    property ChangedObject: TObject read GetChangedObject;
    // ChangedObjectName enthlt den Namen des Objektes, das von der nderung betroffen ist
    property ChangedObjectName: string read FChangedObjectName write SetChangedObjectName;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCommandCol
// Beschreibung : TRakCommandCol wird von TRakCommandGroup als Kontner
                  fr die einzelnen Teilnderungen genutzt.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakCommandCol = class(TRakCollection)
  private
    FCommandGroup: TRakCommandGroup;

    function GetItems(Idx: integer): TRakCustomCommand;
    function GetHistory: TRakHistory; virtual;
  public
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);

    property Items[Idx: integer]: TRakCustomCommand read GetItems; default;

    property CommandGroup: TRakCommandGroup read FCommandGroup;
    property History: TRakHistory read GetHistory;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCommandGroup
// Beschreibung : TRakCommandGroup gruppiert mehrere Teilnderungen.
                  TRakCommandGroup selbst kann eine Teilnderung sein. Somit
                  ist eine hierachische Struktur von Teilnderungen mglich
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  25.5.2002  Neue Methoden Abort und AbortNow
////////////////////////////////////////////////////////////////////////////////}
  TRakCommandGroup = class(TRakCustomCommand)
  private
    function GetCount: integer;
    function GetItems(Idx: integer): TRakCustomCommand;
  protected
    FCommandCol: TRakCommandCol;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    // Mit Add wird eine weitere Teilnderung hinzugefgt
    procedure Add(Command: TRakCustomCommand);

    // UndoNow ruft von jeder Teilnderung das Undo auf
    function UndoNow: boolean; override;
    // RedoNow ruft von jeder Teilnderung das Redo auf
    function RedoNow: boolean; override;
    // AfterDone ruft von jeder Teilnderung dessen AfterDone auf
    function AfterDone(Undo: boolean): boolean; override;

    // AbortNow wird von Abort aufgerufen.
    // In Abort knnen abgeleitete Klassen dafr sorgen, dass die nderung noch
    // sauber abgeschlossen wird, bevor diese beendet wird.
    procedure AbortNow; virtual;
    // GetMenueText sollte von abgeleiteten Klassen berschrieben werden,
    // damit Anwendungen einen ausagekrftigen Text anzeigen knnen, der
    // beschreibt, welche Art von nderung rckgngig gemacht werden kann
    function GetMenueText: string; virtual;
    // SetMenueText wirft eine Exception, kann aber von abgeleiteten Klassen
    // berschrieben werden, wenn sie das ndern des Mentextes untersttzen wollen
    procedure SetMenueText(const Value: string); virtual;
  public
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
    RealMenueText: string;
    function FindByCmdId(CmdId: longint): TRakCustomCommand;
{$ENDIF}

    constructor Create; override;
    destructor Destroy; override;

    // Mit Abort kann diese Gruppierung abgeschlossen werden
    // Abort wird z.B. auch von TRakHistory.AbortAdding aufgerufen.
    // Dies passiert, wenn das Hinzufgen von nderungen unerwartet
    // abgebrochen werden soll.
    // Abort prft zuerst, ob diese Gruppe wirklich die aktuelle ist und wirft
    // die Exception ERakUndoRedoSynchronisationLost, wenn dies nicht so ist
    // Dann wird AbortNow aufgerufen und danach die Gruppierung abgeschlossen (NewGroupStop)
    procedure Abort;

    // MenueText, kann z.B. im Hint von Redo/Undo-Buttons angezeigt werden.
    property MenueText: string read GetMenueText write SetMenueText;

    // Count ist die Anzahl der gruppierten Teilnderungen
    property Count: integer read GetCount;
    // Mit Items kann auf die gruppierten Teilnderungen zugegriffen werden (0 <= Idx < Count)
    property Items[Idx: integer]: TRakCustomCommand read GetItems; default;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCommandGroupCol
// Beschreibung : TRakCommandGroupCol wird von TRakHistory als Kontner fr
                  die einzelnen nderungen verwendet.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakCommandGroupCol = class(TRakCommandCol)
  private
    FHistory: TRakHistory;
    function GetItems(Idx: integer): TRakCommandGroup;
    function GetHistory: TRakHistory; override;
  protected
    procedure ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer); override;
  public
    function Add: TRakCommandGroup; overload;
    procedure FreeOldest(FreeCounter: integer);

    property Items[Idx: integer]: TRakCommandGroup read GetItems; default;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakCGCommon
// Beschreibung : TRakCGCommon erweitert TRakCommandGroup um die Mglichkeit
                  beim Erzeugen eine beliebigen Mentext zu setzen.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
TRakCGCommon = class(TRakCommandGroup)
private
  FMenuText: string;
protected
  function GetMenueText: string; override;
  procedure SetMenueText(const Value: string); override;
public
  constructor Init(MenueText: string);
end;

//******************************************************************************
// Ab hier kommen Klassen, die von TRakCommand abgeleitet sind
// und als Basisklassen fr eigenen TRakCommand-Klassen dienen knnen
// Die meisten der hier angebotenen Klassen gehen davon aus, dass sich die Objekte,
// auf die sich das Undo/Redo auswirkt bei der globalen Objektverwaltung
// der Unit RakGlobalObjectID angemeldet haben.

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcObjRef
// Beschreibung : TRakHcObjRef speichert eine Referenz auf ein Objekt
                  Man sollte sich genau berlegen, ob die Referenz wirklich immer gltig ist!!!!!!
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcObjRef = class(TRakCommand)
  protected
    FValue: TObject;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Value: TObject);

    property Value: TObject read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcChangeData
// Beschreibung : TRakHcChangeData ist nur aus Grnden der Abwrtskompatibilitt vorhanden
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcChangeData = class(TRakCommand)
  protected
    FValue: TRakCommand;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Value: TRakCommand);
    destructor Destroy; override;

    property Value: TRakCommand read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcStringNew
// Beschreibung : TRakHcStringNew speichert einen einzelnen String. Dies knnte
                  z.B. interessant sein, wenn nur bei einem Redo aber nicht
                  bei einem Undo eine ndeung ntig ist
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcStringNew = class(TRakCommand)
  protected
    FValue: string;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public

    constructor Init(Value: string);

    property Value: string read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcIntegerNew
// Beschreibung : TRakHcIntegerNew speichert einen einzelnen Integer. Dies knnte
                  z.B. interessant sein, wenn nur bei einem Redo aber nicht
                  bei einem Undo eine ndeung ntig ist
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcIntegerNew = class(TRakCommand)
  protected
    FValue: integer;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Value: integer);

    property Value: integer read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcExtendedNew
// Beschreibung : TRakHcExtendedNew speichert einen einzelnen Float. Dies knnte
                  z.B. interessant sein, wenn nur bei einem Redo aber nicht
                  bei einem Undo eine ndeung ntig ist
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcExtendedNew = class(TRakCommand)
  protected
    FValue: extended;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public

    constructor Init(Value: extended);

    property Value: extended read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcBinaryTag
// Beschreibung : TRakHcBinaryTag speichert einen einzelnen TRakBinaryTag. Dies knnte
                  z.B. interessant sein, wenn ein Objekt zerstrt wird und fr ein
                  Undo alle persistenten Daten des Objektes gesichert werden mssen
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcBinaryTag = class(TRakCommand)
  protected
    FValue: TRakBinaryTag;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Value: TRakBinaryTag);
    destructor Destroy; override;

    property Value: TRakBinaryTag read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcBigChange
// Beschreibung : TRakHcBigChange speichert zwei TRakBinaryTag. Dies knnte
                  z.B. interessant sein, wenn sich so viele Werte auf einmal
                  ndern, dass es am einfachsten ist, den kompletten Zustand
                  eines Objektes vor und nach der nderung festzuhalten
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcBigChange = class(TRakCommand)
  protected
    FOldValue: TRakBinaryTag;
    FNewValue: TRakBinaryTag;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: TRakBinaryTag; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: TRakBinaryTag);
    destructor Destroy; override;

    property OldValue: TRakBinaryTag read FOldValue;
    property NewValue: TRakBinaryTag read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcString
// Beschreibung : TRakHcString speichert zwei Strings. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcString = class(TRakCommand)
  protected
    FOldValue: string;
    FNewValue: string;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: string; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: string);

    property OldValue: string read FOldValue;
    property NewValue: string read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcInteger
// Beschreibung : TRakHcInteger speichert zwei Integer. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcInteger = class(TRakCommand)
  protected
    FOldValue: integer;
    FNewValue: integer;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: integer; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: integer);

    property OldValue: integer read FOldValue;
    property NewValue: integer read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcSourceDestinationIdx
// Beschreibung : TRakHcSourceDestinationIdx speichert zwei Integer fr z.B. eine Verschiebung (TList.Move)
// erstellt am  : 16.2.2003
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcSourceDestinationIdx = class(TRakCommand)
  protected
    FSrcIdx: integer;
    FDstIdx: integer;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // Move muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure Move(Obj: TObject; Src, Dst: integer; var Done: boolean); virtual; abstract;
  public
    constructor Init(SrcIdx, DstIdx: integer);

    property SrcIdx: integer read FSrcIdx;
    property DstIdx: integer read FDstIdx;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcExtended
// Beschreibung : TRakHcExtended speichert zwei Float. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcExtended = class(TRakCommand)
  protected
    FOldValue: extended;
    FNewValue: extended;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: extended; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: extended);

    property OldValue: extended read FOldValue;
    property NewValue: extended read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcBoolean
// Beschreibung : TRakHcBoolean speichert den Boolean fr das Redo. Der Wert
                  fr das Undo ergibt sich von selbst
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcBoolean = class(TRakCommand)
  protected
    FNewValue: boolean; // FOldValue = not FNewValue

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: boolean; var Done: boolean); virtual; abstract;
  public
    constructor Init(NewValue: boolean);

    property NewValue: boolean read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcTDateTime
// Beschreibung : TRakHcTDateTime speichert zwei TDateTime. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcTDateTime = class(TRakCommand)
  protected
    FOldValue: TDateTime;
    FNewValue: TDateTime;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: TDateTime; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: TDateTime);

    property OldValue: TDateTime read FOldValue;
    property NewValue: TDateTime read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcTColor
// Beschreibung : TRakHcTColor speichert zwei TColor. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcTColor = class(TRakCommand)
  protected
    FOldValue: TColor;
    FNewValue: TColor;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: TColor; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: TColor);

    property OldValue: TColor read FOldValue;
    property NewValue: TColor read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcTExtPoint
// Beschreibung : TRakHcTExtPoint speichert zwei TExtPoint. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcTExtPoint = class(TRakCommand)
  protected
    FOldValue: TExtPoint;
    FNewValue: TExtPoint;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; Value: TExtPoint; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: TExtPoint);

    property OldValue: TExtPoint read FOldValue;
    property NewValue: TExtPoint read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcTShiftState
// Beschreibung : TRakHcTShiftState speichert zwei TShiftState. Einen fr das Undo
                  und einen fr das Redo.
// erstellt am  : 19.7.2003
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcTShiftState = class(TRakCommand)
  protected
    FOldValue: TShiftState;
    FNewValue: TShiftState;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;

    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // SetValue muss von abgeleiteten Klassen berschrieben werden.
    // Es wird von UndoNow und RedoNow aufegrufen
    procedure SetValue(Obj: TObject; NewValue: TShiftState; var Done: boolean); virtual; abstract;
  public
    constructor Init(OldValue, NewValue: TShiftState);

    property OldValue: TShiftState read FOldValue;
    property NewValue: TShiftState read FNewValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcIdxObj
// Beschreibung : TRakHcIdxObj speichert ein Objekt und einen Indexwert fr
                  ein Array oder eine Liste.
                  TRakHcIdxObj ist der Besitzer des Objektes und gibt es auch frei.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcIdxObj = class(TRakCommand)
  protected
    FIdx: integer;
    FValue: TObject;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Idx: integer; Value: TObject);
    destructor Destroy; override;

    property Idx: integer  read FIdx;
    property Value: TObject read FValue;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcObjId
// Beschreibung : TRakHcObjId erweitert TRakHcStringNew um die Annahme, dass
                  der gespeicherte String der global eindeutige Name eines Objekts ist
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcObjId = class(TRakHcStringNew)
  public
    function GetObject: TObject;
  public
    constructor InitByObjekt(Obj: TObject; NewObjId: string);
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcChangeObjId
// Beschreibung : TRakHcChangeObjId ist eine vollwerte Undo/Redo-Klasse und muss nicht
                  mehr abgeleitete werden. Mit der Klasse kann die nderung
                  des globalen Namens eines Objektes in einer History gespeichert werden
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcChangeObjId = class(TRakHcObjId)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcCreateGlobalObject
// Beschreibung : Mit TRakHcCreateGlobalObject kann das Erzeugen eines neuen
                  Objektes in der History untersttzt werden.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcCreateGlobalObject = class(TRakHcStringNew)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // Abgeleitete Klassen mssen CreateObject so berschreiben, dass es eine
    // Instanz der betroffen Klasse erzeugt und zurckgibt
    function CreateObject: TObject; virtual; abstract;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHcDestroyGlobalObject
// Beschreibung : Mit TRakHcDestroyGlobalObject kann das Lschen eines neuen
                  Objektes in der History untersttzt werden.
// erstellt am  : 4.1.2002
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakHcDestroyGlobalObject = class(TRakHcStringNew)
  protected
    function UndoNow: boolean; override;
    function RedoNow: boolean; override;

    // Abgeleitete Klassen mssen CreateObject so berschreiben, dass es eine
    // Instanz der betroffen Klasse erzeugt und zurckgibt
    function CreateObject: TObject; virtual; abstract;
  end;

////////////////////////////////////////////////////////////////////////////////
// Die folgenden Idx-Klassen sind identisch zu den "Nicht-Idx"-Klassen,
// mit dem Unterschied, dass noch zustzlich ein Index-Wert fr ein Array
// oder eine Liste gespeichert wird.

  TRakHcIdxStringNew = class(TRakCommand)
  protected
    FIdx: integer;
    FValue: string;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Idx: integer; Value: string);

    property Idx: integer  read FIdx;
    property Value: string read FValue;
  end;

  TRakHcIdxBinaryTag = class(TRakCommand)
  protected
    FIdx: integer;
    FValue: TRakBinaryTag;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Idx: integer; Value: TRakBinaryTag);
    destructor Destroy; override;

    property Idx: integer  read FIdx;
    property Value: TRakBinaryTag read FValue;
  end;

  TRakHcIdxIntegerNew = class(TRakCommand)
  protected
    FIdx: integer;
    FValue: integer;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
  public
    constructor Init(Idx: integer; Value: integer);

    property Idx: integer  read FIdx;
    property Value: integer read FValue;
  end;

  TRakHcIdxString = class(TRakHcString)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: string);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxInteger = class(TRakHcInteger)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: integer);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxExtended = class(TRakHcExtended)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: extended);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxBoolean = class(TRakHcBoolean)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; NewValue: boolean);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxTDateTime = class(TRakHcTDateTime)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: TDateTime);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxTColor = class(TRakHcTColor)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: TColor);

    property Idx: integer  read FIdx;
  end;

  TRakHcIdxTExtPoint = class(TRakHcTExtPoint)
  protected
    FIdx: integer;
  public
    constructor Init(Idx: integer; OldValue, NewValue: TExtPoint);

    property Idx: integer  read FIdx;
  end;

implementation

uses RakGlobalObjectID, RakUndoRedoMonitorInterface, Contnrs;


var
  RegisteredCustomCommandClasses: TClassList;

{ TRakHistory }

procedure TRakHistory.AddCommand(Command: TRakCommand);
var
  StopGroup: boolean;
begin
  if not IsUnRedoing and not IgnoreAddings and not IsDestroying then
  begin
    StopGroup := FActiveCommandGroup = nil;
    if StopGroup then
      NewGroupStart(TRakCommandGroup.Create);

    FActiveCommandGroup.Add(Command);

    if StopGroup then
      NewGroupStop;
  end
  else
    Command.Free;
end;

procedure TRakHistory.Clear;
begin
  FCommandGroupCol.FreeAll;
end;

procedure TRakHistory.CreateNow;
begin
  inherited;

  FUndoPtr := -1;
  FOnAfterRedo := TRakNotifyEventVerwalter.Create;
  FOnAfterUndo := TRakNotifyEventVerwalter.Create;
  FCommandGroupCol := TRakCommandGroupCol.Create;
  FCommandGroupCol.FHistory := self;
end;

procedure TRakHistory.DestroyNow;
begin
  FCommandGroupCol.Free;
  FOnAfterRedo.Free;
  FOnAfterUndo.Free;

  inherited;
end;

function TRakHistory.GetCommandGroup(Idx: integer): TRakCommandGroup;
begin
  Result := FCommandGroupCol[Idx];
end;

function TRakHistory.GetCount: integer;
begin
  Result := FCommandGroupCol.Count;
end;

function TRakHistory.GetIsUnRedoing: boolean;
begin
  Result := Stati.ExistsInherited(TRakStatusReUndoing);
end;

procedure TRakHistory.NewGroupStart(Group: TRakCommandGroup);
begin
  if not IsUnRedoing and not IsDestroying then
  begin
    if FActiveCommandGroup = nil then
    begin
      FIsAddingCommands := true;
      while (FUndoPtr < FCommandGroupCol.Count - 1) do
        FCommandGroupCol.LastItem.Free;

      FCommandGroupCol.AddItem(Group);
    end
    else
      FActiveCommandGroup.Add(Group);

    FActiveCommandGroup := Group;
  end
  else
    Group.Free;
end;

procedure TRakHistory.NewGroupStop;
var
  MerkGroup: TRakCommandGroup;
begin
  if not IsUnRedoing and IsAddingCommands and not IsDestroying then
  begin
    MerkGroup := FActiveCommandGroup;
    FActiveCommandGroup := FActiveCommandGroup.CommandGroup;

    if MerkGroup.Count = 0 then
    begin
      MerkGroup.Free;
      if FActiveCommandGroup = nil then
      begin
        FIsAddingCommands := false;
        if Assigned(FOnCommandGroupDismissed) then
          FOnCommandGroupDismissed(self);
      end;
    end
    else
    begin
      MerkGroup.AfterAdded;

      if FActiveCommandGroup = nil then
      begin
        FIsAddingCommands := false;
        if FMaxCount > 0 then
          FreeOldest(Count - FMaxCount);

        ChangeFinished;
      end;
    end;
  end;
end;

procedure TRakHistory.ChangeFinished;
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
var
  Data: TRakBinaryStreamData;
  Stream: TMemoryStream;
  CopyDataStruct: TCopyDataStruct;
{$ENDIF}
begin
  FUndoPtr := FCommandGroupCol.Count - 1;

  if Assigned(FOnCommandGroupAdded) then
    FOnCommandGroupAdded(self);

  // Jetzt kommt nur noch Code frs Debugging
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if (FMonitorID > 0) and (Application.MainForm <> nil) then
  begin
    Data := TRakBinaryStreamData.Create;
    try
      Data.ApplicationName := HISTOY_CHANGED_APPLICATIONNAME;
      SaveToBinaryTag(Data.AddTag(HISTOY_CHANGED_TAGNAME));
      Stream := TMemoryStream.Create;
      try
        Data.SaveToStream(Stream);
        CopyDataStruct.dwData := DWDATA_URM_HISTORY_CHANGED;
        CopyDataStruct.cbData := Stream.Size;
        CopyDataStruct.lpData := Stream.Memory;
        SendMessage(FMonitorHandle, WM_COPYDATA, Application.MainForm.Handle, longint(@CopyDataStruct));
      finally
        Stream.Free;
      end;
    finally
      Data.Free;
    end;
  end;
{$ENDIF}
end;

function TRakHistory.Redo: boolean;
var
  Cg: TRakCommandGroup;
begin
  AbortAdding;
  Stati.Add(TRakStatusRedoing);
  try
    Result := false;
    if FUndoPtr < FCommandGroupCol.Count - 1 then
    begin
      inc(FUndoPtr);
      Cg := FCommandGroupCol[FUndoPtr];
      Result := Cg.Redo;
      Result := Cg.AfterDone(false) and Result;
    end;
  finally
    Stati.Delete(TRakStatusRedoing);
  end;

  FOnAfterRedo.FireEvent(self);
end;

function TRakHistory.Undo: boolean;
var
  Cg: TRakCommandGroup;
begin
  AbortAdding;
  Stati.Add(TRakStatusUndoing);
  try
    Result := false;
    if FUndoPtr >= 0 then
    begin
      Cg := FCommandGroupCol[FUndoPtr];
      Result := Cg.Undo;
      Result := Cg.AfterDone(true) and Result;
      dec(FUndoPtr);
    end;
  finally
    Stati.Delete(TRakStatusUndoing);
  end;

  FOnAfterUndo.FireEvent(self);
end;

constructor TRakHistory.Create(AOwner: TComponent);
  procedure FindMonitor;
  begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
    FMonitorHandle := FindWindow('TFrmUndoRedoMonitor', nil);
    if FMonitorHandle > 0 then
      FMonitorID := SendMessage(FMonitorHandle, URM_REGISTER_HISTORY, 0, 0);
{$ENDIF}
  end;
begin
  inherited;
  FindMonitor;
  FStati := TRakStati.Create;
  CreateNow;
end;

destructor TRakHistory.Destroy;
  procedure DisconnectMonitor;
  begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
    if (FMonitorID > 0) then
    begin
      PostMessage(FMonitorHandle, URM_UNREGISTER_HISTORY, 0, FMonitorID);
      FMonitorID := 0;
    end;
{$ENDIF}
  end;
begin
  Stati.Add(TRakStatusDestroying);
  DisconnectMonitor;
  DestroyNow;
  FreeTObject(FStati);
  inherited;
end;

procedure TRakHistory.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  List: IRakBinaryTagList;
  i: integer;
  cg: TRakCommandGroup;
  SubTag: TRakBinaryTag;
begin
  FCommandGroupCol.FreeAll;
  List := Tag.FindTagsByName('BaseGroup');
  for i := 0 to List.Count - 1 do
  begin
    SubTag := List.Items(i);
    cg := FCommandGroupCol.Add;
    cg.LoadFromBinaryTag(SubTag);
  end;

  FUndoPtr := Tag.AttributeAsInteger('UndoPtr', UndoPtr);
end;

procedure TRakHistory.SaveToBinaryTag(Tag: TRakBinaryTag);
var
  i: integer;
begin
  TRakBinaryLongint.AddToTag(Tag, 'UndoPtr', UndoPtr);
  for i := 0 to FCommandGroupCol.Count - 1 do
    FCommandGroupCol[i].SaveToBinaryTag(Tag.AddTag('BaseGroup'));
end;

class function TRakHistory.FindBestClass(
  ClassNameList: string): TRakCustomCommandClass;
var
  i, t: integer;
  List: TStringList;
  Klasse: string;
begin
  Result := nil;
  List := ListToNewTStringList(ClassNameList);
  try
    i := 0;
    while (Result = nil) and (i < List.Count) do
    begin
      Klasse := List[i];
      for t := 0 to RegisteredCustomCommandClasses.Count - 1 do
      begin
        if RegisteredCustomCommandClasses[t].ClassName = Klasse then
        begin
          Result := TRakCustomCommandClass(RegisteredCustomCommandClasses[t]);
          break;
        end;
      end;
      inc(i);
    end;
  finally
    List.Free;
  end;
end;

class procedure TRakHistory.RegisterCustomCommand(
  TheClass: TRakCustomCommandClass);
begin
  if RegisteredCustomCommandClasses.IndexOf(TheClass) = -1 then
    RegisteredCustomCommandClasses.Add(TheClass);
end;

{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
function TRakHistory.FindByCmdId(CmdId: longint): TRakCustomCommand;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    Result := CommandGroup[i].FindByCmdId(CmdId);
    inc(i);
  end;
end;
{$ENDIF}

function TRakHistory.GetIgnoreAddings: boolean;
begin
  Result := Stati.Exists(TRakStatusIgnoreAddings);
end;

procedure TRakHistory.SetIgnoreAddings(const Value: boolean);
begin
  if Value then
    Stati.Add(TRakStatusIgnoreAddings)
  else
    Stati.Delete(TRakStatusIgnoreAddings);
end;

procedure TRakHistory.SetMaxCount(const Value: integer);
begin
  if Value >= 0 then
  begin
    FMaxCount := Value;
    if FMaxCount > 0 then
    begin
      if Count > FMaxCount then
        FreeOldest(Count - FMaxCount);
    end;
  end;
end;

procedure TRakHistory.FreeOldest(FreeCounter: integer);
begin
  if not IsUnRedoing and not IsAddingCommands and not IsDestroying then
  begin
    if FreeCounter > 0 then
    begin
      if FreeCounter >= FCommandGroupCol.Count then
        Clear
      else
      begin
        FCommandGroupCol.FreeOldest(FreeCounter);
        dec(FUndoPtr, FreeCounter);
      end;
    end;
  end;
end;

function TRakHistory.GetIsDestroying: boolean;
begin
  Result := Stati.ExistsInherited(TRakStatusDestroying);
end;

procedure TRakHistory.AbortAdding;
begin
  while FActiveCommandGroup <> nil do
    FActiveCommandGroup.Abort;
end;

procedure TRakHistory.AbortUntil(UntilGroup: TRakCommandGroup;
  OnlySubGroups: boolean);
begin
  while (FActiveCommandGroup <> nil) and (FActiveCommandGroup <> UntilGroup) do
    FActiveCommandGroup.Abort;

  if not OnlySubGroups and (FActiveCommandGroup <> nil) then
    UntilGroup.Abort;
end;

{ TRakCustomCommand }

procedure TRakCustomCommand.AfterAdded;
begin
end;

function TRakCustomCommand.AfterDone(Undo: boolean): boolean;
begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if History.FMonitorID > 0 then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_AFTER_COMMAND, History.FMonitorID, longint(self));
{$ENDIF}
  Result := true;
end;

procedure TRakCustomCommand.CanRedo(var DoIt, Done: boolean);
begin
end;

procedure TRakCustomCommand.CanUndo(var DoIt, Done: boolean);
begin
end;

destructor TRakCustomCommand.Destroy;
begin
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
    FreeTObject(DataAsTag);
{$ENDIF}
  inherited;
end;

function TRakCustomCommand.GetCommandGroup: TRakCommandGroup;
begin
  Result := nil;
  if Collection is TRakCommandCol then
    Result := TRakCommandCol(Collection).CommandGroup;
end;

function TRakCustomCommand.GetHistory: TRakHistory;
begin
  Result := nil;
  if Collection is TRakCommandCol then
    Result := TRakCommandCol(Collection).History;
end;

function TRakCustomCommand.GetTopCommandGroup: TRakCommandGroup;
var
  c: TRakCommandGroup;
begin
  Result := nil;
  c := CommandGroup;
  if c <> nil then
    Result := c.TopCommandGroup
  else if self is TRakCommandGroup then
    Result := TRakCommandGroup(self);
end;

procedure TRakCustomCommand.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
    CmdId := Tag.AttributeAsInteger('CmdId', 0);
    FreeTObject(DataAsTag);
    DataAsTag := TRakBinaryTag.Create;
    DataAsTag.Assign(Tag);
    DataAsTag.FreeAll;            
{$ENDIF}
end;

function TRakCustomCommand.Redo: boolean;
var
  DoIt: boolean;
begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if History.FMonitorID > 0 then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_COMMAND, History.FMonitorID, longint(self));
{$ENDIF}

  RedoUndoSubCommand(self, true, true);

  DoIt := true;
  Result := false;
  CanRedo(DoIt, Result);

  if DoIt then
    Result := RedoNow;

{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if not Result and (History.FMonitorID > 0) then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_FAILED, History.FMonitorID, longint(self));
{$ENDIF}

  RedoUndoSubCommand(self, true, false);
end;

procedure TRakCustomCommand.RedoUndoSubCommand(Command: TRakCustomCommand;
  Redo, Before: boolean);
begin
  if CommandGroup <> nil then
    CommandGroup.RedoUndoSubCommand(Command, Redo, Before);
end;

procedure TRakCustomCommand.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  TRakBinaryLongint.AddToTag(Tag, 'CmdId', longint(self));
end;

function TRakCustomCommand.Undo: boolean;
var
  DoIt: boolean;
begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if History.FMonitorID > 0 then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_COMMAND, History.FMonitorID, longint(self));
{$ENDIF}

  RedoUndoSubCommand(self, false, true);

  DoIt := true;
  Result := false;
  CanUndo(DoIt, Result);

  if DoIt then
    Result := UndoNow;

{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if not Result and (History.FMonitorID > 0) then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_FAILED, History.FMonitorID, longint(self));
{$ENDIF}

  RedoUndoSubCommand(self, false, false);
end;

{ TRakCommand }

procedure TRakCommand.CanRedo(var DoIt, Done: boolean);
var
  Hist: TRakHistory;
begin
  Hist := History;
  if (Hist <> nil) and Assigned(Hist.FOnUndo) then
    Hist.FOnRedo(self, DoIt, Done);
end;

procedure TRakCommand.CanUndo(var DoIt, Done: boolean);
var
  Hist: TRakHistory;
begin
  Hist := History;
  if (Hist <> nil) and Assigned(Hist.FOnUndo) then
    Hist.FOnUndo(self, DoIt, Done);
end;

function TRakCommand.CheckChangedObject(Obj: TObject): boolean;
begin
  Result := Obj <> nil;
end;

function TRakCommand.GetChangedObject: TObject;
begin
  Result := RakGlobalObjectManager.GetObjectOf(ChangedObjectName);
end;

procedure TRakCommand.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FChangedObjectName := Tag.AttributeAsString('FChangedObjectName', FChangedObjectName);
end;

procedure TRakCommand.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryAnsiString.AddToTag(Tag, 'FChangedObjectName', FChangedObjectName);
end;

procedure TRakCommand.SetChangedObjectName(Value: string);
begin
  FChangedObjectName := Value;
end;

{ TRakCommandGroup }

procedure TRakCommandGroup.Abort;
begin
  if History.IsAddingCommands then
  begin // Wenn das ndern nicht bereits abgeschlossen ist
    if History.ActiveCommandGroup <> self then
      raise ERakUndoRedoSynchronisationLost.Create('Undo/Redo-Synchronisation lost');

    AbortNow;

    History.NewGroupStop;
  end;
end;

procedure TRakCommandGroup.AbortNow;
begin
end;

procedure TRakCommandGroup.Add(Command: TRakCustomCommand);
begin
  FCommandCol.AddItem(Command);
end;

function TRakCommandGroup.AfterDone(Undo: boolean): boolean;
var
  i: integer;
  cc: TRakCustomCommand;
begin
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
  if History.FMonitorID > 0 then
    PostMessage(History.FMonitorHandle, URM_UNREDOING_COMMAND, History.FMonitorID, longint(self));
{$ENDIF}
  Result := true;
  for i := 0 to Count - 1 do
  begin
    cc := Items[i];
    Result := cc.AfterDone(Undo);
{$IFDEF _CONTACT_UNDO_REDO_DEBUGGING_MONITOR_}
    if not Result and (History.FMonitorID > 0) then
      PostMessage(History.FMonitorHandle, URM_UNREDOING_AFTER_FAILED, History.FMonitorID, longint(cc));
{$ENDIF}
  end;
end;

constructor TRakCommandGroup.Create;
begin
  inherited;

  FCommandCol := TRakCommandCol.Create;
  FCommandCol.FCommandGroup := self;
end;

destructor TRakCommandGroup.Destroy;
begin
  FreeTObject(FCommandCol);

  inherited;
end;

{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
function TRakCommandGroup.FindByCmdId(CmdId: longint): TRakCustomCommand;
var
  i: integer;
  cc: TRakCustomCommand;
begin
  if self.CmdId = CmdId then
    Result := self
  else
  begin
    Result := nil;
    i := 0;
    while (Result = nil) and (i < Count) do
    begin
      cc := Items[i];
      if cc is TRakCommandGroup then
        Result := TRakCommandGroup(cc).FindByCmdId(CmdId)
      else if cc.CmdId = CmdId then
        Result := cc;
        
      inc(i);
    end;
  end;
end;
{$ENDIF}

function TRakCommandGroup.GetCount: integer;
begin
  Result := FCommandCol.Count;
end;

function TRakCommandGroup.GetItems(Idx: integer): TRakCustomCommand;
begin                  
  Result := FCommandCol[Idx];
end;

function TRakCommandGroup.GetMenueText: string;
begin
  Result := 'unknown change';
end;

procedure TRakCommandGroup.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FCommandCol.LoadFromBinaryTag(Tag);
{$IFDEF _RAK_UNDO_REDO_DEBUG_INFO_}
  RealMenueText := Tag.AttributeAsString('MenueText', '');
{$ENDIF}
end;

function TRakCommandGroup.RedoNow: boolean;
var
  i: integer;
  cc: TRakCustomCommand;
begin
  Result := true;
  for i := 0 to Count - 1 do
  begin
    cc := Items[i];
    Result := cc.Redo and Result;
  end;
end;

procedure TRakCommandGroup.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FCommandCol.SaveToBinaryTag(Tag);
  TRakBinaryAnsiString.AddToTag(Tag, 'MenueText', MenueText);
end;

procedure TRakCommandGroup.SetMenueText(const Value: string);
begin
  raise Exception.Create('TRakCommandGroup.SetMenueText must not be called');
end;

function TRakCommandGroup.UndoNow: boolean;
var
  i: integer;
  cc: TRakCustomCommand;
begin
  Result := true;
  for i := Count - 1 downto 0 do
  begin
    cc := Items[i];
    Result := cc.Undo and Result;
  end;
end;



{ TRakCommandCol }

function TRakCommandCol.GetHistory: TRakHistory;
begin
  Result := nil;
  if CommandGroup <> nil then
    Result := CommandGroup.History;
end;

function TRakCommandCol.GetItems(Idx: integer): TRakCustomCommand;
begin
  Result := TRakCustomCommand(inherited Items[Idx]);
end;

procedure TRakCommandCol.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  List: IRakBinaryTagList;
  i: integer;
  cc: TRakCustomCommand;
  SubTag: TRakBinaryTag;
  Klassen: string;
  CmdClass: TRakCustomCommandClass;
begin
  FreeAll;
  List := Tag.FindTagsByName('SubCmd');
  for i := 0 to List.Count - 1 do
  begin
    SubTag := List.Items(i);
    Klassen := SubTag.AttributeAsString('ClassList', '');
    CmdClass := TRakHistory.FindBestClass(Klassen);
    if CmdClass <> nil then
    begin
      cc := CmdClass.Create;
      AddItem(cc);
      cc.LoadFromBinaryTag(SubTag);
    end;
  end;
end;

procedure TRakCommandCol.SaveToBinaryTag(Tag: TRakBinaryTag);
var
  i: integer;
  cc: TRakCustomCommand;
  SubTag: TRakBinaryTag;
begin
  for i := 0 to Count - 1 do
  begin
    SubTag := Tag.AddTag('SubCmd');
    cc := Items[i];
    TRakBinaryAnsiString.AddToTag(SubTag, 'ClassList', ClassHierarchyAsList(cc.ClassType, TRakCustomCommand));
    cc.SaveToBinaryTag(SubTag);
  end;
end;

{ TRakCommandGroupCol }

function TRakCommandGroupCol.Add: TRakCommandGroup;
begin
  Result := TRakCommandGroup(BasicAdd(TRakCommandGroup));
end;

procedure TRakCommandGroupCol.FreeOldest(FreeCounter: integer);
var
  I: integer;
begin
  if FreeCounter > 0 then
  begin
    for I := 1 to FreeCounter do
    begin
      if Count > 0 then
        Items[0].Free;
    end;
  end;
end;

function TRakCommandGroupCol.GetHistory: TRakHistory;
begin
  Result := FHistory;
end;

function TRakCommandGroupCol.GetItems(Idx: integer): TRakCommandGroup;
begin
  Result := TRakCommandGroup(inherited Items[Idx]);
end;

procedure TRakCommandGroupCol.ItemDeleted(DelItem: TRakCollectionItem;
  OldIdx: integer);
begin
  inherited;
  if History.FUndoPtr >= Count then
    History.FUndoPtr := Count -1;
end;

{ TRakHcObjRef }

constructor TRakHcObjRef.Init(Value: TObject);
begin
  Create;
  FValue := Value;
end;

procedure TRakHcObjRef.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FValue := TObject(Tag.AttributeAsInteger('Value', 0));
end;

procedure TRakHcObjRef.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'Value', longint(FValue));
end;

{ TRakHcStringNew }

constructor TRakHcStringNew.Init(Value: string);
begin
  Create;
  FValue := Value;
end;

procedure TRakHcStringNew.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FValue := Tag.AttributeAsString('Value', FValue);
end;

procedure TRakHcStringNew.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryAnsiString.AddToTag(Tag, 'Value', FValue);
end;

{ TRakHcIntegerNew }

constructor TRakHcIntegerNew.Init(Value: integer);
begin
  Create;
  FValue := Value;
end;

procedure TRakHcIntegerNew.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FValue := Tag.AttributeAsInteger('Value', FValue);
end;

procedure TRakHcIntegerNew.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'Value', longint(FValue));
end;

{ TRakHcIdxObj }

constructor TRakHcIdxObj.Init(Idx: integer; Value: TObject);
begin
  Create;
  FIdx := Idx;
  FValue := Value;
end;

destructor TRakHcIdxObj.Destroy;
begin
  FValue.Free;
  inherited;
end;

procedure TRakHcIdxObj.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FIdx := Tag.AttributeAsInteger('Idx', FIdx);
  FValue := TObject(Tag.AttributeAsInteger('Value', 0));
end;

procedure TRakHcIdxObj.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  TRakBinaryLongint.AddToTag(Tag, 'Idx', longint(FIdx));
  TRakBinaryLongint.AddToTag(Tag, 'Value', longint(FValue));
end;

{ TRakHcIdxStringNew }

constructor TRakHcIdxStringNew.Init(Idx: integer; Value: string);
begin
  Create;
  FIdx := Idx;
  FValue := Value;
end;

procedure TRakHcIdxStringNew.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FIdx := Tag.AttributeAsInteger('Idx', FIdx);
  FValue := Tag.AttributeAsString('Value', FValue);
end;

procedure TRakHcIdxStringNew.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  TRakBinaryLongint.AddToTag(Tag, 'Idx', FIdx);
  TRakBinaryAnsiString.AddToTag(Tag, 'Value', FValue);
end;

{ TRakHcIdxIntegerNew }

constructor TRakHcIdxIntegerNew.Init(Idx, Value: integer);
begin
  Create;
  FIdx := Idx;
  FValue := Value;
end;

procedure TRakHcIdxIntegerNew.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FIdx := Tag.AttributeAsInteger('Idx', FIdx);
  FValue := Tag.AttributeAsInteger('Value', FValue);
end;

procedure TRakHcIdxIntegerNew.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  TRakBinaryLongint.AddToTag(Tag, 'Idx', FIdx);
  TRakBinaryLongint.AddToTag(Tag, 'Value', FValue);
end;

{ TRakHcIdxString }

constructor TRakHcIdxString.Init(Idx: integer; OldValue, NewValue: string);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxInteger }

constructor TRakHcIdxInteger.Init(Idx, OldValue, NewValue: integer);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxExtended }

constructor TRakHcIdxExtended.Init(Idx: integer; OldValue,
  NewValue: extended);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxBoolean }

constructor TRakHcIdxBoolean.Init(Idx: integer; NewValue: boolean);
begin
  inherited Init(NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxTDateTime }

constructor TRakHcIdxTDateTime.Init(Idx: integer; OldValue,
  NewValue: TDateTime);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxTColor }

constructor TRakHcIdxTColor.Init(Idx: integer; OldValue, NewValue: TColor);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakHcIdxBinaryTag }

constructor TRakHcIdxBinaryTag.Init(Idx: integer; Value: TRakBinaryTag);
begin
  Create;

  FIdx := Idx;
  FValue := Value;
end;

destructor TRakHcIdxBinaryTag.Destroy;
begin
  FreeTObject(FValue);

  inherited;
end;

procedure TRakHcIdxBinaryTag.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FIdx := Tag.AttributeAsInteger('Idx', FIdx);
end;

procedure TRakHcIdxBinaryTag.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

  TRakBinaryLongint.AddToTag(Tag, 'Idx', FIdx);
end;

{ TRakHcBinaryTag }

constructor TRakHcBinaryTag.Init(Value: TRakBinaryTag);
begin
  Create;
  FValue := Value;
end;

destructor TRakHcBinaryTag.Destroy;
begin
  FreeTObject(FValue);

  inherited;
end;

procedure TRakHcBinaryTag.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
end;

procedure TRakHcBinaryTag.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
end;

{ TRakHcChangeData }

constructor TRakHcChangeData.Init(Value: TRakCommand);
begin
  Create;
  FValue := Value;
end;

destructor TRakHcChangeData.Destroy;
begin
  FValue.Free;
  inherited;
end;

procedure TRakHcChangeData.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FValue := TRakCommand(Tag.AttributeAsInteger('Value', 0));
end;

procedure TRakHcChangeData.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'Value', longint(FValue));
end;

{ TRakHcBigChange }

constructor TRakHcBigChange.Init(OldValue, NewValue: TRakBinaryTag);
begin
  Create;
  FOldValue := OldValue;
  FNewValue := NewValue;
end;

destructor TRakHcBigChange.Destroy;
begin
  FOldValue.Free;
  FNewValue.Free;

  inherited;
end;

function TRakHcBigChange.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

function TRakHcBigChange.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcBigChange.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
end;

procedure TRakHcBigChange.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;

end;

{ TRakHcExtendedNew }

constructor TRakHcExtendedNew.Init(Value: extended);
begin
  Create;

  FValue := Value;
end;


procedure TRakHcExtendedNew.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FValue := Tag.AttributeAsFloat('Value', FValue);
end;

procedure TRakHcExtendedNew.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryExtended.AddToTag(Tag, 'Value', FValue);
end;

{ TRakHcString }

constructor TRakHcString.Init(OldValue, NewValue: string);
begin
  Create;
  FOldValue := OldValue;
  FNewValue := NewValue;
end;

procedure TRakHcString.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := Tag.AttributeAsString('OldValue', FOldValue);
  FNewValue := Tag.AttributeAsString('NewValue', FNewValue);
end;

function TRakHcString.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcString.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryAnsiString.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryAnsiString.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcString.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcInteger }

constructor TRakHcInteger.Init(OldValue, NewValue: integer);
begin
  Create;

  FOldValue := OldValue;
  FNewValue := NewValue;
end;

procedure TRakHcInteger.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := Tag.AttributeAsInteger('OldValue', FOldValue);
  FNewValue := Tag.AttributeAsInteger('NewValue', FNewValue);
end;

function TRakHcInteger.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcInteger.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryLongint.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcInteger.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcExtended }

constructor TRakHcExtended.Init(OldValue, NewValue: extended);
begin
  Create;

  FOldValue := OldValue;
  FNewValue := NewValue;
end;

procedure TRakHcExtended.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := Tag.AttributeAsFloat('OldValue', FOldValue);
  FNewValue := Tag.AttributeAsFloat('NewValue', FNewValue);
end;

function TRakHcExtended.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcExtended.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryExtended.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryExtended.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcExtended.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcTDateTime }

constructor TRakHcTDateTime.Init(OldValue, NewValue: TDateTime);
begin
  Create;

  FOldValue := OldValue;
  FNewValue := NewValue;
end;

procedure TRakHcTDateTime.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := Tag.AttributeAsFloat('OldValue', FOldValue);
  FNewValue := Tag.AttributeAsFloat('NewValue', FNewValue);
end;

function TRakHcTDateTime.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcTDateTime.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryExtended.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryExtended.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcTDateTime.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcBoolean }

constructor TRakHcBoolean.Init(NewValue: boolean);
begin
  Create;
  FNewValue := NewValue;
end;

procedure TRakHcBoolean.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FNewValue := Tag.AttributeAsBoolean('NewValue', FNewValue);
end;

function TRakHcBoolean.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcBoolean.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryBoolean.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcBoolean.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, not FNewValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcTColor }

constructor TRakHcTColor.Init(OldValue, NewValue: TColor);
begin
  Create;

  FOldValue := OldValue;
  FNewValue := NewValue;
end;


procedure TRakHcTColor.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := Tag.AttributeAsInteger('OldValue', FOldValue);
  FNewValue := Tag.AttributeAsInteger('NewValue', FNewValue);
end;

function TRakHcTColor.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcTColor.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryLongint.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcTColor.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcChangeObjId }


function TRakHcChangeObjId.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  Result := Obj <> nil;
  if Result then
    RakGlobalObjectManager.ChangeIdOf(Obj, FValue);
end;

function TRakHcChangeObjId.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := GetObject;
  Result := Obj <> nil;
  if Result then
    RakGlobalObjectManager.ChangeIdOf(Obj, ChangedObjectName);
end;

{ TRakHcObjId }

function TRakHcObjId.GetObject: TObject;
begin
  Result := RakGlobalObjectManager.GetObjectOf(FValue);
end;

constructor TRakHcObjId.InitByObjekt(Obj: TObject; NewObjId: string);
var
  ObjId: string;
begin
  ObjId := RakGlobalObjectManager.GetIdOf(Obj);
  if ObjId = '' then
    raise Exception.Create('TRakHcObjId.InitByObjekt mit unbekannten Objekt aufgeruden.');

  Init(NewObjId);

  ChangedObjectName := ObjId;
end;

{ TRakHcCreateGlobalObject }

function TRakHcCreateGlobalObject.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := CreateObject;
  Result := Obj <> nil;
  if Result then
    Result := RakGlobalObjectManager.ChangeIdOf(Obj, FValue);
end;

function TRakHcCreateGlobalObject.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := RakGlobalObjectManager.GetObjectOf(FValue);
  Result := Obj <> nil;
  if Result then
    Obj.Free;
end;

{ TRakHcDestroyGlobalObject }

function TRakHcDestroyGlobalObject.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := RakGlobalObjectManager.GetObjectOf(FValue);
  Result := Obj <> nil;
  if Result then
    Obj.Free;
end;

function TRakHcDestroyGlobalObject.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := CreateObject;
  Result := Obj <> nil;
  if Result then
    Result := RakGlobalObjectManager.ChangeIdOf(Obj, FValue);
end;

{ TRakHcTExtPoint }

constructor TRakHcTExtPoint.Init(OldValue, NewValue: TExtPoint);
begin
  Create;

  FOldValue := OldValue;
  FNewValue := NewValue;
end;

procedure TRakHcTExtPoint.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue.X := Tag.AttributeAsFloat('OldValueX', FOldValue.X);
  FOldValue.Y := Tag.AttributeAsFloat('OldValueY', FOldValue.Y);
  FNewValue.X := Tag.AttributeAsFloat('NewValueX', FNewValue.X);
  FNewValue.Y := Tag.AttributeAsFloat('NewValueY', FNewValue.Y);
end;

function TRakHcTExtPoint.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcTExtPoint.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryExtended.AddToTag(Tag, 'OldValueX', FOldValue.X);
  TRakBinaryExtended.AddToTag(Tag, 'OldValueY', FOldValue.Y);
  TRakBinaryExtended.AddToTag(Tag, 'NewValueX', FNewValue.X);
  TRakBinaryExtended.AddToTag(Tag, 'NewValueY', FNewValue.Y);
end;

function TRakHcTExtPoint.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

{ TRakHcIdxTExtPoint }

constructor TRakHcIdxTExtPoint.Init(Idx: integer; OldValue,
  NewValue: TExtPoint);
begin
  inherited Init(OldValue, NewValue);
  FIdx := Idx;
end;

{ TRakCGCommon }

function TRakCGCommon.GetMenueText: string;
begin
  Result := FMenuText;
end;

constructor TRakCGCommon.Init(MenueText: string);
begin
  Create;
  FMenuText := MenueText;
end;

procedure TRakCGCommon.SetMenueText(const Value: string);
begin
  FMenuText := Value;
end;

{ TRakHcSourceDestinationIdx }

constructor TRakHcSourceDestinationIdx.Init(SrcIdx, DstIdx: integer);
begin
  Create;

  FSrcIdx := SrcIdx;
  FDstIdx := DstIdx;
end;

procedure TRakHcSourceDestinationIdx.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FSrcIdx := Tag.AttributeAsInteger('SrcIdx', SrcIdx);
  FDstIdx := Tag.AttributeAsInteger('DstIdx', DstIdx);
end;

function TRakHcSourceDestinationIdx.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    Move(Obj, SrcIdx, DstIdx, Result);
  end
  else
    Result := false;
end;

procedure TRakHcSourceDestinationIdx.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryLongint.AddToTag(Tag, 'SrcIdx', SrcIdx);
  TRakBinaryLongint.AddToTag(Tag, 'DstIdx', DstIdx);
end;

function TRakHcSourceDestinationIdx.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    Move(Obj, DstIdx, SrcIdx, Result);
  end
  else
    Result := false;
end;

{ TRakHcTShiftState }

constructor TRakHcTShiftState.Init(OldValue, NewValue: TShiftState);
begin
	Create;
  FOldvalue := OldValue;
  FNewvalue := NewValue;
end;

procedure TRakHcTShiftState.LoadFromBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  FOldValue := TRakBinaryTShiftState.ReadValue(Tag, 'OldValue', FOldValue);
  FNewValue := TRakBinaryTShiftState.ReadValue(Tag, 'NewValue', FNewValue);
end;

function TRakHcTShiftState.RedoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FNewValue, Result);
  end
  else
    Result := false;
end;

procedure TRakHcTShiftState.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  inherited;
  TRakBinaryTShiftState.AddToTag(Tag, 'OldValue', FOldValue);
  TRakBinaryTShiftState.AddToTag(Tag, 'NewValue', FNewValue);
end;

function TRakHcTShiftState.UndoNow: boolean;
var
  Obj: TObject;
begin
  Obj := ChangedObject;
  if CheckChangedObject(Obj) then
  begin
    Result := true;
    SetValue(Obj, FOldValue, Result);
  end
  else
    Result := false;
end;

initialization
  RegisteredCustomCommandClasses := TClassList.Create;

  TRakHistory.RegisterCustomCommand(TRakCommand);
  TRakHistory.RegisterCustomCommand(TRakCommandGroup);

  TRakHistory.RegisterCustomCommand(TRakHcObjRef);
  TRakHistory.RegisterCustomCommand(TRakHcChangeData);
  TRakHistory.RegisterCustomCommand(TRakHcStringNew);
  TRakHistory.RegisterCustomCommand(TRakHcIntegerNew);
  TRakHistory.RegisterCustomCommand(TRakHcExtendedNew);
  TRakHistory.RegisterCustomCommand(TRakHcBinaryTag);
  TRakHistory.RegisterCustomCommand(TRakHcBigChange);
  TRakHistory.RegisterCustomCommand(TRakHcString);
  TRakHistory.RegisterCustomCommand(TRakHcInteger);
  TRakHistory.RegisterCustomCommand(TRakHcExtended);
  TRakHistory.RegisterCustomCommand(TRakHcBoolean);
  TRakHistory.RegisterCustomCommand(TRakHcTDateTime);
  TRakHistory.RegisterCustomCommand(TRakHcTColor);
  TRakHistory.RegisterCustomCommand(TRakHcIdxObj);
  TRakHistory.RegisterCustomCommand(TRakHcObjId);
  TRakHistory.RegisterCustomCommand(TRakHcChangeObjId);
  TRakHistory.RegisterCustomCommand(TRakHcCreateGlobalObject);
  TRakHistory.RegisterCustomCommand(TRakHcDestroyGlobalObject);
  TRakHistory.RegisterCustomCommand(TRakHcIdxStringNew);
  TRakHistory.RegisterCustomCommand(TRakHcIdxBinaryTag);
  TRakHistory.RegisterCustomCommand(TRakHcIdxIntegerNew);
  TRakHistory.RegisterCustomCommand(TRakHcIdxString);
  TRakHistory.RegisterCustomCommand(TRakHcIdxInteger);
  TRakHistory.RegisterCustomCommand(TRakHcIdxExtended);
  TRakHistory.RegisterCustomCommand(TRakHcIdxBoolean);
  TRakHistory.RegisterCustomCommand(TRakHcIdxTDateTime);
  TRakHistory.RegisterCustomCommand(TRakHcIdxTColor);

 finalization
   RegisteredCustomCommandClasses.Free;

end.
