unit RakBereich;

interface

uses
	Classes,
	RakCollection, RakDestroy;

type
  TRakDataBase = class(TObject, IRakDestroyOwner)
  private
    FDestroyOwner: TRakDestroyOwner;

  	FOnChanged: TNotifyEvent;
  protected
    procedure Changed; virtual;
    procedure DoDestroy; virtual;

    property DestroyOwner: TRakDestroyOwner read FDestroyOwner implements IRakDestroyOwner;
  public
  	constructor Create;
    // Den Destructor in abgeleiteten Objekten nicht mehr berladen, statt dessen die Prozedur DoDestroy berladen.
    destructor Destroy; override;

    procedure Assign(Source: TRakDataBase);
    function DoAssign(Source: TRakDataBase): boolean; virtual;//Der Rckgabewert gibt an, ob sich etwas gendert hat
    // DoAssign sollte nur von Komponentenentwicklern aufgerufen werden.
    // Normale Benutzer der Klasse sollte nur mit der Methode Assign arbeiten,
    // da diese auch das OnChanged-Ereignis auslsst.

  	property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  end;

  TRakBereich = class(TRakDataBase)
  private
    FXAufloesung: extended;
    FYAufloesung: extended;

    procedure SetXAufloesung(const Value: extended);
    procedure SetYAufloesung(const Value: extended);
  protected
  	FXPosition: extended;
  	FYPosition: extended;

    procedure DoDestroy; override;
  public
  	constructor Create;

    function DoAssign(Source: TRakDataBase): boolean; override;

    //Umrechnungsfunktionen fuer die Aufloesung
    function GetXPos(X: extended): integer;
    function GetYPos(Y: extended): integer;

    // Auflsung in Punkte/mm
    property XAufloesung: extended read FXAufloesung write SetXAufloesung;
    property YAufloesung: extended read FYAufloesung write SetYAufloesung;
  end;

  TRakRechteckBereich = class(TRakBereich)
  protected
  	FBreite: extended;
    FHoehe: extended;
  public
    function DoAssign(Source: TRakDataBase): boolean; override;
  end;

  TRakRechteckBereichPublic = class(TRakRechteckBereich)
  private
    procedure SetBreite(const Value: extended);
    procedure SetHoehe(const Value: extended);
    procedure SetLinks(const Value: extended);
    procedure SetOben(const Value: extended);
    procedure SetRechts(const Value: extended);
    procedure SetUnten(const Value: extended);
    function GetRechts: extended;
    function GetUnten: extended;
  public
  	property Links: extended read FXPosition write SetLinks;
  	property Oben: extended read FYPosition write SetOben;
  	property Breite: extended read FBreite write SetBreite;
  	property Hoehe: extended read FHoehe write SetHoehe;
  	property Rechts: extended read GetRechts write SetRechts;
  	property Unten: extended read GetUnten write SetUnten;
  end;

  TRakRechteckRaender = class(TRakRechteckBereich)
  private
    procedure SetLinks(const Value: extended);
    procedure SetOben(const Value: extended);
    procedure SetRechts(const Value: extended);
    procedure SetUnten(const Value: extended);
  public
  	property Links: extended read FXPosition write SetLinks;
  	property Oben: extended read FYPosition write SetOben;
  	property Rechts: extended read FBreite write SetRechts;
  	property Unten: extended read FHoehe write SetUnten;
  end;

  TRakBVAlign = (bvaNone, bvaTop, bvaCenter, bvaClient, bvaAll, bvaBottom);
  TRakBHAlign = (bhaNone, bhaLeft, bhaCenter, bhaClient, bhaAll, bhaRight);
  TRakRechteckBereichAlign = class(TRakRechteckBereichPublic)
  private
    FHorzAlign: TRakBHAlign;
    FVertAlign: TRakBVAlign;
    procedure SetHorzAlign(const Value: TRakBHAlign);
    procedure SetVertAlign(const Value: TRakBVAlign);
  public
    function DoAssign(Source: TRakDataBase): boolean; override;

  	property VertAlign: TRakBVAlign read FVertAlign write SetVertAlign;
  	property HorzAlign: TRakBHAlign read FHorzAlign write SetHorzAlign;
  end;
{  TRakHorizontalBereich = class(TRakRechteckBereich)
  private
    procedure SetHoehe(const Value: extended);
  public
  	property Hoehe: extended read FHoehe write SetHoehe;
  end;

  TRakVertikalBereich = class(TRakRechteckBereich)
  private
    procedure SetBreite(const Value: extended);
  public
  	property Breite: extended read FBreite write SetBreite;
  end;

  TRakLinkerRand = class(TRakVertikalBereich)
  private
    procedure SetLinks(const Value: extended);
  public
  	property Links: extended read FXPosition write SetLinks;
  end;

  TRakRechterRand = class(TRakVertikalBereich)
  private
    procedure SetRechts(const Value: extended);
  public
  	property Rechts: extended read FXPosition write SetRechts;
  end;

  TRakObererRand = class(TRakHorizontalBereich)
  private
    procedure SetOben(const Value: extended);
  public
  	property Oben: extended read FYPosition write SetOben;
  end;

  TRakUntererRand = class(TRakHorizontalBereich)
  private
    procedure SetUnten(const Value: extended);
  public
  	property Unten: extended read FYPosition write SetUnten;
  end;

  TRakRaender = class
  private
    FLinks: TRakLinkerRand;
    FOben: TRakObererRand;
    FRechts: TRakRechterRand;
    FUnten: TRakUntererRand;
  public
  	constructor Create;
    destructor Destroy; override;

  	property Links: TRakLinkerRand read FLinks;
    property Oben: TRakObererRand read FOben;
    property Rechts: TRakRechterRand read FRechts;
    property Unten: TRakUntererRand read FUnten;
  end;}

implementation
(*
{ TRakVertikalBereich }

procedure TRakVertikalBereich.SetBreite(const Value: extended);
begin
  if FBreite <> Value then
  begin
	  FBreite := Value;
    Changed;
  end;
end;

{ TRakHorizontalBereich }

procedure TRakHorizontalBereich.SetHoehe(const Value: extended);
begin
  if FHoehe <> Value then
  begin
	  FHoehe := Value;
    Changed;
  end;
end;
  *)
{ TRakDataBase }

procedure TRakDataBase.Assign(Source: TRakDataBase);
begin
	if DoAssign(Source) then
	  Changed;
end;

procedure TRakDataBase.Changed;
begin
	if Assigned(FOnChanged) then
  	FOnChanged(self);
end;

constructor TRakDataBase.Create;
begin
	inherited;

  FDestroyOwner := TRakDestroyOwner.Create(self, self);
end;

destructor TRakDataBase.Destroy;
begin
	DestroyOwner.CheckDestroy;

  DoDestroy;

  DestroyOwner.Free;

  inherited;
end;

function TRakDataBase.DoAssign(Source: TRakDataBase): boolean;
begin
	Result := false;
end;

procedure TRakDataBase.DoDestroy;
begin
	DestroyOwner.CheckDestroy;
end;

(*
{ TRakLinkerRand }

procedure TRakLinkerRand.SetLinks(const Value: extended);
begin
  if FXPosition <> Value then
  begin
	  FXPosition := Value;
    Changed;
  end;
end;

{ TRakRechterRand }

procedure TRakRechterRand.SetRechts(const Value: extended);
begin
  if FXPosition <> Value then
  begin
	  FXPosition := Value;
    Changed;
  end;
end;

{ TRakObererRand }

procedure TRakObererRand.SetOben(const Value: extended);
begin
  if FYPosition <> Value then
  begin
	  FYPosition := Value;
    Changed;
  end;
end;

{ TRakUntererRand }

procedure TRakUntererRand.SetUnten(const Value: extended);
begin
  if FYPosition <> Value then
  begin
	  FYPosition := Value;
    Changed;
  end;
end;

{ TRakRaender }

constructor TRakRaender.Create;
begin
	inherited;

  FLinks := TRakLinkerRand.Create;
  FOben := TRakObererRand.Create;
  FRechts := TRakRechterRand.Create;
  FUnten := TRakUntererRand.Create;
end;

destructor TRakRaender.Destroy;
begin
  FLinks.Free;
  FOben.Free;
  FRechts.Free;
  FUnten.Free;

  inherited;
end;*)

{ TRakRechteckBereichPublic }

function TRakRechteckBereichPublic.GetRechts: extended;
begin
	Result := Links + Breite;
end;

function TRakRechteckBereichPublic.GetUnten: extended;
begin
	Result := Oben + Hoehe;
end;

procedure TRakRechteckBereichPublic.SetBreite(const Value: extended);
begin
  if FBreite <> Value then
  begin
	  FBreite := Value;
    Changed;
  end;
end;

procedure TRakRechteckBereichPublic.SetHoehe(const Value: extended);
begin
  if FHoehe <> Value then
  begin
	  FHoehe := Value;
    Changed;
  end;
end;

procedure TRakRechteckBereichPublic.SetLinks(const Value: extended);
begin
  if FXPosition <> Value then
  begin
	  FXPosition := Value;
    Changed;
  end;
end;

procedure TRakRechteckBereichPublic.SetOben(const Value: extended);
begin
  if FYPosition <> Value then
  begin
	  FYPosition := Value;
    Changed;
  end;
end;

procedure TRakRechteckBereichPublic.SetRechts(const Value: extended);
begin
  if Rechts <> Value then
  begin
	  FBreite := Value-Links;
    Changed;
  end;
end;

procedure TRakRechteckBereichPublic.SetUnten(const Value: extended);
begin
  if Hoehe <> Value then
  begin
	  FHoehe := Value-Oben;
    Changed;
  end;
end;

{ TRakRechteckBereich }

function TRakRechteckBereich.DoAssign(Source: TRakDataBase): boolean;
var
	Src: TRakRechteckBereich;
begin
	Result := inherited DoAssign(Source);

  if Source is TRakRechteckBereich then
  begin
    Src := TRakRechteckBereich(Source);

    Result := Result or (FBreite <> Src.FBreite) or (FHoehe <> Src.FHoehe);

    FBreite := Src.FBreite;
    FHoehe := Src.FHoehe;
  end;
end;

{ TRakRechteckRaender }

procedure TRakRechteckRaender.SetLinks(const Value: extended);
begin
  if FXPosition <> Value then
  begin
	  FXPosition := Value;
    Changed;
  end;
end;

procedure TRakRechteckRaender.SetOben(const Value: extended);
begin
  if FYPosition <> Value then
  begin
	  FYPosition := Value;
    Changed;
  end;
end;

procedure TRakRechteckRaender.SetRechts(const Value: extended);
begin
  if FBreite <> Value then
  begin
	  FBreite := Value;
    Changed;
  end;
end;

procedure TRakRechteckRaender.SetUnten(const Value: extended);
begin
  if FHoehe <> Value then
  begin
	  FHoehe := Value;
    Changed;
  end;
end;


{ TRakRechteckBereichAlign }

function TRakRechteckBereichAlign.DoAssign(Source: TRakDataBase): boolean;
var
	Src: TRakRechteckBereichAlign;
begin
	Result := inherited DoAssign(Source);

  if Source is TRakRechteckBereichAlign then
  begin
    Src := TRakRechteckBereichAlign(Source);

    Result := Result or (FHorzAlign <> Src.FHorzAlign) or (FVertAlign <> Src.FVertAlign);

    FHorzAlign := Src.FHorzAlign;
    FVertAlign := Src.FVertAlign;
  end;
end;

procedure TRakRechteckBereichAlign.SetHorzAlign(const Value: TRakBHAlign);
begin
  if FHorzAlign <> Value then
  begin
	  FHorzAlign := Value;
    Changed;
  end;
end;

procedure TRakRechteckBereichAlign.SetVertAlign(const Value: TRakBVAlign);
begin
  if FVertAlign <> Value then
  begin
	  FVertAlign := Value;
    Changed;
  end;
end;

{ TRakBereich }

constructor TRakBereich.Create;
begin
	inherited;

  FXAufloesung := 1;
  FYAufloesung := 1;
end;

function TRakBereich.DoAssign(Source: TRakDataBase): boolean;
var
	Src: TRakBereich;
begin
	Result := inherited DoAssign(Source);

  if Source is TRakBereich then
  begin
  	Src := TRakBereich(Source);
    Result :=  Result
            or (FXAufloesung <> Src.FXAufloesung)
            or (FYAufloesung <> Src.FYAufloesung)
            or (FXPosition <> Src.FXPosition)
            or (FYPosition <> Src.FYPosition);

    FXAufloesung := Src.FXAufloesung;
    FYAufloesung := Src.FYAufloesung;
    FXPosition := Src.FXPosition;
    FYPosition := Src.FYPosition;
  end;
end;

procedure TRakBereich.DoDestroy;
begin
	inherited;

end;

function TRakBereich.GetXPos(X: extended): integer;
begin
	Result := round(X * XAufloesung);
end;

function TRakBereich.GetYPos(Y: extended): integer;
begin
	Result := round(Y * YAufloesung);
end;

procedure TRakBereich.SetXAufloesung(const Value: extended);
begin
  FXAufloesung := Value;
end;

procedure TRakBereich.SetYAufloesung(const Value: extended);
begin
  FYAufloesung := Value;
end;

end.
