unit RakChange;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die unit RakChange stellt Klassen zur Verfgung, die ein
									allgemeines Konzept zur Implementierung eines
                  Undo/Redo-Mechanismuses implementieren.
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
	8.12.01 Neue Klasse TRakTopGroupChange
  				Wenn nderungen gruppiert werden, dann ist die Wurzel aller Grupierungen
          von der Klasse TRakTopGroupChange. Die Gruppierungen, die selbst gruppiert
          sind, sind weiterhin von der Klasse TRakGroupChange.
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.

interface

uses
	Sysutils, Graphics, Classes,
	miscfunc, MultiEvent, RakObject, RakCollection,
  RakBinaryStreamData;

type

	TRakStatusReUndoing = class(TRakBaseStatus);
	TRakStatusRedoing = class(TRakStatusReUndoing);
	TRakStatusUndoing = class(TRakStatusReUndoing);

  TRakChange = class;
  TRakGroupChange = class;
  TRakTopGroupChange = class;
  TRakChangeCol = class;
  TRakChangeData = class;

  TChangeEventObjectChanged = procedure (Sender: TObject;
    	Change: TRakChange) of object;
  TChangeEventRedo = procedure (Sender: TObject;
    	Change: TRakChange; var Done: boolean) of object;
  TChangeEventUndo = procedure (Sender: TObject;
    	Change: TRakChange; var Done: boolean) of object;

  TEventGroupChangeStart = procedure (Sender: TObject;
    	ChangeData: TRakChangeData) of object;
  TEventGroupChangeStop = procedure (Sender: TObject) of object;
  TEventAddChange = procedure (Sender: TObject;
    	ChangeData: TRakChangeData) of object;


{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakHistorie
// Beschreibung : TRakHistorie bildet das Kernstck des Undo/Redo-Mechanismuses.
                  Von TRakHistorie wird es in der Regel nur eine Instanz geben.
                  Diese Instanz speichert alle Daten, die ntig sind, um eine
                  nderung rckgngig zu machen bzw. die nderung erneut auszufhren.
                  Hinweis: Wahrscheinlich werde ich TRakHistorie irgendwann
                  				zu einer nicht-visuellen Komponente umwandeln

                  Eine neue nderung muss mit der Methode AddChange bekannt
                  gemacht werden. Wenn man mehrere nderungen gruppieren
                  mchte, so kann dies durch die Methoden GroupChangeStart
                  und GroupChangeStop gemacht werden. GroupChangeStart und
                  GroupChangeStop knnen auch geschachtelt werden.
                  Durch Aufruf der Methode Undo kann eine nderung
                  rckgngig gemacht werden. Das Gegenstck zu Unod ist Redo.
                  Zum Enablen und Beschriften von Undo/Redo-Menpunkten oder
                  ~Buttons kann man auf die Properties Count, Changes und UndoPtr
                  und das Ereignis OnObjectChanged zurckgreifen.
                  Die Ereignisse OnRedo und OnUndo werden von den Methoden
                  Undo und Redo ausgelst. Bei diesen Ereignissen muss die
                  eigentliche Arbeit des Rckgngigmachens von einer nderung
                  erledigt werden.
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
	8.12.01: Neue Events: OnAfterRedo und OnAfterUndo;
////////////////////////////////////////////////////////////////////////////////}
  TRakHistorie = class(TRakObject)
  private
    FOnObjectChanged: TChangeEventObjectChanged;
    FOnRedo: TChangeEventRedo;
    FOnUndo: TChangeEventUndo;
    FOnAfterRedo: TRakNotifyEvent;
    FOnAfterUndo: TRakNotifyEvent;
    function GetChanges(Idx: integer): TRakChange;
    function GetCount: integer;
    function GetUndoPtr: integer;
    function GetIsUnRedoing: boolean;
  protected
  	FChanges: TRakChangeCol;
	  FGrpChange: TRakGroupChange;

  	procedure CreateNow; override;
    procedure DestroyNow; override;

    procedure ObjectChanged(Change: TRakChange); virtual;
  public
  	// Mit Redo wird die zuletzt rckgngig gemachte nderung erneut durchgefhrt
    function Redo: boolean;
    // Undo macht die letzte nderung rckgngig
    function Undo: boolean;
    // Clear lscht alle Daten zu allen nderungen
    procedure Clear;

    // Mit AddChange wird eine neue nderung bekannt gemacht. Sollten Daten fr den Aufruf von Redo bereitstehen, dann werden diese gelscht
    // ChangedData erwartet ein Objekt, das alle Daten enthlt, die zur Rckgngigmachung und erneuten Durchfhrung der nderung bentigt werden.
    // ChangedData geht in den Besitzt von TRakHistorie ber!
    // An Quelle kann irgendetwas bergeben werden, dass einen selbst spter in die Lage versetzt entscheiden zu knnen, wer die Fhigkeit besitzt die nderung rckgngig zu machen
    procedure AddChange(ChangeData: TRakChangeData;
    	ChangedObject: string; Quelle: string); virtual;
    // Mit GroupChangeStart wird eine neue Gruppe von nderungen begonnen. Alle folgenden AddChange aufrufen gehren dann zu dieser Gruppe
    procedure GroupChangeStart(ChangeData: TRakChangeData; Quelle: string); virtual;
    // Mit GroupChangeStop muss eine Gruppe von nderungen abgeschlossen werden. Sollte die Gruppe leer sein, dann wird sie weider verworfen
    procedure GroupChangeStop; virtual;

    // Count gibt die Anzahl aller nderungen in der ersten Ebene zurck. nderungen einer Gruppe werden nicht gezhlt. Es wird nur die Gruppe selbst mitgezhlt
    property Count: integer read GetCount;
    // Mit Changes kann auf alle nderungen der ersten Ebene zugegriffen werden ( 0 <= Idx < Count )
    property Changes[Idx: integer]: TRakChange read GetChanges; default;
    // UndoPtr zeigt an, welche nderung beim nchsten Aufruf von Undo betroffen wre. UndoPtr+1 wre vom Aufruf von Redo betroffen
  	property UndoPtr: integer read GetUndoPtr;
    property IsUnRedoing: boolean read GetIsUnRedoing;

    // OnObjectChanged wird ausgelst, wenn eine neue nderung auf erster Ebene hinzugekommen ist
    property OnObjectChanged: TChangeEventObjectChanged read FOnObjectChanged
    	write FOnObjectChanged;
    // OnRedo wird ausgelst, wenn die Funktion Redo aufgerufen wurde. Es werden die Daten bergeben, mit dessen Hilfe die nderung erneut ausgefhrt werden muss
  	property OnRedo: TChangeEventRedo read FOnRedo write FOnRedo;
    // OnUndo wird ausgelst, wenn die Funktion Undo aufgerufen wurde. Es werden die Daten bergeben, mit dessen Hilfe die nderung rckgngig gemacht werden muss
  	property OnUndo: TChangeEventUndo read FOnUndo write FOnUndo;
    // OnAfterRedo wird am Ende der Funktion Redo ausgelst
    property OnAfterRedo: TRakNotifyEvent read FOnAfterRedo;
    // OnAfterUndo wird am Ende der Funktion Undo ausgelst
    property OnAfterUndo: TRakNotifyEvent read FOnAfterUndo;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakChangeData
// Beschreibung : TRakChangeData ist die Basisklasse aller Klassen, welche die
                  Daten speichern mssen, mit denen eine nderung rckgngig
                  gemacht, bzw. erneut durchgefhrt werden kann.
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
	8.12.01: 	Neue Methoden AfterAdded, AfterRedo, BeforeRedo, AfterUndo,
  						BeforeUndo und RedoUndoSubChange
  					Neues Property Change
////////////////////////////////////////////////////////////////////////////////}
  TRakChangeData = class
  private
    FChange: TRakChange;
  protected
  	FMenueText: string;

    // AfterAdded wird aufgerufen, nachdem alle Informationen zu einer nderung bekannt sind
    procedure AfterAdded; virtual;

  	// AfterRedo wird aufgerufen, nachdem das Redo durchgefhrt wurde
  	procedure AfterRedo; virtual;
  	// BeforeRedo wird aufgerufen, bevor das Redo durchgefhrt wird
  	procedure BeforeRedo; virtual;
  	// AfterUndo wird aufgerufen, nachdem das Undo durchgefhrt wurde
  	procedure AfterUndo; virtual;
  	// BeforeUndo wird aufgerufen, bevor das Undo durchgefhrt wird
  	procedure BeforeUndo; virtual;

    // RedoUndoSubChange gibt dem kompletten Pfad die Chance irgendwie auf einzelne Redo/Undo-Aktionen zu reagieren
    // Change ist die nderung mit der etwas gemacht wird
    // Redo gibt an, ob ein Redo oder Undo auf die nderung durchgefhrt wird
    // Before gibt an, ob die nderung gleich durchgefhrt wird oder bereits erfolgte
    procedure RedoUndoSubChange(Change: TRakChange; Redo, Before: boolean); virtual;

    function GetMenueText: string; virtual;
    procedure SetMenueText(const Value: string); virtual;
  public
  	// MenueText, kann z.B. im Hint von Redo/Undo-Buttons angezeigt werden kann.
    property MenueText: string read GetMenueText write SetMenueText;
    // Change ist die nderung zu der dieses TRakChangeData gehrt
    property Change: TRakChange read FChange;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakChange
// Beschreibung : TRakChange ist die Klasse, die alle Informationen zu einer
                  einzelnen nderung enthlt
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
	8.12.01: Neue Properties TopGroup und Historie
////////////////////////////////////////////////////////////////////////////////}
  TRakChange = class(TRakCollectionItem)
  private
    FObjektName: string;
    FWann: string;
    FWer: string;

    FOnDestroy: TRakNotifyEventVerwalter;
    FData: TRakChangeData;
    FQuelle: string;
    procedure SetData(const Value: TRakChangeData);
    function GetParentGroup: TRakGroupChange;
    function GetTopGroup: TRakTopGroupChange;
  protected
    // read-Methode
  	function GetHistorie: TRakHistorie; virtual;
  public
    constructor Create; override;
  	destructor Destroy; override;

    // GetMenueText ist normalerweise gleich Data.MenueText
  	function GetMenueText: string; virtual;

    // ObjektName ist der eindeutige Name des Objektes, das gendert wurde. Man sollte nie eine Referenz
    // auf das Objekt speichern, da das Objekt gelscht werden knnte und durch ein
    // Undo erneut erzeugt werden mu. Es wre groer Zufall, wenn die Referenz
    // danach auf das richtige Objekt zeigen wrde
    property ObjektName: string read FObjektName write FObjektName;
    // Data enthlt alle Daten, die bentigt werden, damit die nderung rckgngig gemacht werden kann, bzw. erneut durchgefhrt werden kann
    // Was gendert wurde erkennt man in der Regel am Typ von Data, z.B. if Data is TDateZuDieserAenderung.
  	property Data: TRakChangeData read FData write SetData;
    // In Wann steht der Zeitpunkt der nderung
  	property Wann: string read FWann write FWann;
    // In Wer kann man z.B. den angemeldeten Benutzer eintragen
  	property Wer: string read FWer write FWer;
    // Quelle legt fest, wer ber die Fhigkeit verfgt, die nderung rckgngig zu machen
  	property Quelle: string read FQuelle write FQuelle;

    // ParentGroup gibt die Gruppe zurck, durch die diese nderung gruppiert ist; nil, falls die nderung nicht gruppiert ist
    // Manchmal kann es sein, dass eine einzelne nderung nicht in der Lage ist alle bentigten Daten zu speichern. Dies kann dann evtl.
    // durch mehreren nderungen realisiert werden, wobei diese ihre Daten mit Hilfe von ParentGroup austauschen
    property ParentGroup: TRakGroupChange read GetParentGroup;
    // nderungen knnen beliebig gruppiert werden. Gruppierungen knnen auch gruppiert werden
    // TopGroup ist die oberste Gruppierung, ist also selbst nicht mehr gruppiert
    property TopGroup: TRakTopGroupChange read GetTopGroup;
    //Historie gibt die TRakHistorie an, von der diese TRakChange verwaltet wird
    property Historie: TRakHistorie read GetHistorie;

    // Mit OnDestroy kann man dafr sorgen, dass ein Objekt, dass eine Referenz auf die nderung hlt, mitbekommt, wenn sich diese lscht
    property OnDestroy: TRakNotifyEventVerwalter read FOnDestroy;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakGroupChange
// Beschreibung : TRakGroupChange ist die Klasse, die nderungen, auch Gruppen
									von nderungen, gruppieren kann.
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakGroupChange = class(TRakChange)
  private
    FGroup: TRakChangeCol;
  public
  	constructor Create; override;
    destructor Destroy; override;
    //Mit Group kann auf die gruppierten nderungen zugegriffen werden
  	property Group: TRakChangeCol read FGroup;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakChangeCol
// Beschreibung : TRakChangeCol sammelt nderungen und wird von TRakGroupChange
									und TRakHistorie verwendet
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakChangeCol = class(TRakCollection)
  private
  	FGroupChange: TRakGroupChange;

    function GetItems(Idx: integer): TRakChange;
  protected
  	FUndoPtr: integer;
    FRedoCleaning: integer;

    procedure ItemAdded(NewItem: TRakCollectionItem); override;
    procedure ItemDelete(DelItem: TRakCollectionItem); override;
    procedure ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer); override;
  public
  	constructor Create; override;

    function AddTopGroup: TRakTopGroupChange; virtual;
    // Es wird eine neue Gruppe erzeugt
  	function AddGroup: TRakGroupChange; virtual;
    // Es wird eine Einzelnderung erzeugt
  	function AddSingle: TRakChange; virtual;

    // Redo gibt die nderung zurck, die erneut ausgefhrt werden soll
  	function Redo: TRakChange;
    // Undo gibt die nderung zurck, die erneut rckgngig gemacht werden soll
  	function Undo: TRakChange;

    // FreeAll gibt alle nderunge frei
    procedure FreeAll; override;

    // Mit Items kann auf die einzelnen nderungen zugegriffen werden (0 <= Idx < Count)
    property Items[Idx: integer]: TRakChange read GetItems; default;
    // UndoPtr gibt an, welche nderung zuletzt ausgefhrt wurde
  	property UndoPtr: integer read FUndoPtr;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakTopGroupChange
// Beschreibung : TRakTopGroupChange ist die Wurzel eines nderungsbaumes
// erstellt am  : 8.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
	TRakTopGroupChange = class(TRakGroupChange)
  private
  	FHistorie: TRakHistorie;
  protected
  	function GetHistorie: TRakHistorie; override;
  end;

//******************************************************************************
// Ab hier kommen Klassen, die von TRakChangeData abgeleitet sind
// und als Basisklassen fr eigenen TRakChangeData-Klassen dienen knnen


// TRakChangeDataObj speichert eine Referenz auf ein Objekt
// Man sollte sich genau berlegen, ob die Referenz wirklich immer gltig ist!!!!!!
  TRakChangeDataObj = class(TRakChangeData)
  public
  	Obj: TObject;

  	constructor Create(AObj: TObject);
    destructor Destroy; override;
  end;

  TRakChangeDataChangeData = class(TRakChangeData)
  public
  	Obj: TRakChangeData;

  	constructor Create(AObj: TRakChangeData);
    destructor Destroy; override;
  end;

	TRakChangeDataStringNew = class(TRakChangeData)
  public
    str: string;

  	constructor Create(aStr: string);
  end;

	TRakChangeDataBinaryTag = class(TRakChangeData)
  public
    Tag: TRakBinaryTag;

  	constructor Create(Tag: TRakBinaryTag);
    destructor Destroy; override;
  end;

  TRakChangeBigChange = class(TRakChangeData)
  public
    Alt: TRakBinaryTag;
    Neu: TRakBinaryTag;

  	constructor Create(Alt, Neu: TRakBinaryTag);
    destructor Destroy; override;
  end;

  TRakChangeDataIntegerNew = class(TRakChangeData)
  public
    int: integer;

  	constructor Create(aInt: integer);
  end;

  TRakChangeDataExtendedNew = class(TRakChangeData)
  public
    Value: extended;

  	constructor Create(val: extended);
  end;

	TRakChangeDataString = class(TRakChangeData)
  public
  	Alt: string;
    Neu: string;

  	constructor Create(Alt, Neu: string);
  end;

	TRakChangeDataInteger = class(TRakChangeData)
  public
  	Alt: integer;
    Neu: integer;

  	constructor Create(Alt, Neu: integer);
  end;

	TRakChangeDataExtended = class(TRakChangeData)
  public
  	Alt: extended;
    Neu: extended;

  	constructor Create(Alt, Neu: extended);
  end;

	TRakChangeDataBoolean = class(TRakChangeData)
  public
    Neu: boolean; // Alt = not Neu

  	constructor Create(Neu: boolean);
  end;

	TRakChangeDataTDateTime = class(TRakChangeData)
  public
  	Alt: TDateTime;
    Neu: TDateTime;

  	constructor Create(Alt, Neu: TDateTime);
  end;

	TRakChangeDataTColor = class(TRakChangeData)
  public
  	Alt: TColor;
    Neu: TColor;

  	constructor Create(Alt, Neu: TColor);
  end;

  TRakChangeDataIdxObj = class(TRakChangeData)
  public
  	Idx: integer;
  	Obj: TObject;

  	constructor Create(Idx: integer; AObj: TObject);
    destructor Destroy; override;
  end;

	TRakChangeDataIdxStringNew = class(TRakChangeData)
  public
  	Idx: integer;
    str: string;

  	constructor Create(Idx: integer; aStr: string);
  end;

	TRakChangeDataIdxBinaryTag = class(TRakChangeData)
  public
  	Idx: integer;
    Tag: TRakBinaryTag;

  	constructor Create(Idx: integer; Tag: TRakBinaryTag);
    destructor Destroy; override;
  end;

  TRakChangeDataIdxIntegerNew = class(TRakChangeData)
  public
  	Idx: integer;
    int: integer;

  	constructor Create(Idx: integer; aInt: integer);
  end;

	TRakChangeDataIdxString = class(TRakChangeData)
  public
  	Idx: integer;
  	Alt: string;
    Neu: string;

  	constructor Create(Idx: integer; Alt, Neu: string);
  end;

	TRakChangeDataIdxInteger = class(TRakChangeData)
  public
  	Idx: integer;
  	Alt: integer;
    Neu: integer;

  	constructor Create(Idx: integer; Alt, Neu: integer);
  end;

	TRakChangeDataIdxExtended = class(TRakChangeData)
  public
  	Idx: integer;
  	Alt: extended;
    Neu: extended;

  	constructor Create(Idx: integer; Alt, Neu: extended);
  end;

	TRakChangeDataIdxBoolean = class(TRakChangeData)
  public
  	Idx: integer;
    Neu: boolean;

  	constructor Create(Idx: integer; Neu: boolean);
  end;

	TRakChangeDataIdxTDateTime = class(TRakChangeData)
  public
  	Idx: integer;
  	Alt: TDateTime;
    Neu: TDateTime;

  	constructor Create(Idx: integer; Alt, Neu: TDateTime);
  end;

	TRakChangeDataIdxTColor = class(TRakChangeData)
  public
  	Idx: integer;
  	Alt: TColor;
    Neu: TColor;

  	constructor Create(Idx: integer; Alt, Neu: TColor);
  end;

implementation


{ TRakChangeCol }

function TRakChangeCol.AddGroup: TRakGroupChange;
begin
  Result := TRakGroupChange(BasicAdd(TRakGroupChange));
end;

function TRakChangeCol.AddSingle: TRakChange;
begin
  Result := TRakChange(BasicAdd(TRakChange));
end;

function TRakChangeCol.AddTopGroup: TRakTopGroupChange;
begin
  Result := TRakTopGroupChange(BasicAdd(TRakTopGroupChange));
end;

constructor TRakChangeCol.Create;
begin
	inherited;

	FUndoPtr := -1;
end;

procedure TRakChangeCol.FreeAll;
begin
  inherited;

	FUndoPtr := -1;
end;

function TRakChangeCol.GetItems(Idx: integer): TRakChange;
begin
  Result := TRakChange(inherited Items[Idx]);
end;

procedure TRakChangeCol.ItemAdded(NewItem: TRakCollectionItem);
var
  i: integer;
begin
  inherited;
  inc(FRedoCleaning);
  try
    for i := Count - 2 downto UndoPtr + 1 do
    	Items[i].Free;
  finally
	  dec(FRedoCleaning);
  end;
	FUndoPtr := Count - 1;
end;

procedure TRakChangeCol.ItemDelete(DelItem: TRakCollectionItem);
var
  i: integer;
begin
  for i := Count - 1 downto DelItem.Index + 1 do
  begin
    if ((FRedoCleaning = 0) and (i < Count)) or ((FRedoCleaning > 0) and (i < Count-1)) then
      Items[i].Free;
  end;
end;

procedure TRakChangeCol.ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer);
begin
  inherited;
	FUndoPtr := Count - 1;
end;

function TRakChangeCol.Redo: TRakChange;
begin
	if UndoPtr < Count - 1 then
  begin
    inc(FUndoPtr);
    Result := Items[UndoPtr];
  end
  else
  begin
  	Result := nil;
  end;
end;

function TRakChangeCol.Undo: TRakChange;
begin
	if UndoPtr >= 0 then
  begin
    Result := Items[UndoPtr];
    dec(FUndoPtr);
  end
  else
  begin
  	Result := nil;
  end;
end;

{ TRakGroupChange }

constructor TRakGroupChange.Create;
begin
  inherited;

	FGroup := TRakChangeCol.Create;
  FGroup.FGroupChange := self;
end;

destructor TRakGroupChange.Destroy;
begin
	FGroup.Free;

  inherited;
end;

{ TRakChange }

constructor TRakChange.Create;
begin
  inherited;

	FWann := FloatToStr(Now);

	FOnDestroy := TRakNotifyEventVerwalter.Create;
end;

destructor TRakChange.Destroy;
begin
  OnDestroy.FireEvent(self); // Das Ereignis auslsen
  FOnDestroy.Free;
  FOnDestroy := nil;

  Data := nil;

  inherited;
end;

function TRakChange.GetParentGroup: TRakGroupChange;
begin
	Result := TRakChangeCol(Collection).FGroupChange;
end;

function TRakChange.GetMenueText: string;
begin
	Result := Data.MenueText;
end;

procedure TRakChange.SetData(const Value: TRakChangeData);
begin
  if FData <> Value then
  begin
		FreeTObject(FData);
  	FData := Value;
  end;
end;

function TRakChange.GetTopGroup: TRakTopGroupChange;
begin
	if ParentGroup = nil then
  begin
    if self is TRakTopGroupChange then
    	Result := TRakTopGroupChange(self)
    else
    	Result := nil;
  end
  else
		Result := ParentGroup.TopGroup;
end;

function TRakChange.GetHistorie: TRakHistorie;
var
	Grp: TRakTopGroupChange;
begin
	Grp := TopGroup;
  if Grp <> nil then
  	Result := Grp.Historie
  else
  	Result := nil;
end;

{ TRakHistorie }

procedure TRakHistorie.AddChange(ChangeData: TRakChangeData;
  ChangedObject: string; Quelle: string);
var
	Change: TRakChange;
begin
	if not Stati.ExistsInherited(TRakStatusReUndoing) then
  begin
  	if FGrpChange = nil then
		  Change := FChanges.AddSingle
    else
    	Change := FGrpChange.Group.AddSingle;

    Change.ObjektName := ChangedObject;
    ChangeData.FChange := Change;
    Change.Data := ChangeData;
    Change.Quelle := Quelle;

    ChangeData.AfterAdded;

    if FGrpChange = nil then
    	ObjectChanged(Change);
  end
  else
  begin
    ChangeData.Free;
  end;
end;

procedure TRakHistorie.Clear;
begin
	FChanges.FreeAll;
end;

procedure TRakHistorie.CreateNow;
begin
  inherited;

  FOnAfterRedo := TRakNotifyEvent.Create;
  FOnAfterUndo := TRakNotifyEvent.Create;
	FChanges := TRakChangeCol.Create;
end;

procedure TRakHistorie.DestroyNow;
begin
	FChanges.Free;
  FOnAfterRedo.Free;
  FOnAfterUndo.Free;

  inherited;
end;

function TRakHistorie.GetChanges(Idx: integer): TRakChange;
begin
	Result := FChanges[Idx];
end;

function TRakHistorie.GetCount: integer;
begin
	Result := FChanges.Count;
end;

function TRakHistorie.GetIsUnRedoing: boolean;
begin
	Result := Stati.ExistsInherited(TRakStatusReUndoing);
end;

function TRakHistorie.GetUndoPtr: integer;
begin
	Result := FChanges.UndoPtr;
end;

procedure TRakHistorie.GroupChangeStart(ChangeData: TRakChangeData;
	Quelle: string);
begin
	if not Stati.ExistsInherited(TRakStatusReUndoing)then
  begin
  	if FGrpChange = nil then
    begin
      FGrpChange := FChanges.AddTopGroup;
      TRakTopGroupChange(FGrpChange).FHistorie := self;
    end
    else
    	FGrpChange := FGrpChange.Group.AddGroup;

    ChangeData.FChange := FGrpChange;
    FGrpChange.Data := ChangeData;
    FGrpChange.Quelle := Quelle;
  end
  else
	  ChangeData.Free;
end;

procedure TRakHistorie.GroupChangeStop;
var
	MerkGroup: TRakGroupChange;
begin
	if not Stati.ExistsInherited(TRakStatusReUndoing) and (FGrpChange <> nil) then
  begin
  	MerkGroup := FGrpChange;
  	FGrpChange := FGrpChange.ParentGroup;

    if MerkGroup.Group.Count = 0 then
      MerkGroup.Free
    else if FGrpChange = nil then
    begin
	    MerkGroup.Data.AfterAdded;
      
      ObjectChanged(MerkGroup);
    end;
  end;
end;

procedure TRakHistorie.ObjectChanged(Change: TRakChange);
begin
  if Assigned(FOnObjectChanged) then
    FOnObjectChanged(self, FGrpChange);
end;

function TRakHistorie.Redo: boolean;
	function RedoNow(CustChange: TRakChange): boolean;
  var
    i: integer;
    Group: TRakChangeCol;
  begin
  	CustChange.Data.BeforeRedo;

    if CustChange is TRakGroupChange then
    begin
      Group := TRakGroupChange(CustChange).Group;
      Result := Group.Count > 0;

      for i := 0 to Group.Count - 1 do
        Result :=  RedoNow(Group[i]) and Result;
    end
    else
      FOnRedo(self, CustChange, Result);

    CustChange.Data.AfterRedo;
  end;
var
	CustChange: TRakChange;
begin
	Stati.Add(TRakStatusRedoing);
  try
    Result := false;
    CustChange := FChanges.Redo;
    if (CustChange <> nil) and Assigned(FOnRedo) then
    	Result := RedoNow(CustChange);
  finally
		Stati.Delete(TRakStatusRedoing);
  end;

  FOnAfterRedo.FireEvent(self);
end;

function TRakHistorie.Undo: boolean;
	function UndoNow(CustChange: TRakChange): boolean;
  var
    i: integer;
    Group: TRakChangeCol;
  begin
  	CustChange.Data.BeforeUndo;

    if CustChange is TRakGroupChange then
    begin
      Group := TRakGroupChange(CustChange).Group;
      Result := Group.Count > 0;

      for i := Group.Count - 1 downto 0 do
        Result := UndoNow(Group[i]) and Result;

    end
    else
      FOnUndo(self, CustChange, Result);

    CustChange.Data.AfterUndo;
  end;
var
	CustChange: TRakChange;
begin
	Stati.Add(TRakStatusUndoing);
  try
    Result := false;
    CustChange := FChanges.Undo;
    if (CustChange <> nil) and Assigned(FOnUndo) then
    	Result := UndoNow(CustChange);
  finally
		Stati.Delete(TRakStatusUndoing);
  end;

  FOnAfterUndo.FireEvent(self);
end;

{ TRakChangeDataString }

constructor TRakChangeDataString.Create(Alt, Neu: string);
begin
	self.Alt := Alt;
	self.Neu := Neu;
end;

{ TRakChangeDataInteger }

constructor TRakChangeDataInteger.Create(Alt, Neu: integer);
begin
	self.Alt := Alt;
	self.Neu := Neu;
end;

{ TRakChangeDataExtended }

constructor TRakChangeDataExtended.Create(Alt, Neu: extended);
begin
	self.Alt := Alt;
	self.Neu := Neu;
end;

{ TRakChangeDataTDateTime }

constructor TRakChangeDataTDateTime.Create(Alt, Neu: TDateTime);
begin
	self.Alt := Alt;
	self.Neu := Neu;
end;

{ TRakChangeDataBoolean }

constructor TRakChangeDataBoolean.Create(Neu: boolean);
begin
	self.Neu := Neu;
end;

{ TRakChangeDataTColor }

constructor TRakChangeDataTColor.Create(Alt, Neu: TColor);
begin
	self.Alt := Alt;
	self.Neu := Neu;

  FMenueText := 'Farbe ndern';
end;

{ TRakChangeData }

procedure TRakChangeData.AfterAdded;
begin
end;

procedure TRakChangeData.AfterRedo;
begin
	RedoUndoSubChange(Change, true, false);
end;

procedure TRakChangeData.AfterUndo;
begin
	RedoUndoSubChange(Change, false, false);
end;

procedure TRakChangeData.BeforeRedo;
begin
	RedoUndoSubChange(Change, true, true);
end;

procedure TRakChangeData.BeforeUndo;
begin
	RedoUndoSubChange(Change, false, true);
end;

function TRakChangeData.GetMenueText: string;
begin
	Result := FMenueText;
end;

procedure TRakChangeData.RedoUndoSubChange(Change: TRakChange; Redo,
  Before: boolean);
var
	Owner: TRakChange;
begin
	Owner := self.Change;
  if Owner.ParentGroup <> nil then
		Owner.ParentGroup.Data.RedoUndoSubChange(Change, Redo, Before);
end;

procedure TRakChangeData.SetMenueText(const Value: string);
begin
	FMenueText := Value;
end;

{ TRakChangeDataObj }

constructor TRakChangeDataObj.Create(AObj: TObject);
begin
	inherited Create;
	Obj := AObj;
end;

destructor TRakChangeDataObj.Destroy;
begin
	Obj.Free;
  inherited;
end;

{ TRakChangeDataStringNew }

constructor TRakChangeDataStringNew.Create(aStr: string);
begin
	inherited Create;
	str := aStr;
end;

{ TRakChangeDataIntegerNew }

constructor TRakChangeDataIntegerNew.Create(aInt: integer);
begin
	inherited Create;
	int := aInt;
end;

{ TRakChangeDataIdxObj }

constructor TRakChangeDataIdxObj.Create(Idx: integer; AObj: TObject);
begin
	inherited Create;
  self.Idx := Idx;
	Obj := AObj;
end;

destructor TRakChangeDataIdxObj.Destroy;
begin
	Obj.Free;
  inherited;
end;

{ TRakChangeDataIdxStringNew }

constructor TRakChangeDataIdxStringNew.Create(Idx: integer; aStr: string);
begin
	inherited Create;
  self.Idx := Idx;
  str := aStr;
end;

{ TRakChangeDataIdxIntegerNew }

constructor TRakChangeDataIdxIntegerNew.Create(Idx, aInt: integer);
begin
	inherited Create;
  self.Idx := Idx;
  int := aInt;
end;

{ TRakChangeDataIdxString }

constructor TRakChangeDataIdxString.Create(Idx: integer; Alt, Neu: string);
begin
	inherited Create;
  self.Idx := Idx;
  self.Alt := Alt;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxInteger }

constructor TRakChangeDataIdxInteger.Create(Idx, Alt, Neu: integer);
begin
	inherited Create;
  self.Idx := Idx;
  self.Alt := Alt;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxExtended }

constructor TRakChangeDataIdxExtended.Create(Idx: integer; Alt,
  Neu: extended);
begin
	inherited Create;
  self.Idx := Idx;
  self.Alt := Alt;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxBoolean }

constructor TRakChangeDataIdxBoolean.Create(Idx: integer; Neu: boolean);
begin
	inherited Create;
  self.Idx := Idx;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxTDateTime }

constructor TRakChangeDataIdxTDateTime.Create(Idx: integer; Alt,
  Neu: TDateTime);
begin
	inherited Create;
  self.Idx := Idx;
  self.Alt := Alt;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxTColor }

constructor TRakChangeDataIdxTColor.Create(Idx: integer; Alt, Neu: TColor);
begin
	inherited Create;
  self.Idx := Idx;
  self.Alt := Alt;
  self.Neu := Neu;
end;

{ TRakChangeDataIdxBinaryTag }

constructor TRakChangeDataIdxBinaryTag.Create(Idx: integer; Tag: TRakBinaryTag);
begin
	inherited Create;

  self.Idx := Idx;
  self.Tag := Tag;
end;

destructor TRakChangeDataIdxBinaryTag.Destroy;
begin
	FreeTObject(Tag);

  inherited;
end;

{ TRakChangeDataBinaryTag }

constructor TRakChangeDataBinaryTag.Create(Tag: TRakBinaryTag);
begin
	inherited Create;
  self.Tag := Tag;
end;

destructor TRakChangeDataBinaryTag.Destroy;
begin
	FreeTObject(Tag);

  inherited;
end;

{ TRakChangeDataChangeData }

constructor TRakChangeDataChangeData.Create(AObj: TRakChangeData);
begin
	inherited Create;
	Obj := AObj;
end;

destructor TRakChangeDataChangeData.Destroy;
begin
	Obj.Free;
  inherited;
end;

{ TRakChangeBigChange }

constructor TRakChangeBigChange.Create(Alt, Neu: TRakBinaryTag);
begin
	inherited Create;
  self.Alt := Alt;
  self.Neu := Neu;
end;

destructor TRakChangeBigChange.Destroy;
begin
	Alt.Free;
	Neu.Free;

  inherited;
end;

{ TRakTopGroupChange }

function TRakTopGroupChange.GetHistorie: TRakHistorie;
begin
	Result := FHistorie;
end;

{ TRakChangeDataExtendedNew }

constructor TRakChangeDataExtendedNew.Create(val: extended);
begin
	inherited Create;

  Value := val;
end;

end.
