unit RakDatenObj;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit RakDatenObj stellt Basisklassen fr die Speicherung
                  persistenter Daten zur Verfgung.
                  Die persitenten Daten knnen in einem XML-Format
                  exportiert werden.
                  Ausserdem werden noch einige konkrete Datenklassen zur
                  Verfgung gestellt, die auch den Undo/Redo-Mechanismuses
                  aus der Unit RakChange untersttzen.
                  Dies sind Klassen zum Speichern von Pen-,Brush-,Font-
                  und Boundsdaten.
                  ACHTUNG: Zur Untersttzung des XML-Formates wird der
                  				XML-Parser XDOM von Dieter Khler in der Version 2.3.3.
                          bentigt. Hhere Versionen funktionieren wahrscheinlich
                          genauso gut. Diese wurden aber nicht von mir getestet.
                          Den Code des XML-Parsers kann man sich unter
                          "http://www.philo.de/homepage.htm" herunterladen
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.

interface

uses
	graphics, sysutils, classes,
  XDOM_2_3,
  WYSIWYGFont,
	misctype, miscfunc, RakObject, RakCollection, RakChange;

type
	TRakStatusDataChanging = class(TRakBaseStatus);
  TRakStatusXMLReading = class(TRakStatusDataChanging);

	TRakDatenObjClass = class of TRakDatenObj;
  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakDatenObjClass
  // Beschreibung : TRakDatenObjClass sit die Basisklasse aller Klassen, die
                    persitente Daten halten und diese in Form eines XML-Formates
                    exportieren und importieren knnen
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakDatenObj = class(TRakObject)
  private
    function GetAsXML: string;
    procedure SetAsXML(const Value: string);
  protected
  	FOwner: TObject;

    // AsXMLNow muss von abgeleiteten Klassen berscchrieben werden. Hier mssen die persistenten Daten in einem XML-Format zurckgegeben werden
  	function AsXMLNow: string; overload; virtual; abstract;
    // AsXMLNow muss von abgeleiteten Klassen berscchrieben werden. Hier mssen die persistenten Daten in einem XML-Format bernommen werden
  	procedure AsXMLNow(NewValues: string); overload; virtual; abstract;

    //read-Methode
    function GetOwner: TObject; virtual;
    //write-Methode
    procedure SetOwner(Value: TObject);
    procedure SetOwnerNow(Value: TObject); virtual;

    // Ermglicht abgeleiteten Klassen das ndern des Owner zu beeinflussen
    procedure OwnerChangeBefore(var NewOwner: TObject); virtual;
    // Ermglicht abgeleiteten Klassen auf einen genderten Owner zu reagieren
    procedure OwnerChangeAfter(OldOwner: TObject); virtual;

    // Der Eigentmer des Datenobjektes
  	property Owner: TObject read GetOwner write SetOwner;
  public
  	// Zugriff auf die persistenten Daten in Form von XML
    property AsXML: string read GetAsXML write SetAsXML;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataTPenMode
  // Beschreibung : TRakChangeDataTPenMode wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakChangeDataTPenMode = class(TRakChangeData)
  public
  	Alt: TPenMode;
    Neu: TPenMode;

  	constructor Create(Alt, Neu: TPenMode);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataTPenStyle
  // Beschreibung : TRakChangeDataTPenStyle wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakChangeDataTPenStyle = class(TRakChangeData)
  public
  	Alt: TPenStyle;
    Neu: TPenStyle;

  	constructor Create(Alt, Neu: TPenStyle);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataPenColor
  // Beschreibung : TRakChangeDataPenColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataPenColor = class(TRakChangeDataTColor)
  public
  	constructor Create(Alt, Neu: TColor);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataPenMmBreite
  // Beschreibung : TRakChangeDataPenMmBreite wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataPenMmBreite = class(TRakChangeDataExtended)
  public
  	constructor Create(Alt, Neu: extended);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakPenData
  // Beschreibung : TRakPenData hlt die Daten eines Stiftes zum Zeichnen,
                    wobei die Breite des Stiftes nicht in Pixel, sondern in
                    Millimeter angegeben wird
  									Auer dem XML-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakChange untersttzt
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakPenData = class(TRakDatenObj)
  private
    FColor: TColor;
    FMode: TPenMode;
    FStyle: TPenStyle;
    FmmBreite: extended;
    procedure SetColor(const Value: TColor);
    procedure SetMode(const Value: TPenMode);
    procedure SetStyle(const Value: TPenStyle);
    procedure SetMmBreite(const Value: extended);
  protected
  	procedure CreateNow; override;

  	function AsXMLNow: string; override;
  	procedure AsXMLNow(NewValues: string); override;

  	procedure ColorChangeAfter(OldValue: TColor); virtual;
  	procedure ModeChangeAfter(OldValue: TPenMode); virtual;
  	procedure StyleChangeAfter(OldValue: TPenStyle); virtual;
  	procedure MmBreiteChangeAfter(OldValue: extended); virtual;

    procedure DataChanged(ChangeData: TObject); virtual;
    function Redo(ChangeData: TObject): boolean; virtual;
    function Undo(ChangeData: TObject): boolean; virtual;
  public
    procedure SetPen(APen: TPen); virtual;

  	property Mode: TPenMode read FMode write SetMode;
    property Style: TPenStyle read FStyle write SetStyle;
    property Color: TColor read FColor write SetColor;
    property MmBreite: extended read FMmBreite write SetMmBreite;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataBrushColor
  // Beschreibung : TRakChangeDataBrushColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataBrushColor = class(TRakChangeDataTColor)
  public
  	constructor Create(Alt, Neu: TColor);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataTBrushStyle
  // Beschreibung : TRakChangeDataTBrushStyle wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakChangeDataTBrushStyle = class(TRakChangeData)
  public
  	Alt: TBrushStyle;
    Neu: TBrushStyle;

  	constructor Create(Alt, Neu: TBrushStyle);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakBrushData
  // Beschreibung : TRakBrushData hlt die Daten eines Pinseld zum Fllen.
  									Auer dem XML-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakChange untersttzt
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakBrushData = class(TRakDatenObj)
  private
    FStyle: TBrushStyle;
    FColor: TColor;
    procedure SetColor(const Value: TColor);
    procedure SetStyle(const Value: TBrushStyle);
  protected
  	procedure CreateNow; override;

  	function AsXMLNow: string; override;
  	procedure AsXMLNow(NewValues: string); override;
  	procedure ColorChangeAfter(OldValue: TColor); virtual;
  	procedure StyleChangeAfter(OldValue: TBrushStyle); virtual;

    procedure DataChanged(ChangeData: TObject); virtual;
    function Redo(ChangeData: TObject): boolean; virtual;
    function Undo(ChangeData: TObject): boolean; virtual;
  public
    procedure SetBrush(ABrush: TBrush); virtual;

    property Style: TBrushStyle read FStyle write SetStyle;
    property Color: TColor read FColor write SetColor;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataFontColor
  // Beschreibung : TRakChangeDataFontColor wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataFontColor = class(TRakChangeDataTColor)
  public
  	constructor Create(Alt, Neu: TColor);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataFontMmHoehe
  // Beschreibung : TRakChangeDataFontMmHoehe wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataFontMmHoehe = class(TRakChangeDataExtended)
  public
  	constructor Create(Alt, Neu: extended);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataFontName
  // Beschreibung : TRakChangeDataFontName wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakChangeDataFontName = class(TRakChangeDataString)
  public
  	constructor Create(Alt, Neu: string);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakChangeDataTFontStyles
  // Beschreibung : TRakChangeDataTFontStyles wird fr die Untersttzung
                    des Undo/Redo-Mechanismuses aus der Unit RakChange verwendet
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakChangeDataTFontStyles = class(TRakChangeData)
  public
  	Alt: TFontStyles;
    Neu: TFontStyles;

  	constructor Create(Alt, Neu: TFontStyles);
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakFontData
  // Beschreibung : TRakFontData hlt die Daten eines Fonts zum Schreiben,
                    wobei die Hehe des Fonts nicht in 72tel-Zoll sondern
                    in Millimeter angegeben wird
  									Auer dem XML-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakChange untersttzt
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakFontData = class(TRakDatenObj)
  private
    FMmHoehe: extended;
    FName: string;
    FColor: TColor;
    FStyle: TFontStyles;
    procedure SetColor(const Value: TColor);
    procedure SetMmHoehe(const Value: extended);
    procedure SetName(const Value: string);
    procedure SetStyle(const Value: TFontStyles);
  protected
  	procedure CreateNow; override;

  	function AsXMLNow: string; override;
  	procedure AsXMLNow(NewValues: string); override;
  	procedure ColorChangeAfter(OldValue: TColor); virtual;
  	procedure MmHoeheChangeAfter(OldValue: extended); virtual;
  	procedure NameChangeAfter(OldValue: string); virtual;
  	procedure StyleChangeAfter(OldValue: TFontStyles); virtual;

    procedure DataChanged(ChangeData: TRakChangeData); virtual;
    function Redo(ChangeData: TObject): boolean; virtual;
    function Undo(ChangeData: TObject): boolean; virtual;
  public
  	procedure Assign(Source: TRakFontData); virtual;

    function TextWidth(Text: string): extended; virtual;
    // SetFont setzt die eigenen Werte in AFont ein
	  procedure SetFont(AFont: TFont); virtual;
    // WrapText zerlegt Zeile so ein einzelne Zeilen, dass keine Zeile
    // breiter ist als MmBreite, wobei in jeder Zeile mindestens ein Buchstabe
    // sein muss. Beim Umbrechen wird versucht so umzubrechen, dass der
    // Text noch mglichst gut lesbar ist
		procedure WrapText(Zeile: string; MmBreite: extended; Zeilen: TStrings);

  	property Name: string read FName write SetName;
    property Style: TFontStyles read FStyle write SetStyle;
    property Color: TColor read FColor write SetColor;
    property MmHoehe: extended read FMmHoehe write SetMmHoehe;
  end;


	TRakChangeDataMmLinks = class(TRakChangeDataExtended);
	TRakChangeDataMmOben = class(TRakChangeDataExtended);
	TRakChangeDataMmBreite = class(TRakChangeDataExtended);
	TRakChangeDataMmHoehe = class(TRakChangeDataExtended);

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakMmBounds
  // Beschreibung : TRakMmBounds hlt die Millimeter-Koordinaten eines Rechteckes
  									Auer dem XML-Import/Export wird auch
                    der Redo/Undo-Mechanismus der Unit RakChange untersttzt
  // erstellt am  : 1.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
	TRakMmBounds = class(TRakDatenObj)
  private
    function GetMmRechts: extended;
    function GetMmUnten: extended;
    procedure SetMmBreite(const Value: extended);
    procedure SetMmHoehe(const Value: extended);
    procedure SetMmLinks(const Value: extended);
    procedure SetMmOben(const Value: extended);
    procedure SetMmRechts(const Value: extended);
    procedure SetMmUnten(const Value: extended);
    function GetMmHorzCenter: extended;
    function GetMmVertCenter: extended;
    procedure SetMmHorzCenter(const Value: extended);
    procedure SetMmVertCenter(const Value: extended);
  protected
    FMmLinks: extended;
    FMmOben: extended;
    FMmBreite: extended;
    FMmHoehe: extended;

  	function AsXMLNow: string; override;
  	procedure AsXMLNow(NewValues: string); override;

		procedure SetMmNow(Links, Oben, Breite, Hoehe: extended); virtual;
		procedure MmChangeBefore(var NewLinks, NewOben, NewBreite,
    	NewHoehe: extended); virtual;
		procedure MmChangeAfter(const OldLinks, OldOben, OldBreite,
    	OldHoehe: extended); virtual;

    procedure DataChanged(ChangeData: TRakChangeData); virtual;
    function Redo(ChangeData: TObject): boolean; virtual;
    function Undo(ChangeData: TObject): boolean; virtual;
  public
    function GetMmRect: TExtRect;
		procedure SetMm(Links, Oben, Breite, Hoehe: extended); overload;

    property MmLinks: extended read FMmLinks write SetMmLinks;
    property MmOben: extended read FMmOben write SetMmOben;
    property MmBreite: extended read FMmBreite write SetMmBreite;
    property MmHoehe: extended read FMmHoehe write SetMmHoehe;
    property MmRechts: extended read GetMmRechts write SetMmRechts;
    property MmUnten: extended read GetMmUnten write SetMmUnten;
    property MmHorzCenter: extended read GetMmHorzCenter write SetMmHorzCenter;
    property MmVertCenter: extended read GetMmVertCenter write SetMmVertCenter;
  end;
implementation

{ TRakDatenObj }

function TRakDatenObj.GetAsXML: string;
begin
	Result := '<' + ClassName + '>' + AsXMLNow + '</' + ClassName + '>';
end;

function TRakDatenObj.GetOwner: TObject;
begin
	Result := FOwner;
end;

procedure TRakDatenObj.OwnerChangeAfter(OldOwner: TObject);
begin

end;

procedure TRakDatenObj.OwnerChangeBefore(var NewOwner: TObject);
begin

end;

procedure TRakDatenObj.SetAsXML(const Value: string);
var
	name: string;
begin
	Stati.Add(TRakStatusXMLReading);
  try
    name := ClassName;
    if (Pos('<' + name +'>', Value) = 1) and (Right(Value, length(Name) + 3) = '</' + name + '>') then
      AsXMLNow(DeleteXMLTag(Value, name));
  finally
  	Stati.Delete(TRakStatusXMLReading);
  end;
end;

procedure TRakDatenObj.SetOwner(Value: TObject);
begin
	SetOwnerNow(Value);
end;

procedure TRakDatenObj.SetOwnerNow(Value: TObject);
var
	lclOld: TObject;
begin
	if Value <> FOwner then
  begin
  	OwnerChangeBefore(Value);

    lclOld := FOwner;
    FOwner := Value;

  	OwnerChangeAfter(lclOld);
  end;
end;

{ TRakPenData }

function TRakPenData.AsXMLNow: string;
var
  xdi: TDomImplementation;
	doc: TdomDocument;
  Base: TdomElement;
begin
  xdi := TDomImplementation.Create(nil);
  try
    doc := xdi.createDocument('Data', nil);
    Base := doc.documentElement;
    Base.SetAttribute('Version', '1');
    Base.SetAttribute('Mode', IntToStr(Ord(Mode)));
    Base.SetAttribute('Style', IntToStr(Ord(Style)));
    Base.SetAttribute('Color', IntToStr(Color));
    Base.SetAttribute('MmBreite', FloatToNormStr(MmBreite));

    Result := UTF16BEToUTF8Str(Base.Code, true);
  finally
  	xdi.Free;
  end;
end;

procedure TRakPenData.AsXMLNow(NewValues: string);
var
  xdi: TDomImplementation;
  xdp: TXmlToDomParser;
	doc: TdomDocument;
  Base: TdomElement;
begin
  xdi := TDomImplementation.Create(nil);
  xdp := TXmlToDomParser.Create(nil);
  try
    xdp.domImpl := xdi;
    doc := xdp.stringToDom(NewValues);

    Base := doc.documentElement;
    if Base.NodeName = 'Data' then
    begin
 			Mode := TPenMode(StrToIntDef(Base.GetAttribute('Mode'), Ord(Mode)));
 			Style := TPenStyle(StrToIntDef(Base.GetAttribute('Style'), Ord(Style)));
 			Color := StrToIntDef(Base.GetAttribute('Color'), Color);
 			MmBreite := NormStrToFloat(Base.GetAttribute('MmBreite'), MmBreite);
    end;
  finally
    xdp.Free;
    xdi.Free;
  end;
end;

procedure TRakPenData.ColorChangeAfter(OldValue: TColor);
begin
	DataChanged(TRakChangeDataPenColor.Create(OldValue, Color));
end;

procedure TRakPenData.CreateNow;
begin
  FColor := clBlack;
  FMode := pmCopy;
  FStyle := psSolid;
  FmmBreite := 0;
end;

procedure TRakPenData.DataChanged(ChangeData: TObject);
begin
	ChangeData.Free;
end;

procedure TRakPenData.MmBreiteChangeAfter(OldValue: extended);
begin
	DataChanged(TRakChangeDataPenMmBreite.Create(OldValue, MmBreite));
end;

procedure TRakPenData.ModeChangeAfter(OldValue: TPenMode);
begin
	DataChanged(TRakChangeDataTPenMode.Create(OldValue, Mode));
end;

function TRakPenData.Redo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataTPenMode then
  	Mode := TRakChangeDataTPenMode(ChangeData).Neu
	else if ChangeData is TRakChangeDataTPenStyle then
  	Style := TRakChangeDataTPenStyle(ChangeData).Neu
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Neu
	else if ChangeData is TRakChangeDataExtended then
  	MmBreite := TRakChangeDataExtended(ChangeData).Neu
  else
  	Result := false;
end;

procedure TRakPenData.SetColor(const Value: TColor);
var
	OldValue: TColor;
begin
  if FColor <> Value then
  begin
  	OldValue := FColor;

	  FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

procedure TRakPenData.SetMmBreite(const Value: extended);
var
	OldValue: extended;
begin
  if FMmBreite <> Value then
  begin
  	OldValue := FMmBreite;

	  FMmBreite := Value;

    MmBreiteChangeAfter(OldValue);
  end;
end;

procedure TRakPenData.SetMode(const Value: TPenMode);
var
	OldValue: TPenMode;
begin
  if FMode <> Value then
  begin
  	OldValue := FMode;

	  FMode := Value;

    ModeChangeAfter(OldValue);
  end;
end;

procedure TRakPenData.SetPen(APen: TPen);
begin
	APen.Mode := Mode;
  APen.Style := Style;
  APen.Color := Color;
  APen.Width := round(MmBreite);
end;

procedure TRakPenData.SetStyle(const Value: TPenStyle);
var
	OldValue: TPenStyle;
begin
  if FStyle <> Value then
  begin
  	OldValue := FStyle;

	  FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakPenData.StyleChangeAfter(OldValue: TPenStyle);
begin
	DataChanged(TRakChangeDataTPenStyle.Create(OldValue, Style));
end;

function TRakPenData.Undo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataTPenMode then
  	Mode := TRakChangeDataTPenMode(ChangeData).Alt
	else if ChangeData is TRakChangeDataTPenStyle then
  	Style := TRakChangeDataTPenStyle(ChangeData).Alt
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Alt
	else if ChangeData is TRakChangeDataExtended then
  	MmBreite := TRakChangeDataExtended(ChangeData).Alt
  else
  	Result := false;
end;

{ TRakBrushData }

function TRakBrushData.AsXMLNow: string;
var
  xdi: TDomImplementation;
	doc: TdomDocument;
  Base: TdomElement;
begin
  xdi := TDomImplementation.Create(nil);
  try
    doc := xdi.createDocument('Data', nil);
    Base := doc.documentElement;
    Base.SetAttribute('Version', '1');
    Base.SetAttribute('Style', IntToStr(Ord(Style)));
    Base.SetAttribute('Color', IntToStr(Color));

    Result := UTF16BEToUTF8Str(Base.Code, true);
  finally
  	xdi.Free;
  end;
end;

procedure TRakBrushData.AsXMLNow(NewValues: string);
var
  xdi: TDomImplementation;
  xdp: TXmlToDomParser;
	doc: TdomDocument;
  Base: TdomElement;
begin
  xdi := TDomImplementation.Create(nil);
  xdp := TXmlToDomParser.Create(nil);
  try
    xdp.domImpl := xdi;
    doc := xdp.stringToDom(NewValues);

    Base := doc.documentElement;
    if Base.NodeName = 'Data' then
    begin
 			Style := TBrushStyle(StrToIntDef(Base.GetAttribute('Style'), Ord(Style)));
 			Color := StrToIntDef(Base.GetAttribute('Color'), Color);
    end;
  finally
    xdp.Free;
    xdi.Free;
  end;
end;

procedure TRakBrushData.ColorChangeAfter(OldValue: TColor);
begin
	DataChanged(TRakChangeDataTColor.Create(OldValue, Color));
end;

procedure TRakBrushData.CreateNow;
begin
  FStyle := bsSolid;
  FColor := clBlack;
end;

procedure TRakBrushData.DataChanged(ChangeData: TObject);
begin
	ChangeData.Free;
end;

function TRakBrushData.Redo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataTBrushStyle then
  	Style := TRakChangeDataTBrushStyle(ChangeData).Neu
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Neu
  else
  	Result := false;
end;

procedure TRakBrushData.SetBrush(ABrush: TBrush);
begin
	ABrush.Color := Color;
  ABrush.Style := Style;
end;

procedure TRakBrushData.SetColor(const Value: TColor);
var
	OldValue: TColor;
begin
  if FColor <> Value then
  begin
  	OldValue := FColor;

	  FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

procedure TRakBrushData.SetStyle(const Value: TBrushStyle);
var
	OldValue: TBrushStyle;
begin
  if FStyle <> Value then
  begin
  	OldValue := FStyle;

	  FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakBrushData.StyleChangeAfter(OldValue: TBrushStyle);
begin
	DataChanged(TRakChangeDataTBrushStyle.Create(OldValue, Style));
end;

function TRakBrushData.Undo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataTBrushStyle then
  	Style := TRakChangeDataTBrushStyle(ChangeData).Alt
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Alt
  else
  	Result := false;
end;

{ TRakFontData }

function TRakFontData.AsXMLNow: string;
var
  xdi: TDomImplementation;
	doc: TdomDocument;
  Base: TdomElement;
  Elem: TdomElement;
  TextElem: TdomText;
begin
  xdi := TDomImplementation.Create(nil);
  try
    doc := xdi.createDocument('Data', nil);
    Base := doc.documentElement;
    Base.SetAttribute('Version', '1');
    Base.SetAttribute('Color', IntToStr(Color));
    Base.SetAttribute('MmHoehe', FloatToNormStr(MmHoehe));

    Elem := doc.CreateElement('Name');
    TextElem := doc.CreateTextNode(Name);
    Elem.AppendChild(TextElem);
    Base.AppendChild(Elem);

    if fsBold in Style then
	    Base.SetAttribute('Bold', '1');
    if fsItalic in Style then
	    Base.SetAttribute('Italic', '1');
    if fsUnderline in Style then
	    Base.SetAttribute('Underline', '1');
    if fsStrikeOut in Style then
	    Base.SetAttribute('StrikeOut', '1');

    Result := UTF16BEToUTF8Str(Base.Code, true);
  finally
  	xdi.Free;
  end;
end;

procedure TRakFontData.Assign(Source: TRakFontData);
begin
	FMmHoehe := Source.FMmHoehe;
	FName := Source.FName;
	FColor := Source.FColor;
	FStyle := Source.FStyle;
end;

procedure TRakFontData.AsXMLNow(NewValues: string);
var
  xdi: TDomImplementation;
  xdp: TXmlToDomParser;
	doc: TdomDocument;
  Base: TdomElement;
  sWert: string;
begin
  xdi := TDomImplementation.Create(nil);
  xdp := TXmlToDomParser.Create(nil);
  try
    xdp.domImpl := xdi;
    doc := xdp.stringToDom(NewValues);

    Base := doc.documentElement;
    if Base.NodeName = 'Data' then
    begin
 			Color := StrToIntDef(Base.GetAttribute('Color'), Color);
 			MmHoehe := NormStrToFloat(Base.GetAttribute('MmHoehe'), MmHoehe);
      sWert := Base.GetAttribute('Name');
      if sWert <> '' then
        Name := sWert;

      Style := [];
      if Base.GetAttribute('Bold') = '1' then
      	Style := Style + [fsBold];
      if Base.GetAttribute('Italic') = '1' then
      	Style := Style + [fsItalic];
      if Base.GetAttribute('Underline') = '1' then
      	Style := Style + [fsUnderline];
      if Base.GetAttribute('StrikeOut') = '1' then
      	Style := Style + [fsStrikeOut];
    end;
  finally
    xdp.Free;
    xdi.Free;
  end;
end;

procedure TRakFontData.ColorChangeAfter(OldValue: TColor);
begin
	DataChanged(TRakChangeDataFontColor.Create(OldValue, Color));
end;

procedure TRakFontData.CreateNow;
begin
  FMmHoehe := 3.5;
  FName := 'Arial';
  FColor := clBlack;
  FStyle := [];
end;

procedure TRakFontData.DataChanged(ChangeData: TRakChangeData);
begin
	ChangeData.Free;
end;

procedure TRakFontData.MmHoeheChangeAfter(OldValue: extended);
begin
	DataChanged(TRakChangeDataFontMmHoehe.Create(OldValue, MmHoehe));
end;

procedure TRakFontData.NameChangeAfter(OldValue: string);
begin
	DataChanged(TRakChangeDataFontName.Create(OldValue, Name));
end;

function TRakFontData.Redo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataString then
  	Name := TRakChangeDataString(ChangeData).Neu
	else if ChangeData is TRakChangeDataTFontStyles then
  	Style := TRakChangeDataTFontStyles(ChangeData).Neu
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Neu
	else if ChangeData is TRakChangeDataExtended then
  	MmHoehe := TRakChangeDataExtended(ChangeData).Neu
  else
  	Result := false;
end;

procedure TRakFontData.SetColor(const Value: TColor);
var
	OldValue: TColor;
begin
  if FColor <> Value then
  begin
  	OldValue := FColor;

	  FColor := Value;

    ColorChangeAfter(OldValue);
  end;
end;

procedure TRakFontData.SetFont(AFont: TFont);
begin
	AFont.Color := Color;
  AFont.Name := Name;
  AFont.Style := Style;
  AFont.Height := round(MmHoehe);
end;

procedure TRakFontData.SetMmHoehe(const Value: extended);
var
	OldValue: extended;
begin
  if FMmHoehe <> Value then
  begin
  	OldValue := FMmHoehe;

	  FMmHoehe := Value;

    MmHoeheChangeAfter(OldValue);
  end;
end;

procedure TRakFontData.SetName(const Value: string);
var
	OldValue: string;
begin
  if FName <> Value then
  begin
  	OldValue := FName;

	  FName := Value;

    NameChangeAfter(OldValue);
  end;
end;

procedure TRakFontData.SetStyle(const Value: TFontStyles);
var
	OldValue: TFontStyles;
begin
  if FStyle <> Value then
  begin
  	OldValue := FStyle;

	  FStyle := Value;

    StyleChangeAfter(OldValue);
  end;
end;

procedure TRakFontData.StyleChangeAfter(OldValue: TFontStyles);
begin
	DataChanged(TRakChangeDataTFontStyles.Create(OldValue, Style));
end;

function TRakFontData.TextWidth(Text: string): extended;
var
	Font: TRakWYSIWYGFont;
begin
	Font := TRakWYSIWYGFont.Create;
  try
  	Font.Font.Height := trunc(MmHoehe * 10);
    Font.Font.Name := Name;
	  Font.Font.Style := Style;
    Result := Font.TextWidth(Text) / 10;
  finally
    Font.Free;
  end;
end;

function TRakFontData.Undo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataString then
  	Name := TRakChangeDataString(ChangeData).Alt
	else if ChangeData is TRakChangeDataTFontStyles then
  	Style := TRakChangeDataTFontStyles(ChangeData).Alt
	else if ChangeData is TRakChangeDataTColor then
  	Color := TRakChangeDataTColor(ChangeData).Alt
	else if ChangeData is TRakChangeDataExtended then
  	MmHoehe := TRakChangeDataExtended(ChangeData).Alt
  else
  	Result := false;
end;

procedure TRakFontData.WrapText(Zeile: string; MmBreite: extended;
  Zeilen: TStrings);
const
	WHITESPACE: set of char = [#0..#32];
  ZEILENANFANG: set of char = ['<','(','[','{','#','$','&','*','+','/','@','\','`','^','|',#126,#127,#129..#191];
  ZEILENENDE: set of char = ['>',')',']','}','-','.',';',',',':','!','?','%','=',#128];
  GROSSBUCHSTABEN: set of char = ['A'..'Z','','',''];
  BINDUNG_LOSE: set of char = ['0'..'9','_','''','"',#192..#255];
var
  Rest: string;
  Idx: integer;
  TmpBreite: extended;
  MinBreite: extended;
  MinChar: integer;
  MaxChar: integer;
  TryChar: integer;
begin
	Zeilen.Clear;
  Rest := trim(Zeile);
  while Rest <> '' do //Von Rest wird jetzt solange was abgezwickt, was in den zur Verfgung stehenden Platz passt, bis Rest leer ist
  begin
    Zeile := Rest;
    //Zeile wird jetzt solange vergrert und verkleinert, bis die Zeile ideal in den zur Verfgung stehenden Platz passt
    TmpBreite := TextWidth(Zeile);
    if TmpBreite > MmBreite then
    begin // Wenn die Zeile noch zu breit ist
      MinChar := 1; //Mindestens ein Zeichen muss in die Zeile
      MaxChar := length(Zeile);//Es knnen maximal alle Zeichen in der Zeile sein
      TryChar := MaxChar;//TmpBreite enthlt die Breite von allen Zeichen
      MinBreite := 0;//MinChar hat mindestens eine Breite von 0
      while MinChar < MaxChar do
      begin //Solange nicht klar ist, wieviele Zeichen am besten in den Platz passen
      	if TmpBreite = MinBreite then //Wenn bei der Division durch (TmpBreite - MinBreite) ein Fehler auftreten wrde
        	TryChar := MinChar + 1
        else
        begin // Wenn das neue TryChar ideal berechnet werden kann
        	// Bei der folgenden Berechnung von TryChar wird davon ausgegangen, dass alle Zeichen die gleich Breite haben.
          // So wird mit Hilfe der zentrischen Streckung der wahrscheinlich beste Wert fr TryChar ermittelt
	      	TryChar := trunc(MinChar + (TryChar - MinChar) / (TmpBreite - MinBreite) * (MmBreite - MinBreite));
          if TryChar > MaxChar then //Wenn TryChar einen Wert angenommen hat, von dem wir schon wissen, dass er nicht in MmBreite reinpasst
          	TryChar := MaxChar
          else if TryChar <= MinChar then //Wenn TryChar einen Wert angenommen hat, von dem wir schon wissen, dass er in MmBreite reinpasst
          	TryChar := MinChar + 1;
        end;
        //Die Breite ermitteln
		    TmpBreite := TextWidth(Copy(Rest, 1, TryChar));
		    if TmpBreite > MmBreite then // Wenn es zu breit ist
        	MaxChar := TryChar - 1 //Dann wissen wir, dass maximal ein Zeichen weniger reinpasst
		    else if TmpBreite < MmBreite then
        begin // Wenn es reinpasst, aber nicht exakt
        	MinChar := TryChar; //Dann wissen wir wieviel Zeichen mindestens reinpassen
          MinBreite := TmpBreite; //und merken uns die ermittelte Breite fr die nchste Berechnung von TryChar
        end
        else
        begin // Wenn es exakt gepasst hat
        	// Dann knnen wir fr einen Abbruch der Schleife sorgen
          MinChar := TryChar;
          MaxChar := TryChar;
        end;
      end; // while MinChar < MaxChar do
      //In Zeile den String speichern, der den zur Verfgung stehenden Platz ideal ausntzt
      Zeile := Copy(Rest, 1, MinChar);
    end;
    // Zeile passt jetzt rein.

    if (length(Zeile) < length(Rest)) and not (Rest[length(Zeile)+1] in WHITESPACE) then
    begin // Wenn die Zeile nicht der ganze Rest ist und das erste Zeichen von Rest, das nicht in Zeile enthalten ist, kein Whitespace ist
      // Dann schauen wir, ob wir in Zeile eine bessere Stelle finden, an der wir einen Zeilenumbruch vornehmen knnen
      // Dazu suchen wir zuerst nach dem letzten Whitespace in Zeile
      Idx := PosRight(WHITESPACE, Zeile);
      if Idx > 0 then
      begin // Wenn in Zeile ein Whitespace vorhanden war
        Zeile := Left(Zeile, Idx);
      end
      else
      begin // Wenn in der ganzen Zeile kein einziges Whitespace vorhanden ist
        Idx := PosRight(ZEILENANFANG+ZEILENENDE, Zeile);
        if (Idx > 1) or ((Idx > 0) and (Zeile[Idx] in ZEILENENDE)) then
        begin // Wenn in Zeile ein Sonderzeichen vorhanden war
          if Zeile[Idx] in ZEILENANFANG then
            Zeile := Left(Zeile, Idx-1)
          else
            Zeile := Left(Zeile, Idx);
        end
        else
        begin // Wenn in der ganzen Zeile kein einziges Sonderzeichen vorhanden ist
          Idx := PosRight(GROSSBUCHSTABEN, Zeile);
          if Idx > 1 then
            Zeile := Left(Zeile, Idx-1)
          else
          begin
            Idx := PosRight(BINDUNG_LOSE, Zeile);
            if Idx > 0 then
              Zeile := Left(Zeile, Idx);
          end;
        end;
      end;
    end;

    // Die neue Zeile merken
    Zeilen.Add(trim(Zeile));
    // Den restlichen String ermitteln, wobei fhrende Whitespaces gleich entfernt werden
    Rest := trim(CutLeft(Rest, length(Zeile)));
  end;
end;

{ TRakMmBounds }

function TRakMmBounds.AsXMLNow: string;
var
  xdi: TDomImplementation;
	doc: TdomDocument;
  Base: TdomElement;
begin
  xdi := TDomImplementation.Create(nil);
  try
    doc := xdi.createDocument('Data', nil);
    Base := doc.documentElement;
    Base.SetAttribute('Version', '1');
    Base.SetAttribute('Links', FloatToNormStr(MmLinks));
    Base.SetAttribute('Oben', FloatToNormStr(MmOben));
    Base.SetAttribute('Breite', FloatToNormStr(MmBreite));
    Base.SetAttribute('Hoehe', FloatToNormStr(MmHoehe));

    Result := UTF16BEToUTF8Str(Base.Code, true);
  finally
  	xdi.Free;
  end;
end;

procedure TRakMmBounds.AsXMLNow(NewValues: string);
var
  xdi: TDomImplementation;
  xdp: TXmlToDomParser;
	doc: TdomDocument;
  Base: TdomElement;
  Links, Oben, Breite, Hoehe: extended;
begin
  xdi := TDomImplementation.Create(nil);
  xdp := TXmlToDomParser.Create(nil);
  try
    xdp.domImpl := xdi;
    doc := xdp.stringToDom(NewValues);

    Base := doc.documentElement;
    if Base.NodeName = 'Data' then
    begin
 			Links := NormStrToFloat(Base.GetAttribute('Links'), MmLinks);
 			Oben := NormStrToFloat(Base.GetAttribute('Oben'), MmOben);
 			Breite := NormStrToFloat(Base.GetAttribute('Breite'), MmBreite);
 			Hoehe := NormStrToFloat(Base.GetAttribute('Hoehe'), MmHoehe);

      SetMm(Links, Oben, Breite, Hoehe);
    end;
  finally
    xdp.Free;
    xdi.Free;
  end;
end;

procedure TRakMmBounds.DataChanged(ChangeData: TRakChangeData);
begin
	ChangeData.Free;
end;

function TRakMmBounds.GetMmHorzCenter: extended;
begin
	Result := MmLinks + MmBreite / 2;
end;

function TRakMmBounds.GetMmRechts: extended;
begin
	Result := MmLinks + MmBreite;
end;

function TRakMmBounds.GetMmRect: TExtRect;
begin
	Result := ExtBounds(MmLinks, MmOben, MmBreite, MmHoehe);
end;

function TRakMmBounds.GetMmUnten: extended;
begin
	Result := MmOben + MmHoehe;
end;

function TRakMmBounds.GetMmVertCenter: extended;
begin
	Result := MmOben + MmHoehe / 2;
end;

procedure TRakMmBounds.MmChangeAfter(const OldLinks, OldOben, OldBreite,
  OldHoehe: extended);
begin

end;

procedure TRakMmBounds.MmChangeBefore(var NewLinks, NewOben, NewBreite,
  NewHoehe: extended);
begin

end;

function TRakMmBounds.Redo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataMmLinks then
  	MmLinks := TRakChangeDataMmLinks(ChangeData).Neu
	else if ChangeData is TRakChangeDataMmOben then
  	MmOben := TRakChangeDataMmOben(ChangeData).Neu
	else if ChangeData is TRakChangeDataMmBreite then
  	MmBreite := TRakChangeDataMmBreite(ChangeData).Neu
	else if ChangeData is TRakChangeDataMmHoehe then
  	MmHoehe := TRakChangeDataMmHoehe(ChangeData).Neu
  else
  	Result := false;
end;

procedure TRakMmBounds.SetMm(Links, Oben, Breite, Hoehe: extended);
begin
	if Breite < 0 then
  	Breite := 0;

	if Hoehe < 0 then
  	Hoehe := 0;

  if (Links <> MmLinks) or (Oben <> MmOben) or (Breite <> MmBreite)
  	or (Hoehe <> MmHoehe) then
  begin
		SetMmNow(Links, Oben, Breite, Hoehe);
  end;
end;

procedure TRakMmBounds.SetMmBreite(const Value: extended);
begin
	SetMm(MmLinks, MmOben, Value, MmHoehe);
end;

procedure TRakMmBounds.SetMmHoehe(const Value: extended);
begin
	SetMm(MmLinks, MmOben, MmBreite, Value);
end;

procedure TRakMmBounds.SetMmHorzCenter(const Value: extended);
begin
	MmLinks := Value - MmBreite / 2;
end;

procedure TRakMmBounds.SetMmLinks(const Value: extended);
begin
	SetMm(Value, MmOben, MmBreite, MmHoehe);
end;

procedure TRakMmBounds.SetMmNow(Links, Oben, Breite, Hoehe: extended);
var
	lclLinks, lclOben, lclBreite, lclHoehe: extended;
begin
	MmChangeBefore(Links, Oben, Breite, Hoehe);

  lclLinks := FMmLinks;
  lclOben := FMmOben;
  lclBreite := FMmBreite;
  lclHoehe := FMMHoehe;

	FMmLinks := Links;
  FMmOben := Oben;
  FMmBreite := Breite;
  FMMHoehe := Hoehe;

	MmChangeAfter(lclLinks, lclOben, lclBreite, lclHoehe);
end;

procedure TRakMmBounds.SetMmOben(const Value: extended);
begin
	SetMm(MmLinks, Value, MmBreite, MmHoehe);
end;

procedure TRakMmBounds.SetMmRechts(const Value: extended);
begin
	MmLinks := Value - MmBreite;
end;

procedure TRakMmBounds.SetMmUnten(const Value: extended);
begin
	MmOben := Value - MmHoehe;
end;

procedure TRakMmBounds.SetMmVertCenter(const Value: extended);
begin
	MmOben := Value - MmHoehe / 2;
end;

function TRakMmBounds.Undo(ChangeData: TObject): boolean;
begin
	Result := true;

	if ChangeData is TRakChangeDataMmLinks then
  	MmLinks := TRakChangeDataMmLinks(ChangeData).Alt
	else if ChangeData is TRakChangeDataMmOben then
  	MmOben := TRakChangeDataMmOben(ChangeData).Alt
	else if ChangeData is TRakChangeDataMmBreite then
  	MmBreite := TRakChangeDataMmBreite(ChangeData).Alt
	else if ChangeData is TRakChangeDataMmHoehe then
  	MmHoehe := TRakChangeDataMmHoehe(ChangeData).Alt
  else
  	Result := false;
end;

{ TRakChangeDataTPenMode }

constructor TRakChangeDataTPenMode.Create(Alt, Neu: TPenMode);
begin
	self.Alt := Alt;
	self.Neu := Neu;

  FMenueText := 'Stiftmodus ndern';
end;

{ TRakChangeDataTPenStyle }

constructor TRakChangeDataTPenStyle.Create(Alt, Neu: TPenStyle);
begin
	self.Alt := Alt;
	self.Neu := Neu;

  FMenueText := 'Stiftstil ndern';
end;

{ TRakChangeDataTBrushStyle }

constructor TRakChangeDataTBrushStyle.Create(Alt, Neu: TBrushStyle);
begin
	self.Alt := Alt;
	self.Neu := Neu;

  FMenueText := 'Pinselstil ndern';
end;

{ TRakChangeDataTFontStyles }

constructor TRakChangeDataTFontStyles.Create(Alt, Neu: TFontStyles);
begin
	self.Alt := Alt;
	self.Neu := Neu;

  FMenueText := 'Schriftstil ndern';
end;

{ TRakChangeDataPenColor }

constructor TRakChangeDataPenColor.Create(Alt, Neu: TColor);
begin
	inherited;

  FMenueText := 'Stiftfarbe ndern';
end;

{ TRakChangeDataPenMmBreite }

constructor TRakChangeDataPenMmBreite.Create(Alt, Neu: extended);
begin
	inherited;

  FMenueText := 'Stiftbreite ndern';
end;

{ TRakChangeDataBrushColor }

constructor TRakChangeDataBrushColor.Create(Alt, Neu: TColor);
begin
	inherited;

  FMenueText := 'Pinselfarbe ndern';
end;

{ TRakChangeDataFontColor }

constructor TRakChangeDataFontColor.Create(Alt, Neu: TColor);
begin
	inherited;

  FMenueText := 'Schriftfarbe ndern';
end;

{ TRakChangeDataFontMmHoehe }

constructor TRakChangeDataFontMmHoehe.Create(Alt, Neu: extended);
begin
	inherited;

  FMenueText := 'Schrifthhe ndern';
end;

{ TRakChangeDataFontName }

constructor TRakChangeDataFontName.Create(Alt, Neu: string);
begin
	inherited;

  FMenueText := 'Schrift ndern';
end;

end.
