unit WYSIWYGFont;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit WYSIWYGFont stellt Klassen zur Verfgung,
									mit denen Text auf dem Bildschirm so dargestellt
                  wird, wie er auf einen Drucker ausgegeben wird.
                  Diese Klassen sind also hauptschlich fr eine Druckvorschau
                  interessant.
                  Die Klasse TRakWYSIWYGFont ist die "Hauptklasse". Die anderen
                  Klassen sind nur Hilfsklassen
// erstellt am  : ???
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    24.9.01: Bugfix in TRakWYSIWYGFont
    10.3.02: Bugfix: TRakWYSIWYGFont.DrawText hat DestCanvas durcheinander
                gebracht, da Style und Color von Brush inerhalb von
                SaveDC/RestoreDC gendert wird.
    25.2.04: Bugfix: TRakWYSIWYGFont.DrawText hat GDI-Resourcen verschwinden lassen
    				 nderung: TRakWYSIWYGFont.DrawText werden die Buchstaben etwas anders
             					gesetzt, damit das Gesamtbild verbessert wird.
    27.2.04: Bugfix: TRakWYSIWYGFont.DrawText hat die Einstellungen des Brushes ignoriert
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses Windows, Classes, Controls, Graphics, MiscFunc, SysUtils;

type
{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TDrawData
// Beschreibung : TDrawData wird als Parameter bei der Methode
                  TRakWYSIWYGFont.DrawText verwendet.
// erstellt am  :
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TDrawData = class
  public
    Text: string; // Der zu zeichnede Text
    Left: integer; // Die linke Position des Textes
    Top: integer; // Die obere Position des Textes
    Winkel: integer; // Der Winkel in 10-tel Grad, mit dem der Text um Left/Top gedreht wird
    DoClip: boolean; // Legt fest, ob AClipRect verwendet wird
    AClipRect: TRect; // Legt das Rechteck fest innerhalb dem etwas gezeichnet wird. Schriftteile ausserhalb davon werden abgeschnitten
    ClipWinkel: integer; // Der Winkel in 10-tel Grad, mit dem AClipRect um AClipRect.Left/Top gedreht wird
    Aufloesung: extended; // Legt fest wie gro der Text tatschlich ausgegeben wird
                          // Beispiel: Font.Size = 10; Aufloesung = 0.5; Die Schrift wird mit einer Hhe von 20 Pixel ausgegeben
                          // Beispiel: Font.Size = 10; Aufloesung = 2; Die Schrift wird mit einer Hhe von 5 Pixel ausgegeben
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TExtendedFont
// Beschreibung : TExtendedFont speichert die Daten, welcher Font verwendet
                  werden soll.
                  Die Properties lehnen sich stark an die entsprechenden
                  Mglichkeiten der WinAPI an. Da zum Speichern der meisten Daten
                  tatschlich ein WinAPI-Font verwendet wird ist TExtendedFont
                  ein Resourcenfresser. Man sollte als nur wenige oder gar keine
                  Instanzen von TExtendedFont auf Dauer im Speicher halten
// erstellt am  :
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  7.4.2002 Property Handle entfernt
            Methode FreeHandle entfernt
            Destruktor entfernt
            Neue Methode CreateHandle
////////////////////////////////////////////////////////////////////////////////}
  TExtendedFont = class(TPersistent)
  private
    FColor: longint;
    FLogFont: TLogFont;
  protected
    function GetAngle: integer; virtual;
    procedure SetAngle(Value: integer); virtual;
    procedure SetColor(Value: longint); virtual;
    function GetHeight: Integer; virtual;
    procedure SetHeight(Value: Integer); virtual;
    function GetName: TFontName; virtual;
    procedure SetName(Value: TFontName); virtual;
    function GetPitch: TFontPitch; virtual;
    procedure SetPitch(Value: TFontPitch); virtual;
    function GetSize: Integer; virtual;
    procedure SetSize(Value: Integer); virtual;
    function GetStyle: TFontStyles; virtual;
    procedure SetStyle(Value: TFontStyles); virtual;
    function GetLogFont: TLogFont; virtual;
    procedure SetLogFont(Value: TLogFont); virtual;
    function GetWidth: Integer; virtual;
    procedure SetWidth(Value: Integer); virtual;
  public
    constructor Create;

    procedure Assign(Source: TPersistent); override;
    function CreateHandle: HFont;
    procedure CopyFromDevice(DC: HDC); virtual;

    property Angle: integer read GetAngle write SetAngle;
    property Color: longint read FColor write SetColor;
    property Height: Integer read GetHeight write SetHeight;
    property Name: TFontName read GetName write SetName;
    property Pitch: TFontPitch read GetPitch write SetPitch default fpDefault;
    property Size: Integer read GetSize write SetSize stored False;
    property Style: TFontStyles read GetStyle write SetStyle;
    property Width: Integer read GetWidth write SetWidth;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakWYSIWYGFont
// Beschreibung : TRakWYSIWYGFont ist die Klasse, die versucht einen Text auf
                  dem Bildschirm mglichst so auszugeben, wie er spter
                  ausgedruckt werden wrde. Da Windows Text nicht stufenlos
                  skallieren kann klappt das nicht hundertprozentig. Da aber
                  jeder Buchstabe einzeln gezeichnet wird, ist der Fehler,
                  der in der Breite des Textes entsteht vernachlssigbar.
                  Durch die einzelne Ausgabe jedes Buchstaben ist die
                  Gesamtausgabe eines Textes aber leider auch recht Zeitintensiv.
                  Wenn ein WYSIWYG nicht wichtig ist, dann sollte man also
                  lieber die Standardfunktionen von Delphi fr die Ausgabe von
                  Text verwenden.
                  Da TRakWYSIWYGFont auch ein Resourcenfresser ist, sollte man
                  nur zur Ausgabe in der Paint-Methode eine Instanz davon bilden
                  und diese gleich wieder zerstren.
// erstellt am  :
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    24.9.01:   Bugfix in Mehode DrawText: Bei berlappenden Buchstaben, wurden
              Teile des zuerst gezeichneten Buchstaben wieder gelscht
////////////////////////////////////////////////////////////////////////////////}
  TRakWYSIWYGFont = class
  private
    FBitmap : TBitmap;
    FClibingActive: boolean;
    FFont: TExtendedFont;
  protected
    function GetDrawDC: HDC; virtual;

  public
    constructor Create;
    destructor Destroy; override;

    // DrawText zeichnet eine Textzeile;
    // Das Property Font bestimmt die zu verwendente Schrift;
    // Mit den Fit-Funktionen kann man vorher die Gre der Schrift berechnen lassen
    procedure DrawText(DestCanvas: TCanvas; Data: TDrawData); virtual;

    // FitFontSize stellt die Font.Size so ein, dass die "Zeile" mglichst viel Platz von MaxWidth/MaxHeight nutzt
    procedure FitFontSize(MaxWidth, MaxHeight: integer; Zeile: string);
    // FitFontSizeByHeight stellt die Font.Size so ein, dass mglichst viel Platz von MaxHeight genutzt wird
    procedure FitFontSizeByHeight(MaxHeight: integer);
    // FitFontSizeByWidth stellt die Font.Size so ein, dass die "Zeile" mglichst viel Platz von MaxWidth nutzt
    procedure FitFontSizeByWidth(MaxWidth: integer; Zeile: string);

    // TextHeight gibt zurck, wieviele Pixel der Text hoch ist
    function TextHeight(Text: string): integer; virtual;
    // TextWidth gibt zurck, wieviele Pixel der Text breit ist
    function TextWidth(Text: string): integer; virtual;

    // ClibingActive wird zur Zeit nicht verwendet, s.a. TDrawData.DoClip
    property ClibingActive: boolean read FClibingActive
      write FClibingActive default true;
    // Das Handle der Zeichenflche, die fr die Fit- und Text-Funktionen verwendet wird
    property DrawDC: HDC read GetDrawDC;
    // Font legt die Schrift fest, mit der gezeichnet werden soll
    property Font: TExtendedFont read FFont;
  end;


implementation

const
  BMSIZE = 150;

constructor TExtendedFont.Create;
begin
  inherited Create;

  with FLogFont do
  begin
    lfHeight := -120;
    lfWidth := 0; { have font mapper choose }
    lfEscapement := 0; { only straight fonts }
    lfOrientation := 0; { no rotation }
    lfWeight := FW_NORMAL;
    lfItalic := 0;
    lfUnderline := 0;
    lfStrikeOut := 0;
    lfCharSet := DEFAULT_CHARSET;
    StrPCopy(lfFaceName, 'Arial');
    lfQuality := DEFAULT_QUALITY;
    { Everything else as default }
    lfOutPrecision := OUT_DEFAULT_PRECIS;
    lfClipPrecision := CLIP_DEFAULT_PRECIS;
    lfPitchAndFamily := DEFAULT_PITCH;
  end;

  Color := clBlack;
end;

procedure TExtendedFont.Assign(Source: TPersistent);
var
  Src: TExtendedFont;
  FSrc: TFont;
begin
//  inherited Assign(Source);

  if Source is TExtendedFont then
  begin
    Src := TExtendedFont(Source);
    Size := Src.Size;
    Name := Src.Name;
    Pitch := Src.Pitch;
    Style := Src.Style;
    Color := Src.Color;
    FColor := Src.FColor;
    FLogFont := Src.FLogFont;
  end
  else if Source is TFont then
  begin
    FSrc := TFont(Source);

    Size := FSrc.Size;
    Name := FSrc.Name;
    Pitch := FSrc.Pitch;
    Style := FSrc.Style;
    Color := FSrc.Color;
  end
end;

procedure TExtendedFont.CopyFromDevice(DC: HDC);
var
  Metrics: TTextMetric;
  Handle: HFont;
begin
  GetTextMetrics(DC, Metrics);
  with Metrics, FLogFont do
  begin
    lfHeight := tmHeight;
    lfWidth := 0;
    lfEscapement := 0;
    lfOrientation := 0;
    lfWeight := tmWeight;
    lfItalic := tmItalic;
    lfUnderline := tmUnderlined;
    lfStrikeOut := tmStruckOut;
    lfCharSet := tmCharSet;
    lfOutPrecision := OUT_TT_ONLY_PRECIS;
    lfClipPrecision := CLIP_LH_ANGLES or CLIP_MASK;
    lfQuality := PROOF_QUALITY;
    lfPitchAndFamily := FF_MODERN;
    Handle := CreateHandle;
    try
      GetTextFace(Handle, sizeof(lfFaceName), lfFaceName);
    finally
      DeleteObject(Handle);
    end;
  end;
end;

function TExtendedFont.GetAngle: integer;
begin
  Result := -FLogFont.lfEscapement;
end;

procedure TExtendedFont.SetAngle(Value: integer);
begin
  FLogFont.lfEscapement := -Value;
  FLogFont.lfOrientation := -Value;
end;

procedure TExtendedFont.SetColor(Value: longint);
begin
  FColor := Value;
end;

function TExtendedFont.CreateHandle: HFont;
begin
  Result := CreateFontIndirect(FLogFont);
end;

function TExtendedFont.GetHeight: Integer;
begin
  Result := FLogFont.lfHeight;
end;

procedure TExtendedFont.SetHeight(Value: Integer);
begin
  FLogFont.lfHeight := Value;
end;

function TExtendedFont.GetWidth: Integer;
begin
  Result := FLogFont.lfWidth;
end;

procedure TExtendedFont.SetWidth(Value: Integer);
begin
  FLogFont.lfWidth := Value;
end;

function TExtendedFont.GetName: TFontName;
begin
  Result := FLogFont.lfFaceName;
end;

procedure TExtendedFont.SetName(Value: TFontName);
begin
  StrPCopy(FLogFont.lfFaceName, Value);
end;

function TExtendedFont.GetPitch: TFontPitch;
begin
  if FLogFont.lfPitchAndFamily = VARIABLE_PITCH then
    Result := fpVariable
  else if FLogFont.lfPitchAndFamily = FIXED_PITCH then
    Result := fpFixed
  else
    Result := fpDefault;
end;

procedure TExtendedFont.SetPitch(Value: TFontPitch);
begin
  case Pitch of
    fpVariable: FLogFont.lfPitchAndFamily := VARIABLE_PITCH;
    fpFixed: FLogFont.lfPitchAndFamily := FIXED_PITCH;
  else
    FLogFont.lfPitchAndFamily := DEFAULT_PITCH;
  end;
end;

function TExtendedFont.GetSize: Integer;
begin
  Result := -MulDiv(Height, 72, 120);
end;

procedure TExtendedFont.SetSize(Value: Integer);
begin
  Height := -MulDiv(Value, 120, 72);
end;

function TExtendedFont.GetStyle: TFontStyles;
begin
  Result := [];
  if FLogFont.lfWeight = FW_BOLD then
    Result := Result + [fsBold];

  if boolean(FLogFont.lfItalic) then
    Result := Result + [fsItalic];
  if boolean(FLogFont.lfUnderline) then
    Result := Result + [fsUnderline];
  if boolean(FLogFont.lfStrikeOut) then
    Result := Result + [fsStrikeOut];
end;

procedure TExtendedFont.SetStyle(Value: TFontStyles);
begin
  if fsBold in Value then
    FLogFont.lfWeight := FW_BOLD
  else
    FLogFont.lfWeight := FW_NORMAL;
  FLogFont.lfItalic := Byte(fsItalic in Value);
  FLogFont.lfUnderline := Byte(fsUnderline in Value);
  FLogFont.lfStrikeOut := Byte(fsStrikeOut in Value);
end;

function TExtendedFont.GetLogFont: TLogFont;
begin
  Result := FLogFont;
end;

procedure TExtendedFont.SetLogFont(Value: TLogFont);
begin
  FLogFont := Value;
end;


constructor TRakWYSIWYGFont.Create;
begin
  inherited Create;

  FBitmap := TBitmap.Create;
  FBitmap.Height := BMSIZE;
  FBitmap.Width := BMSIZE;
  FClibingActive := true;

  FFont := TExtendedFont.Create;
end;

destructor TRakWYSIWYGFont.Destroy;
begin
  FBitmap.Free;
  FFont.Free;

  inherited Destroy;
end;

function TRakWYSIWYGFont.GetDrawDC: HDC;
begin
  Result := FBitmap.Canvas.Handle;
end;

function TRakWYSIWYGFont.TextHeight(Text: string): integer;
begin
  FBitmap.Canvas.Font.Handle := FFont.CreateHandle;
  Result := FBitmap.Canvas.TextHeight(Text);
end;

function TRakWYSIWYGFont.TextWidth(Text: string): integer;
begin
  FBitmap.Canvas.Font.Handle := FFont.CreateHandle;
  Result := FBitmap.Canvas.TextWidth(Text);
end;

procedure TRakWYSIWYGFont.DrawText(DestCanvas: TCanvas; Data: TDrawData);
var
  VersatzFaktor: extended;

  function GetCharPoint(i, YOffset: integer): TPoint;
  var
    X: integer;
  begin
    with Data do
    begin
      if i < 2 then
        X := 0
      else
        X := trunc(DestCanvas.TextWidth(Copy(Text, 1, i-1)) * VersatzFaktor);

      Drehe(X, YOffset, Winkel);
      Result.X := Left + X;
      Result.Y := Top + YOffset;
    end;
  end;

  procedure SetClipRects;
  const
    Offset = 0;
  var
    X, Y: integer;
    Corners: array[1..4] of TPoint;
    ARgn: HRGN;
    BLeft, BTop: integer;
    BRight, BBottom: integer;
    DP: array[1..2] of TPoint;
    WindowOrg: TPoint;
  begin
    GetWindowOrgEx(DestCanvas.Handle, WindowOrg);

    DP[1] := Point(Data.AClipRect.Left, Data.AClipRect.Top);
    DP[2] := Point(Data.AClipRect.Right, Data.AClipRect.Bottom);
    LPToDP(DestCanvas.Handle, DP, 2);

    if Data.ClipWinkel = 0 then
    begin
      Corners[1] := Point(DP[1].X - Offset, DP[1].Y - Offset);
      Corners[2] := Point(DP[2].X + Offset, DP[1].Y - Offset);
      Corners[3] := Point(DP[2].X + Offset, DP[2].Y + Offset);
      Corners[4] := Point(DP[1].X - Offset, DP[2].Y + Offset);
    end
    else
    begin
      BLeft := DP[1].X;
      BTop := DP[1].Y;
      BRight := DP[2].X;
      BBottom := DP[2].Y;
      X := - Offset;
      Y := - Offset;
      Drehe(X, Y, Data.ClipWinkel);
      Corners[1] := Point(BLeft + X, BTop + Y);

      X := BRight - BLeft + Offset;
      Y := -Offset;
      Drehe(X, Y, Data.ClipWinkel);
      Corners[2] := Point(BLeft + X, BTop + Y);

      X := BRight - BLeft + Offset;
      Y := BBottom - BTop + Offset;
      Drehe(X, Y, Data.ClipWinkel);
      Corners[3] := Point(BLeft + X, BTop + Y);

      X := -Offset;
      Y := BBottom - BTop + Offset;
      Drehe(X, Y, Data.ClipWinkel);
      Corners[4] := Point(BLeft + X, BTop + Y);
    end;

    ARgn := CreatePolygonRgn(Corners, 4, WINDING);
    try
      ExtSelectClipRgn(DestCanvas.Handle, ARgn, RGN_AND);
    finally
      DeleteObject(ARgn);
    end;
  end;

  procedure PaintBackground;
  const
    Offset = 0;
  var
    X, Y: integer;
    Corners: array[1..4] of TPoint;
    ARgn: HRGN;
    BLeft, BTop: integer;
    BRight, BBottom: integer;
    DP: array[1..2] of TPoint;
    WindowOrg: TPoint;
  begin
    GetWindowOrgEx(DestCanvas.Handle, WindowOrg);

    DP[1] := Point(Data.Left, Data.Top);
    DP[2] := Point(Data.Left + trunc(TextWidth(Data.Text) / Data.Aufloesung),
      Data.Top + trunc(TextHeight(Data.Text) / Data.Aufloesung));
    LPToDP(DestCanvas.Handle, DP, 2);

    if Data.Winkel = 0 then
    begin
      Corners[1] := Point(DP[1].X - Offset, DP[1].Y - Offset);
      Corners[2] := Point(DP[2].X + Offset, DP[1].Y - Offset);
      Corners[3] := Point(DP[2].X + Offset, DP[2].Y + Offset);
      Corners[4] := Point(DP[1].X - Offset, DP[2].Y + Offset);
    end
    else
    begin
      BLeft := DP[1].X;
      BTop := DP[1].Y;
      BRight := DP[2].X;
      BBottom := DP[2].Y;
      X := - Offset;
      Y := - Offset;
      Drehe(X, Y, Data.Winkel);
      Corners[1] := Point(BLeft + X, BTop + Y);

      X := BRight - BLeft + Offset;
      Y := -Offset;
      Drehe(X, Y, Data.Winkel);
      Corners[2] := Point(BLeft + X, BTop + Y);

      X := BRight - BLeft + Offset;
      Y := BBottom - BTop + Offset;
      Drehe(X, Y, Data.Winkel);
      Corners[3] := Point(BLeft + X, BTop + Y);

      X := -Offset;
      Y := BBottom - BTop + Offset;
      Drehe(X, Y, Data.Winkel);
      Corners[4] := Point(BLeft + X, BTop + Y);
    end;

    DestCanvas.Pen.Style := psClear;
    ARgn := CreatePolygonRgn(Corners, 4, WINDING);
    try
      DestCanvas.Polygon(Corners);
    finally
      DeleteObject(ARgn);
    end;
    DestCanvas.Pen.Handle := 0;
  end;
var
  OldDc: integer;
  i: integer;
  APoint: TPoint;
  lFont : TExtendedFont;
  Hoehe, Breite: integer;
  rHoehe, rBreite: integer;
  YOffset: integer;
  BaseY: integer;
  metrics: TextMetric;
  Zeile: string;
  Pen: TPen;
  Brush: TBrush;
begin //TRakWYSIWYGFont.DrawText
  Pen := TPen.Create;
  Brush := TBrush.Create;
  try
    //TCanvas hat Probleme, wenn man mit SaveDc/RestoreDc arbeitet
    //und dazwischen den Pen oder Brush ndert.
    // Deshalb merken  wir uns hier den aktuellen Zustand und stellen ihn am Schlu wieder her.
    Pen.Assign(DestCanvas.Pen);
    Brush.Assign(DestCanvas.Brush);
    // Auerdem gibt TCanvas in Zusammenahng mit SaveDC/RestoreDC Resourcen nicht sauber frei
    // Deswegen setzen wir vor dem Aufruf von SaveDC und auch vor RestoreDC die Handles auf 0
    DestCanvas.Font.Handle := 0;
    DestCanvas.Brush.Handle := 0;
    DestCanvas.Pen.Handle := 0;
    OldDc := SaveDC(DestCanvas.Handle);
    try
	    DestCanvas.Pen.Assign(Pen);
  	  DestCanvas.Brush.Assign(Brush);

      if Data.Aufloesung = 0 then
        Data.Aufloesung := 1;

      if Data.DoClip then
        SetClipRects;

      if DestCanvas.Brush.Style <> bsClear then
      begin
        PaintBackground;
        DestCanvas.Brush.Style := bsClear;
      end;

      lFont := TExtendedFont.Create;
      try
        lFont.Assign(FFont);
        // Wenn wir den Font mit seiner momentanen Hhe verwenden, knnte es
        // passieren, da mit der Umrechnerrei von Mapmode ein Font verwendet
        // wird, der fr uns zu gro ist. Dadurch wrde dann vielleicht
        // unten etwas abgeschnitten und Buchstaben knnten sich berlappen.
        // Deshalb wird jetzt der Font solange
        // verkleinert, bis der umgerechnete Font kleinergleich dem
        // internen Font ist
        Zeile := Data.Text;
        if Zeile = '' then
          Zeile := ' ';
        Hoehe := TextHeight(Zeile);
        Breite := TextWidth(Zeile);

        lFont.Size := trunc(lFont.Size / Data.Aufloesung);
        // Den physikalischen Font in DestCanvas verwenden
        DestCanvas.Font.Handle := lFont.CreateHandle;

        rBreite := DestCanvas.TextWidth(Zeile);
        rHoehe := DestCanvas.TextHeight(Zeile);
        while (abs(lFont.Size) > 1)
              and
              (  (rHoehe * Data.Aufloesung > Hoehe)
                or
                (rBreite * Data.Aufloesung > Breite)
              ) do
        begin
          if lFont.Size > 0 then
            lFont.Size := lFont.Size - 1
          else
            lFont.Size := lFont.Size + 1;
          DestCanvas.Font.Handle := lFont.CreateHandle;
          rBreite := DestCanvas.TextWidth(Zeile);
          rHoehe := DestCanvas.TextHeight(Zeile);
        end;

        // Wir setzen hier den Winkel auf Null, da sonst bei einigen
        // Schriften unter Windows NT falsche Werte zurckgfeliefert werden.
        // Windows 95 hat dieses Problem nicht; Unter NT tritt das
        // Problem auch bei unterschiedlichen Grafikkartentreibern auf
        // Das Problem besteht auch nicht bei allen Schriftarten.
        lFont.Angle := 0;
        DestCanvas.Font.Handle := lFont.CreateHandle;

        GetTextMetrics(DestCanvas.Handle, metrics);
        BaseY := metrics.tmAscent;

        // Jetzt setzen wir den Winkel wieder ein, da wir alle Werte, die
        // durch den Winkel verflscht werden (metrics.tmAscent), ermittelt haben
        lFont.Angle := FFont.Angle;
        DestCanvas.Font.Handle := lFont.CreateHandle;

        YOffset := BaseY + trunc((Hoehe - DestCanvas.TextHeight(Zeile) * Data.Aufloesung) / Data.Aufloesung);

        // Die Schriftfarbe muss separat gesetzt werden (ein hoch auf das objektorientierte Windows)
        SetTextColor(DestCanvas.Handle, lFont.Color);

        // Wir mssen mit der Schriftgrundlinie arbeiten, da es unter
        // Windows NT bei einigen Schriften Probleme gibt, wenn diese
        // gedreht ausgegeben werden und die Drehung nicht auf Basis
        // der Grundlinie erfolgt
        SetTextAlign(DestCanvas.Handle, TA_LEFT + TA_BASELINE + TA_NOUPDATECP);

        VersatzFaktor := Breite / Data.Aufloesung / rBreite;
        for i := 1 to length(Data.Text) do
        begin
          APoint := GetCharPoint(i, YOffset);

          DestCanvas.TextOut(APoint.X, APoint.Y, Data.Text[i]);
        end;
      finally
        lFont.Free;
      end;
      // Auerdem gibt TCanvas in Zusammenahng mit SaveDC/RestoreDC Resourcen nicht sauber frei
      // Deswegen setzen wir vor dem Aufruf von SaveDC und auch vor RestoreDC die Handles auf 0
      DestCanvas.Font.Handle := 0;
      DestCanvas.Brush.Handle := 0;
      DestCanvas.Pen.Handle := 0;
    finally
      RestoreDC(DestCanvas.Handle, OldDc);
    end;
    // Hier wird Pen und Brush wieder in den Ausgangszustand gebracht,
    // damit das SaveDC/RestoreDC keine Nebeneffekte in TCanvas erzeugt
    DestCanvas.Pen.Assign(Pen);
    DestCanvas.Brush.Assign(Brush);
  finally
    Pen.Free;
    Brush.Free;
  end;
end; //TRakWYSIWYGFont.DrawText


procedure TRakWYSIWYGFont.FitFontSize(MaxWidth, MaxHeight: integer; Zeile: string);
var
  Min, Max: integer;
  TmpWidth, TmpHeight, NextSize: integer;
begin
  if Zeile = '' then
    Zeile := 'H';
  // Min enthlt die Gre, welche die Schrift mindestens haben mu
  Min := 1;
  // Max enthlt die Gre, bei der die Schrift noch zu gro ist
  Max := -1;
  // Min und Max werden solange aneinander angeglichen, bis in Min
  // die gesuchte Schriftgre steht. Am Anfang ist Max noch -1, da
  // noch nicht bekannt ist bei welcher Gre die Schrift zu Hoch wre.
  // Solange noch keine Schriftgre probiert wurde, die zu gro ist,
  // bleibt Max auf -1 und die nchste Gre, die probiert wird, ist
  // doppelt so gro wie die alte.
  // Sobald Max einen Wert grer als -1 (und auch grer als Min) hat,
  // wird der Mittelwert von Min und Max fr die nchste Gre genommen,
  // die probiert wird.
  // Entsprechend, ob die gerade probierte Gre in den zur Verfgung
  // stehenen Platz pat oder nicht, wird Min oder Max der Wert der
  // gerade probierte Gre gegeben

  // Die erste zu probierende Gre festlegen
  if MaxHeight > 0 then
    Font.Size := MaxHeight
  else
    Font.Size := 1;

  while (Min < Max-1) or (Max = -1) do
  begin // Solange nicht die maximal mgliche Schriftgre gefunden wurde
    // Die Schriftgre ermitteln
    TmpHeight := TextHeight(Zeile);
    TmpWidth := TextWidth(Zeile);

    if (TmpHeight < MaxHeight) and (TmpWidth < MaxWidth) then
    begin // Wenn die Schrift nicht den zur Verfgung
          // stehenden Platz ausfllt
      //Merken, da der Font mindestens so gro sein mu, wie er jetzt ist
      Min := Font.Size;
      // Die nchste zu probierende Schriftgre festlegen
      if Max > -1 then
        NextSize := (Max + Min) div 2
      else
        NextSize := Min * 2;
    end
    else if (TmpHeight > MaxHeight) or (TmpWidth > MaxWidth) then
    begin // Wenn die Schrift den zur Verfgung
          // stehenden Platz berschreitet
      //Merken, da der Font nicht so gro sein darf, wie er jetzt ist
      Max := Font.Size;
      // Die nchste zu probierende Schriftgre festlegen
      NextSize := (Max + Min) div 2;
    end
    else
    begin // Wenn die Schrift den zur Verfgung
          // stehenden Platz genau ausfllt
      Min := Font.Size;
      Max := Min;
      NextSize := 1; // Diese Zeile unterdrckt nur eine Warnung
    end;

    if (Min < Max-1) or (Max = -1) then
    begin // Wenn die ideale Schriftgre noch nicht gefunden wurde
      // Dann wird mit der nchsten weitergemacht
      Font.Size := NextSize;
    end;
  end;

  Font.Size := Min;
end;

procedure TRakWYSIWYGFont.FitFontSizeByHeight(MaxHeight: integer);
var
  Min, Max: integer;
  TmpHeight, NextSize: integer;
begin
  // Min enthlt die Gre, welche die Schrift mindestens haben mu
  Min := 1;
  // Max enthlt die Gre, bei der die Schrift noch zu gro ist
  Max := -1;
  // Min und Max werden solange aneinander angeglichen, bis in Min
  // die gesuchte Schriftgre steht. Am Anfang ist Max noch -1, da
  // noch nicht bekannt ist bei welcher Gre die Schrift zu Hoch wre.
  // Solange noch keine Schriftgre probiert wurde, die zu gro ist,
  // bleibt Max auf -1 und die nchste Gre, die probiert wird, ist
  // doppelt so gro wie die alte.
  // Sobald Max einen Wert grer als -1 (und auch grer als Min) hat,
  // wird der Mittelwert von Min und Max fr die nchste Gre genommen,
  // die probiert wird.
  // Entsprechend, ob die gerade probierte Gre in den zur Verfgung
  // stehenen Platz pat oder nicht, wird Min oder Max der Wert der
  // gerade probierte Gre gegeben

  // Die erste zu probierende Gre festlegen
  if MaxHeight > 0 then
    Font.Size := MaxHeight
  else
    Font.Size := 1;

  while (Min < Max-1) or (Max = -1) do
  begin // Solange nicht die maximal mgliche Schriftgre gefunden wurde
    // Die Schriftgre ermitteln
    TmpHeight := TextHeight('JGYPjgyp');

    if TmpHeight < MaxHeight then
    begin // Wenn die Schrift nicht den in der Hhe zur Verfgung
          // stehenden Platz ausfllt
      //Merken, da der Font mindestens so gro sein mu, wie er jetzt ist
      Min := Font.Size;
      // Die nchste zu probierende Schriftgre festlegen
      if Max > -1 then
        NextSize := (Max + Min) div 2
      else
        NextSize := Min * 2;
    end
    else if TmpHeight > MaxHeight then
    begin // Wenn die Schrift den in der Hhe zur Verfgung
          // stehenden Platz berschreitet
      //Merken, da der Font nicht so gro sein darf, wie er jetzt ist
      Max := Font.Size;
      // Die nchste zu probierende Schriftgre festlegen
      NextSize := (Max + Min) div 2;
    end
    else
    begin // Wenn die Schrift den in der Hhe zur Verfgung
          // stehenden Platz genau ausfllt
      Min := Font.Size;
      Max := Min;
      NextSize := 1; // Diese Zeile unterdrckt nur eine Warnung
    end;

    if (Min < Max-1) or (Max = -1) then
    begin // Wenn die ideale Schriftgre noch nicht gefunden wurde
      // Dann wird mit der nchsten weitergemacht
      Font.Size := NextSize;
    end;
  end;

  Font.Size := Min;
end;


procedure TRakWYSIWYGFont.FitFontSizeByWidth(MaxWidth: integer; Zeile: string);
var
  Min, Max: integer;
  TmpWidth, NextSize: integer;
begin
  if Zeile <> '' then
  begin
    // Min enthlt die Gre, welche die Schrift mindestens haben mu
    Min := 1;
    // Max enthlt die Gre, bei der die Schrift noch zu gro ist
    Max := -1;
    // Min und Max werden solange aneinander angeglichen, bis in Min
    // die gesuchte Schriftgre steht. Am Anfang ist Max noch -1, da
    // noch nicht bekannt ist bei welcher Gre die Schrift zu Hoch wre.
    // Solange noch keine Schriftgre probiert wurde, die zu gro ist,
    // bleibt Max auf -1 und die nchste Gre, die probiert wird, ist
    // doppelt so gro wie die alte.
    // Sobald Max einen Wert grer als -1 (und auch grer als Min) hat,
    // wird der Mittelwert von Min und Max fr die nchste Gre genommen,
    // die probiert wird.
    // Entsprechend, ob die gerade probierte Gre in den zur Verfgung
    // stehenen Platz pat oder nicht, wird Min oder Max der Wert der
    // gerade probierte Gre gegeben

    // Die erste zu probierende Gre festlegen
    if Font.Size = 0 then
      Font.Size := 16;

    while (Min < Max-1) or (Max = -1) do
    begin // Solange nicht die maximal mgliche Schriftgre gefunden wurde
      // Die Schriftgre ermitteln
      TmpWidth := TextWidth(Zeile);

      if TmpWidth < MaxWidth then
      begin // Wenn die Schrift nicht den in der Hhe zur Verfgung
            // stehenden Platz ausfllt
        //Merken, da der Font mindestens so gro sein mu, wie er jetzt ist
        Min := Font.Size;
        // Die nchste zu probierende Schriftgre festlegen
        if Max > -1 then
          NextSize := (Max + Min) div 2
        else
          NextSize := Min * 2;
      end
      else if TmpWidth > MaxWidth then
      begin // Wenn die Schrift den in der Hhe zur Verfgung
            // stehenden Platz berschreitet
        //Merken, da der Font nicht so gro sein darf, wie er jetzt ist
        Max := Font.Size;
        // Die nchste zu probierende Schriftgre festlegen
        NextSize := (Max + Min) div 2;
      end
      else
      begin // Wenn die Schrift den in der Hhe zur Verfgung
            // stehenden Platz genau ausfllt
        Min := Font.Size;
        Max := Min;
        NextSize := 1; // Diese Zeile unterdrckt nur eine Warnung
      end;

      if (Min < Max-1) or (Max = -1) then
      begin // Wenn die ideale Schriftgre noch nicht gefunden wurde
        // Dann wird mit der nchsten weitergemacht
        Font.Size := NextSize;
      end;
    end;

    Font.Size := Min;
  end;
end;

end.
