unit miscfuncXML;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung :  Die Unit miscfuncXML enthlt kleine Hilfsfunktionen,
                  welche die Arbeit mit dem XML-Parser XDOM von
                  Dieter Khler ("http://www.philo.de/homepage.htm")
                  erleichtern sollen. Die Funktionen funktionieren mit
                  XDOM 2.3.37, sollten aber auch mit hheren Versionen
                  funktionieren
// erstellt am  : ???
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    11.1.2003: An die Version 2.3.37 von XDOM angepasst
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses XDOM_2_3,
  miscfunc;

var
  //Eine stndig zur Verfgung stehende Instanz von TDomImplementation. Auf xdi darf nicht schreibend zugegriffen werden (xdi := blabla; xdi.Free; das ist verboten)
  xdi: TDomImplementation;
  //Eine stndig zur Verfgung stehende Instanz von TXmlToDomParser. Auf xdp pdarf nicht schreibend zugegriffen werden (xdp := blabla; xdp.Free; das ist verboten)
  xdp: TXmlToDomParser;
  //Eine stndig zur Verfgung stehende Instanz von TDomToXmlParser. Auf dxp pdarf nicht schreibend zugegriffen werden (xdp := blabla; xdp.Free; das ist verboten)
  dxp: TDomToXmlParser;

// Erzeugt aus einem wohlgeformten XML-String ein neues TdomDocument. Das Dokument sollte mit FreeXmlDocument wieder freigegeben werden
function CreateXmlDocument(xml: string): TdomDocument;
// Erzeugt neues TdomDocument, das nur den Basisknoten enthlt. Das Dokument sollte mit FreeXmlDocument wieder freigegeben werden
function CreateXmlDocumentWithTag(tag: string): TdomDocument;
// Gibt den Speicher eines TdomDocument wieder frei
procedure FreeXmlDocument(doc: TdomDocument);

// Verpasst dem Tag "Base" ein neues Untertag (Rckgabe) mit dem Namen "NewTagName", das in seinem Body den Text "Text" enthlt
function AddXmlTextNode(Base: TdomElement; NewTagName: string; Text: string): TdomElement;
// GetXmlNodeText ist das Gegenstck zu AddXmlTextNode
function GetXmlNodeText(Base: TdomElement; TextTagName: string): string; overload;
function GetXmlNodeText(Elem: TdomElement): string; overload;
// FirstChildTag gibt den ersten unmittelbaren UnterTag von "base" zurck, der den Namen "ChildTagName" hat
function FirstChildTag(base: TdomElement; ChildTagName: string): TdomElement;
// AddChildTag erzeugt in "base" einen UnterTag, der dem xml-String entspricht
function AddChildTag(base: TdomElement; xml: string): TdomElement;
// ChildXml gibt den Body von "base" als xml-String zurck. Es wird also der umgebende Tag von base entfernt
// Beispiel: base.Code = '<baseTagName x="2"><Untertag>jjkl</Untertag></baseTagName>'; Rckgabewert: '<Untertag>jjkl</Untertag>'
function ChildXml(base: TdomElement): string;

implementation

function CreateXmlDocument(xml: string): TdomDocument;
begin
  Result := xdp.stringToDom(xml);
end;

function CreateXmlDocumentWithTag(tag: string): TdomDocument;
begin
  Result := xdi.createDocument(tag, nil);
end;

procedure FreeXmlDocument(doc: TdomDocument);
begin
  doc.domImplementation.FreeDocument(doc);
end;

function ChildXml(base: TdomElement): string;
begin
  dxp.writeToString(base,  'Latin1', Result);
  Result := DeleteXMLTag(Result, base.NodeName);
end;

function AddXmlTextNode(Base: TdomElement; NewTagName: string; Text: string): TdomElement;
var
  TextElem: TdomText;
begin
  Result := Base.OwnerDocument.CreateElement(NewTagName);
  TextElem := Base.OwnerDocument.CreateTextNode(Text);
  Result.AppendChild(TextElem);
  Base.AppendChild(Result);
end;

function GetXmlNodeText(Base: TdomElement; TextTagName: string): string;
var
  Elem: TdomElement;
begin
  Result := '';
  Elem := FirstChildTag(Base, TextTagName);
  if Elem <> nil then
    Result := GetXmlNodeText(Elem);
end;

function GetXmlNodeText(Elem: TdomElement): string;
begin
  Result := '';
  if Elem.HasChildNodes then
  begin
    if Elem.ChildNodes.Item(0) is TdomText then
      Result := TdomText(Elem.ChildNodes.Item(0)).Data;
  end;
end;


function AddChildTag(base: TdomElement; xml: string): TdomElement;
var
  xdp: TXmlToDomParser;
  doc: TdomDocument;
begin
  xdp := TXmlToDomParser.Create(nil);
  try
    xdp.domImpl := Base.OwnerDocument.domImplementation;
    doc := xdp.stringToDom(xml);
    try
      Result := doc.documentElement;

      //Base.OwnerDocument.adoptNode(Result);
      Base.AppendChild(Result);
    finally
      FreeXmlDocument(doc);
    end;
  finally
    xdp.Free;
  end;
end;

function FirstChildTag(base: TdomElement; ChildTagName: string): TdomElement;
var
  i: integer;
  Node: TdomNode;
begin
  Result := nil;
  if base <> nil then
  begin
    i := 0;
    Result := nil;
    while (Result = nil) and (i < base.ChildNodes.Length) do
    begin
      Node := base.ChildNodes.Item(i);
      if (Node is TdomElement) and (TdomElement(Node).TagName = ChildTagName) then
        Result := TdomElement(Node);

      inc(i);
    end;
  end;
end;

initialization
  xdp := TXmlToDomParser.Create(nil);
  xdi := TDomImplementation.Create(nil);
  xdp.domImpl := xdi;
  dxp := TDomToXmlParser.Create(nil);
  dxp.domImpl := xdi;


finalization
	dxp.Free;
  xdp.Free;
  xdi.Free;


end.
