unit RakListBox;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit RakListBox stellt Klassen fr die
                  die Kapselung eines TListBox-Objektes zur Verfgung.
                  Bei Listboxen hat man oft das Problem, dass der angezeigte
                  Text einer Zeile kein eindeutiger Schlssel fr diese Zeile ist.
                  Man kann zwar an jede Zeile eine Referenz eines Objektes
                  anhngen (TListBox.Items.AddObject), aber die Listbox wird
                  nicht Eigentmer des Objektes und gibt es somit auch nicht
                  wieder frei.
                  Hier werden Klassen zur Verfgung gestellt, die eine TListbox
                  kapseln, fr jede Zeile ein Objekt fr weitere Daten zur
                  Verfgung stellen und sich auch um die Freigabe des
                  Objektes kmmert.
                  Ausserdem knnen einzelne Zeilen der Listbox unsichtbar gemacht
                  werden, ohne dass die Verbindung zu den Daten verloren geht.
// erstellt am  : ???
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses Classes, StdCtrls, RakCollection, Windows, Messages, SysUtils;

type
  TRakListBox = class;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakListItem
  // Beschreibung : TRakListItem kapselt eine Zeile einer TListbox
                    Klassen, die von TRakListItem abgeleitet sind,
                    knnen Daten zur weiteren Beschreibung der Zeile hinzufgen
                    z.B. Klasse TRakIdListItem
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakListItem = class(TRakCollectionItem)
  private
    function GetListBox: TListBox;
    function GetStrings: TStrings;
  protected
    FListId: integer;
    FRakListBox: TRakListBox;
    FText: string;
    FVisible: boolean;

    procedure SetCollection(Value: TRakCollection); override;
    procedure SetIndex(Value: integer); override;

    function GetSelected: boolean; virtual;
    procedure SetSelected(Value: boolean); virtual;
    function GetText: string; virtual;
    procedure SetText(Value: string); virtual;
    procedure SetVisible(Value: boolean); virtual;

    property Strings: TStrings read GetStrings;
  public
    constructor Create; override;

    procedure Assign(Source: TRakCollectionItem); override;

    // Liefert die nchste Zeile zurck, die auch wirklich angezeigt wird
    function GetNextVisible: TRakListItem;
    // Liefert die vorherige Zeile zurck, die auch wirklich angezeigt wird
    function GetPrevVisible: TRakListItem;

    // Die Listbox, in der die gekapselte Zeile angezeigt wird
    property ListBox: TListBox read GetListBox;
    // Legt fest, ob die Zeile selektiert ist
    property Selected: boolean read GetSelected write SetSelected;
    // Der angezeigte Text in der Listbox
    property Text: string read GetText write SetText;
    // Legt fest, ob die Zeile wirklich in Listbox angezeigt wird
    property Visible: boolean read FVisible write SetVisible;
  end;



  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakListBox
  // Beschreibung : TRakListBox kapselt eine TListBox
                    Wenn man TRakListBox verwendet, dann darf man nicht mehr
                    auf die Methoden von TListBox.Items zugreifen, da z.B.
                    ein Aufruf von TListBox.Items.Add dazu fhren wrde,
                    dass die Listbox eine Zeile enthlt, von der
                    TRakListBox nichts wei.
                    Methoden von TListbox werden ersetzt durch TRakListBox-Methoden:
                      Eine neue Zeile hinzufgen:
                        herkmmlich:
                          TListbox.Items.Add('EinText')
                        mit TRakListBox:
                          TRakListBox.Add.Text := 'EinText';
                      Eine neue Zeile mit einem neuem Objekt hinzufgen:
                        herkmmlich:
                          EinObject := TEinObject.Create;
                          TListbox.Items.AddObject('EinText', EinObject)
                        mit TRakListBox:
                          EinObject := TRakListBox.Add;
                          EinObject.Text := 'EinText';
                      Eine Zeile mit seinem Objekt lschen:
                        herkmmlich:
                          TObject(TListbox.Items.Objects[EinIndex]).Free;
                          TListbox.Items.Delete(EinIndex);
                        mit TRakListBox:
                          TRakListBox[EinIndex].Free;
                      Die ganze Liste leeren
                        herkmmlich:
                          for i := 0 to TListBox.Items.Count - 1 do
                            TObject(TListBox.Items.Objects[i]).Free;
                          TListBox.Items.Clear;
                        mit TRakListBox:
                          TRakListBox.FreeAll;
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakListBox = class(TRakCollection)
  private
    FAlwaysSelected: boolean;
    FListBox: TListBox;
    FStrings: TStrings;

    procedure SetAlwaysSelected(Value: boolean);
    procedure SetListBox(Value: TListBox);
    function GetListBoxItemIndex: integer;
    procedure SetListBoxItemIndex(Value: integer);
    function GetItems(Index: Integer): TRakListItem;
    function GetSelected: TRakListItem;
    procedure SetSelected(Value: TRakListItem);
    function GetSelections(Idx: Integer): TRakListItem;
    function GetSelectionsCount: integer;
    function GetText(Index: Integer): string;
    procedure SetText(Index: Integer; Value: string);
  protected
    function BasicAdd(TheClass: TRakCollectionItemClass): TRakListItem;
    procedure DoAssign(Source: TRakCollection); override;

    property ListBoxItemIndex: integer read GetListBoxItemIndex
      write SetListBoxItemIndex;
  public
    constructor Create; override;

    // Eine neue Zeile am Ende der Liste hinzufgen
    function Add: TRakListItem;
    // Ein Objekt anhand seines angezeigten Textes finden. Sollte es mehrere Zeilen mit den gleichen Text geben, dann wird der erste genommen
    function ItemByText(Text: string): TRakListItem;
    // Das Objekt finden, dessen Text mit einem bestimmten String anfngt; Gro/Kleinschreibung wird ignoriert
    function ItemByTextpart(Text: string): TRakListItem;

    procedure Sort; override;

    // Legt fest, ob immer eine Zeile selektiert sein muss
    property AlwaysSelected: boolean read FAlwaysSelected
      write SetAlwaysSelected;
    // Die Listbox, die gekapselt wird
    property ListBox: TListBox read FListBox write SetListBox;
    // Die Objekte, welche die einzelnen Zeilen kapseln (0 <= Index < Count)
    property Items[Index: Integer]: TRakListItem read GetItems; default;
    // Gibt das Objekt fr die selektierte Zeile zurck
    property Selected: TRakListItem read GetSelected write SetSelected;
    // Die Objekte fr die selektierten Zeilen (Property TListBox.MultiSelect = true) 0<= Idx < SelectionsCount)
    property Selections[Idx: Integer]: TRakListItem read GetSelections;
    // Anzahl der selektierten Zeilen
    property SelectionsCount: integer read GetSelectionsCount;
    // Die Texte der Zeilen, auch von den Zeilen, die unsichtbar sind
    property Text[Index: Integer]: string read GetText write SetText;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakIdListItem
  // Beschreibung : TRakIdListItem kapselt eine Zeile einer TListbox
                    und speichert zu der Zeile einen Integer-Wert
                    TRakIdListItem wird von TRakIdListBox verwaltet
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakIdListItem = class(TRakListItem)
  public
    Id: integer;

    procedure Assign(Source: TRakCollectionItem); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakIdListBox
  // Beschreibung : TRakIdListBox kapselt eine TListBox
                    Zu jeder Zeile der Listbox kann zustzlich ein
                    Integer-Wert gespeichert werden
                    weitere Info siehe TRakListBox
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakIdListBox = class(TRakListBox)
  private
    function GetItems(Index: Integer): TRakIdListItem;

    function GetSelected: TRakIdListItem;
    procedure SetSelected(Value: TRakIdListItem);
  public
    function Add: TRakIdListItem;
    function ItemById(Id: integer): TRakIdListItem;
    function ItemByText(Text: string): TRakIdListItem;

    property Items[Index: Integer]: TRakIdListItem read GetItems; default;

    property Selected: TRakIdListItem read GetSelected write SetSelected;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakStrIdListItem
  // Beschreibung : TRakStrIdListItem kapselt eine Zeile einer TListbox
                    und speichert zu der Zeile einen String-Wert
                    TRakStrIdListItem wird von TRakStrIdListBox verwaltet
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakStrIdListItem = class(TRakListItem)
  public
    Id: string;

    procedure Assign(Source: TRakCollectionItem); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRak2IdListBox
  // Beschreibung : TRak2IdListBox kapselt eine TListBox
                    Zu jeder Zeile der Listbox kann zustzlich ein
                    String-Wert gespeichert werden
                    weitere Info siehe TRakListBox
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakStrIdListBox = class(TRakListBox)
  private
    function GetItems(Index: Integer): TRakStrIdListItem;
  public
    function Add: TRakStrIdListItem;
    function ItemById(Id: string): TRakStrIdListItem;
    function ItemByText(Text: string): TRakStrIdListItem;

    property Items[Index: Integer]: TRakStrIdListItem read GetItems; default;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRak2IdListItem
  // Beschreibung : TRak2IdListItem kapselt eine Zeile einer TListbox
                    und speichert zu der Zeile zwei Integer-Werte
                    TRak2IdListItem wird von TRak2IdListBox verwaltet
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRak2IdListItem = class(TRakListItem)
  public
    Id1: integer;
    Id2: integer;

    procedure Assign(Source: TRakCollectionItem); override;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRak2IdListBox
  // Beschreibung : TRak2IdListBox kapselt eine TListBox
                    Zu jeder Zeile der Listbox knnen zustzlich zwei
                    Integer-Werte gespeichert werden
                    weitere Info siehe TRakListBox
  // erstellt am  :
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRak2IdListBox = class(TRakListBox)
  private
    function GetItems(Index: Integer): TRak2IdListItem;

    function GetSelected: TRak2IdListItem;
    procedure SetSelected(Value: TRak2IdListItem);
  public
    function Add: TRak2IdListItem;
    function ItemById(Id1, Id2: integer): TRak2IdListItem;
    function ItemByText(Text: string): TRak2IdListItem;

    property Items[Index: Integer]: TRak2IdListItem read GetItems; default;

    property Selected: TRak2IdListItem read GetSelected write SetSelected;
  end;

implementation

constructor TRakListItem.Create;
begin
  inherited Create;

  FVisible := true;
end;

procedure TRakListItem.Assign(Source: TRakCollectionItem);
var
  Src: TRakListItem;
begin
  inherited Assign(Source);

  Src := TRakListItem(Source);
  Text := Src.Text;
  Visible := Src.Visible;
end;

function TRakListItem.GetNextVisible: TRakListItem;
var
  i: integer;
  Zeile: TRakListItem;
begin
  Result := nil;
  if Collection <> nil then
  begin
    i := Index + 1;
    while (Result = nil) and (i < Collection.Count) do
    begin
      Zeile := TRakListItem(Collection[i]);
      if Zeile.Visible then
        Result := Zeile;
      inc(i);
    end;
  end;
end;

function TRakListItem.GetPrevVisible: TRakListItem;
var
  i: integer;
  Zeile: TRakListItem;
begin
  Result := nil;
  if Collection <> nil then
  begin
    i := Index - 1;
    while (Result = nil) and (i >= 0) do
    begin
      Zeile := TRakListItem(Collection[i]);
      if Zeile.Visible then
        Result := Zeile;
      dec(i);
    end;
  end;
end;

procedure TRakListItem.SetCollection(Value: TRakCollection);
var
  WasSelected: boolean;
begin
  WasSelected := Selected;

  if Visible then
  begin
    Visible := false;
    FVisible := true;
  end;

  if Value = nil then
  begin
    if (Collection <> nil) and TRakListBox(Collection).AlwaysSelected
      and WasSelected then
    begin
      if Collection.Count > 1 then
      begin
        if Collection.Count - 1 = Index then
          TRakListBox(Collection)[Index - 1].Selected := true
        else
          TRakListBox(Collection)[Index + 1].Selected := true;
      end;
    end;

    FRakListBox := nil;

    inherited SetCollection(Value);
  end
  else if Value is TRakListBox then
  begin
    FRakListBox := Value as TRakListBox;

    inherited SetCollection(Value);

    if Visible then
    begin
      FVisible := false;
      Visible := true;
    end;
  end
end;

procedure TRakListItem.SetIndex(Value: integer);
var
  OldIndex: integer;
  Zeile: TRakListItem;
  OldListId, SelId: integer;
begin
  OldIndex := Index;

  inherited SetIndex(Value);

  if (FRakListBox <> nil) and Visible then
  begin
    SelId := FRakListbox.ListBoxItemIndex;
    OldListId := FListId;
    if Index < OldIndex then
    begin
      Zeile := GetNextVisible;
      if Zeile <> nil then
      begin
        if Zeile.Index <= OldIndex then
        begin
          Strings.Move(FListId, Zeile.FListId);
          OldIndex := FListId;
          FListId := Zeile.FListId;
          while (Zeile <> nil) and (Zeile.FListId < OldIndex) do
          begin
            Zeile.FListId := Zeile.FListId + 1;
            Zeile := Zeile.GetNextVisible;
          end;
        end;
      end;
    end
    else if Index > OldIndex then
    begin
      Zeile := GetPrevVisible;
      if Zeile <> nil then
      begin
        if Zeile.Index >= OldIndex then
        begin
          Strings.Move(FListId, Zeile.FListId);
          OldIndex := FListId;
          FListId := Zeile.FListId;
          while (Zeile <> nil) and (Zeile.FListId > OldIndex) do
          begin
            Zeile.FListId := Zeile.FListId - 1;
            Zeile := Zeile.GetPrevVisible;
          end;
        end;
      end;
    end;

    if FRakListBox.AlwaysSelected then
    begin
      if OldListId = SelId then
        FRakListbox.ListBoxItemIndex := FListId
      else if FListId < OldListId then
      begin
        if (SelId > OldListId) or (SelId < FListId) then
          FRakListbox.ListBoxItemIndex := OldListId
        else
          FRakListbox.ListBoxItemIndex := OldListId + 1;
      end
      else if FListId > OldListId then
      begin
        if (SelId < OldListId) or (SelId > FListId) then
          FRakListbox.ListBoxItemIndex := OldListId
        else
          FRakListbox.ListBoxItemIndex := OldListId - 1;
      end;
    end;
  end;
end;

function TRakListItem.GetSelected: boolean;
begin
  if (Strings = nil) or not Visible then
    Result := false
  else
  begin
    if Listbox.MultiSelect then
      Result := ListBox.Selected[FListId]
    else
      Result := FRakListbox.ListBoxItemIndex = FListId;
  end;
end;

procedure TRakListItem.SetSelected(Value: boolean);
begin
  if Value <> Selected then
  begin
    if (Strings <> nil)  then
    begin
      if Value then
      begin
         Visible := true;
        if Listbox.MultiSelect then
          ListBox.Selected[FListId] := true
        else
          FRakListbox.ListBoxItemIndex := FListId;
      end
      else
      begin
        if Listbox.MultiSelect then
          ListBox.Selected[FListId] := false
        else
          FRakListbox.ListBoxItemIndex := -1;
      end;
    end;
  end;
end;

procedure TRakListItem.SetVisible(Value: boolean);
var
  Zeile: TRakListItem;
begin
  if Value <> FVisible then
  begin
    if Strings <> nil then
    begin
      if Value then
      begin
        Zeile := GetNextVisible;
        if Zeile = nil then
          Zeile := GetPrevVisible;
        if Zeile = nil then
          FListId := Strings.Add(FText)
        else
        begin
          if Index < Zeile.Index then
            FListId := Zeile.FListId
          else
            FListId := Zeile.FListId + 1;
          
          Strings.Insert(FListId, FText);
          Zeile := GetNextVisible;
          while Zeile <> nil do
          begin
            Zeile.FListId := Zeile.FListId + 1;
            Zeile := Zeile.GetNextVisible;
          end;
        end;
      end
      else
      begin
        Selected := false;
        Strings.Delete(FListId);
        Zeile := GetNextVisible;
        while Zeile <> nil do
        begin
          Zeile.FListId := Zeile.FListId - 1;
          Zeile := Zeile.GetNextVisible;
        end;
      end;
    end;
    FVisible := Value;
  end;
end;

function TRakListItem.GetListBox: TListBox;
begin
  if FRakListBox = nil then
    Result := nil
  else
    Result := FRakListBox.ListBox;
end;

function TRakListItem.GetStrings: TStrings;
begin
  if FRakListBox <> nil then
  begin
    Result := FRakListBox.FStrings;
  end
  else
  begin
    Result := nil;
  end;
end;

function TRakListItem.GetText: string;
begin
   Result := FText;
end;

procedure TRakListItem.SetText(Value: string);
var
  MerkSelected: integer;
begin
  FText := Value;
  if Visible and (Strings <> nil) then
  begin
    MerkSelected := FRakListbox.ListBoxItemIndex;
    Strings[FListId] := Value;
    FRakListbox.ListBoxItemIndex := MerkSelected;
  end;
end;


constructor TRakListBox.Create;
begin
  inherited Create;

  FAlwaysSelected := false;
end;

function TRakListBox.BasicAdd(TheClass: TRakCollectionItemClass): TRakListItem;
begin
  Result := TRakListItem(inherited BasicAdd(TheClass));
  Result.Text := 'NewItem';

  if AlwaysSelected and (Selected = nil) then
    Result.Selected := true;
end;

procedure TRakListBox.SetAlwaysSelected(Value: boolean);
begin
  FAlwaysSelected  := Value;

  if AlwaysSelected and (Selected = nil) and (Count > 0) then
    Items[0].Selected := true;
end;

function TRakListBox.Add: TRakListItem;
begin
  Result := BasicAdd(TRakListItem);
end;

procedure TRakListBox.DoAssign(Source: TRakCollection);
var
  i: integer;
  Item: TRakCollectionItem;
begin
  FreeAll;
  for i := 0 to Source.Count - 1 do
  begin
    Item := BasicAdd(TRakCollectionItemClass(Source.Items[i].ClassType));
    Item.Assign(Source.Items[i]);
  end;
end;

function TRakListBox.ItemByText(Text: string): TRakListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if Items[i].Text = Text then
      Result := Items[i];
    inc(i);
  end;
end;

procedure TRakListBox.Sort;
var
  WasSelected: TRakListItem;
  ListId, i: integer;
begin
  WasSelected := Selected;

  inherited Sort;

  ListId := 0;
  for i := 0 to Count - 1 do
  begin
    if Items[i].Visible then
    begin
      Items[i].FListId := ListId;
      inc(ListId);
    end;
    Items[i].Text := Items[i].FText;
  end;

  if WasSelected <> nil then
  begin
    for i := 0 to Count -1 do
    begin
      if Items[i] = WasSelected then
      begin
        WasSelected.Selected := true;
        break;
      end;
    end;
  end;
end;

procedure TRakListBox.SetListBox(Value: TListBox);
begin
   FListBox := Value;
  if FListBox <> nil then
    FStrings := FListBox.Items
  else
    FStrings := nil;
end;

function TRakListBox.GetItems(Index: Integer): TRakListItem;
begin
  Result := TRakListItem(inherited Items[Index]);
end;

function TRakListBox.GetSelected: TRakListItem;
var
  i: integer;
begin
  if ListBoxItemIndex = -1 then
    Result := nil
  else
  begin
    Result := nil;
    i := 0;
    while (Result = nil) and (i < Count) do
    begin
      if Items[i].Visible and (Items[i].FListId = ListBoxItemIndex) then
        Result := Items[i];
      inc(i);
    end;
  end;
end;

procedure TRakListBox.SetSelected(Value: TRakListItem);
begin
  if Value = nil then
    ListBoxItemIndex := -1
  else
    Value.Selected := true;
end;

function TRakListBox.GetSelections(Idx: Integer): TRakListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if Items[i].Selected then
    begin
      dec(Idx);
      if Idx = -1 then
        Result := Items[i];
    end;
    inc(i);
  end;
end;

function TRakListBox.GetSelectionsCount: integer;
var
  i: integer;
begin
  Result := 0;
  for i := 0 to Count - 1 do
  begin
    if Items[i].Selected then
      inc(Result);
  end;
end;

function TRakListBox.GetText(Index: Integer): string;
begin
  Result := Items[Index].Text;
end;

procedure TRakListBox.SetText(Index: Integer; Value: string);
begin
  Items[Index].Text := Value;
end;



procedure TRakIdListItem.Assign(Source: TRakCollectionItem);
begin
  inherited Assign(Source);

  Id := (Source as TRakIdListItem).Id;
end;

function TRakIdListBox.Add: TRakIdListItem;
begin
  Result := TRakIdListItem(BasicAdd(TRakIdListItem));
end;

function TRakIdListBox.ItemById(Id: integer): TRakIdListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if Items[i].ID = Id then
      Result := Items[i];
    inc(i);
  end;
end;

function TRakIdListBox.ItemByText(Text: string): TRakIdListItem;
begin
  Result := TRakIdListItem(inherited ItemByText(Text));
end;

function TRakIdListBox.GetSelected: TRakIdListItem;
begin
  Result := TRakIdListItem(inherited Selected);
end;

procedure TRakIdListBox.SetSelected(Value: TRakIdListItem);
begin
  inherited Selected := Value;
end;

function TRakIdListBox.GetItems(Index: Integer): TRakIdListItem;
begin
  Result := TRakIdListItem(inherited Items[Index]);
end;



procedure TRakStrIdListItem.Assign(Source: TRakCollectionItem);
begin
  inherited Assign(Source);

  Id := (Source as TRakStrIdListItem).Id;
end;

function TRakStrIdListBox.Add: TRakStrIdListItem;
begin
  Result := TRakStrIdListItem(BasicAdd(TRakStrIdListItem));
end;

function TRakStrIdListBox.ItemById(Id: string): TRakStrIdListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if Items[i].ID = Id then
      Result := Items[i];
    inc(i);
  end;
end;

function TRakStrIdListBox.ItemByText(Text: string): TRakStrIdListItem;
begin
  Result := TRakStrIdListItem(inherited ItemByText(Text));
end;

function TRakStrIdListBox.GetItems(Index: Integer): TRakStrIdListItem;
begin
  Result := TRakStrIdListItem(inherited Items[Index]);
end;



procedure TRak2IdListItem.Assign(Source: TRakCollectionItem);
begin
  inherited Assign(Source);

  Id1 := (Source as TRak2IdListItem).Id1;
  Id2 := (Source as TRak2IdListItem).Id2;
end;

function TRak2IdListBox.Add: TRak2IdListItem;
begin
  Result := TRak2IdListItem(BasicAdd(TRak2IdListItem));
end;

function TRak2IdListBox.ItemById(Id1, Id2: integer): TRak2IdListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if (Items[i].ID1 = Id1) and (Items[i].ID2 = Id2) then
      Result := Items[i];
    inc(i);
  end;
end;

function TRak2IdListBox.ItemByText(Text: string): TRak2IdListItem;
begin
  Result := TRak2IdListItem(inherited ItemByText(Text));
end;

function TRak2IdListBox.GetSelected: TRak2IdListItem;
begin
  Result := TRak2IdListItem(inherited Selected);
end;

procedure TRak2IdListBox.SetSelected(Value: TRak2IdListItem);
begin
  inherited Selected := Value;
end;

function TRak2IdListBox.GetItems(Index: Integer): TRak2IdListItem;
begin
  Result := TRak2IdListItem(inherited Items[Index]);
end;



function TRakListBox.GetListBoxItemIndex: integer;
begin
{  if not MultiSelect then
    Result := SendMessage(Handle, LB_GETCARETINDEX, 0, 0) else
    Result := SendMessage(Handle, LB_GETCURSEL, 0, 0);}
  if Listbox = nil then
    Result := -1
  else
    Result := SendMessage(Listbox.Handle, LB_GETCURSEL, 0, 0);
end;

procedure TRakListBox.SetListBoxItemIndex(Value: integer);
begin
  if Listbox <> nil then
  begin
    if Listbox.MultiSelect then
      SendMessage(Listbox.Handle, LB_SETCARETINDEX, Value, 0)
    else
      SendMessage(Listbox.Handle, LB_SETCURSEL, Value, 0);
  end;
end;

function TRakListBox.ItemByTextpart(Text: string): TRakListItem;
var
  i: integer;
begin
  Result := nil;
  i := 0;
  while (Result = nil) and (i < Count) do
  begin
    if CompareText(Copy(Items[i].Text, 1, length(Text)), Text) = 0 then
      Result := Items[i];
    inc(i);
  end;
end;

end.
